#include "core.h"
/* validate the operation array
call error for each error message
returns true if validation success
*/
extern rccl_data rc; //common data
extern short operation_array[];

void Calc::Valid_err(const string err) {//error detected during the validation
	erreur(getOp_linenumber(),getOp_linenumberM(),err);
}

bool Calc::Valid_len(int n) {//return true when error  n  = number of  variables (output + inputs)
	if(getOp_len() != n) {
		erreur(getOp_linenumber(),getOp_linenumberM(),"Number of variables is not correct");
		printf("------------------------------\n");
		return true;
	}
	return false;
}
bool Calc::Valid_typ(int n, VarNum t) { // check the type (NUM/LOGICAL of the variable n  
	if(getOp_len() < n-1) {
		erreur(getOp_linenumber(),getOp_linenumberM(),"Number of variables is not correct");
		return true;
	}
	short x = operation_array[iop+2+n];
	//printf("Valid_typ x =%d  n=%d\n",x,n);
	if(x < TOVAR) { // this argument i is a constant...numeric
		if(t!=NUM) {
			erreur(getOp_linenumber(),getOp_linenumberM(),"Variable type NUM/LOGICAL  is not correct");
			return true;
		}
		return false;
	}
	Var v = getVar(x-TOVAR);
	if(v.getTypeOrig()==PARAM_PVAR)
		return false;	// !not filled by the DTRIM
	if(v.typeNum!=t) {
		erreur(getOp_linenumber(),getOp_linenumberM(),"Variable type NUM/LOGICAL  is not correct");
		return true;
	}
	return false;
}
bool Calc::Valid_orig(int n, VarOrig t) {
	if(getOp_len() < n-1) {
		erreur(getOp_linenumber(),getOp_linenumberM(),"Number of variables is not correct");
		return true;
	}
	short x = operation_array[iop+2+n];
	if(x < TOVAR) {
		erreur(getOp_linenumber(),getOp_linenumberM(),"constant where a variable is needed");
		return true ; // this argument i is a constant...numeric
	}
	Var v = getVar(x-TOVAR);
	if(v.getTypeOrig()!=t) {
		erreur(getOp_linenumber(),getOp_linenumberM(),"constant where a variable is needed");
		return true;
	}
	return false;
}




void Calc::Validate() {
	setOperationBegin();//position = before first operation
	int skip=0; //how many IF levels to skip
	while(1) {
		setOperationNext();//process one operation at iop
		//prtOperation("?");
		//--- parameters of this operation -------
		char opcode= getOp_code(); //current operation code
		int  oplen = getOp_len(); //number of variables
		if(opcode==OP_EOF) {
			if(skip !=0)
				Valid_err(" IF / ENDIF not in balance"); //error
			break;
		}
		//---counting the IF / ENDIF
		if(opcode ==OP_ENDIF) skip--;
		if(opcode ==OP_IF) skip++;
		if(opcode ==OP_IFNOT) skip++;
		// validation
		if(opcode ==OP_IF || opcode ==OP_IFNOT) {
			Valid_len(1);
			Valid_typ(0,LOGICAL) ;
		}
		if(opcode ==OP_ENDIF)	Valid_len(0);
		if(opcode ==OP_SLOW) {
			Valid_len(1);
			Valid_typ(0,NUM);
			//test 0-20
		}
		//PHASE INIT
		if(opcode ==OP_UPD) {  // UPD param: value
			Valid_len(2);
			Valid_typ(0,NUM);
			Valid_orig(0,PARAM_PVAR);
			Valid_typ(1,NUM);
		}
		if(opcode == OP_DTRIM1 ||opcode == OP_DTRIM2 ||opcode == OP_DTRIM3 ||opcode == OP_DTRIM4) {
			Valid_orig(0,PARAM_PVAR);
			continue;
		}
		if(opcode ==OP_EQU) {
			Valid_len(2);
			Valid_typ(0,NUM);
			continue;
		}
		if(opcode ==OP_ADD || opcode ==OP_SUB ) {
			for(int i=1; i<oplen; ++i)
				Valid_typ(i,NUM);
			continue;
		}

		if(opcode ==OP_MULT  || opcode ==OP_MULT_PC) {
			Valid_len(3);
			Valid_typ(1,NUM);
			Valid_typ(2,NUM);
		}

		if(opcode ==OP_RAL) {  //exponential slow
			Valid_len(3);
			Valid_typ(1,NUM);
			Valid_typ(2,NUM);
		}
		if(opcode ==OP_DELTA) {
			Valid_len(3);
			Valid_typ(1,NUM);
			continue;
		}
		// ---- mixers --------------------------------------
		if(opcode==OP_LINE  || opcode==OP_CURVE) {
			if(oplen < 6 || (oplen %2 >0))
				erreur(getOp_linenumber(),getOp_linenumberM(),"not enough points in this mixer");
			for(int i=2; i<oplen-1; i=i+1)
				Valid_typ(i,NUM);
		}
		if(opcode==OP_SERV) {
			Valid_len(5);
			Valid_typ(1,NUM);
			Valid_typ(2,NUM);
			Valid_typ(3,NUM);
			Valid_typ(4,NUM);
			continue;
		}
		if(opcode==OP_EXPO) {
			Valid_len(3);
			Valid_typ(1,NUM);
			Valid_typ(2,NUM);
			continue;
		}
		if(opcode==OP_OUTN) {
			Valid_len(4);
			Valid_typ(1,NUM);
			Valid_typ(2,NUM);
			Valid_typ(3,NUM);
		}
		if(opcode==OP_OUT) {
			Valid_len(5);
			Valid_typ(1,NUM);
			Valid_typ(2,NUM);
			Valid_typ(3,NUM);
		}
		if(opcode==OP_ADDS || opcode==OP_SUBS) {
			Valid_len(3);
			Valid_typ(1,NUM);
			Valid_typ(2,NUM);
			continue;
		}

		if(opcode==OP_ADDAS || opcode==OP_SUBAS) {
			Valid_len(4);
			Valid_typ(1,NUM);
			Valid_typ(2,NUM);
			Valid_typ(3,NUM);
			continue;
		}
		if(opcode ==OP_AND || opcode ==OP_OR) {
			for(int i=1; i<oplen; ++i)
				Valid_typ(i,LOGICAL);
			continue;
		}
		if(opcode==OP_NOT) {
			Valid_len(2);
			Valid_typ(1,LOGICAL);
			continue;
		}
		if(opcode==OP_MEM) {
			Valid_len(2);
			Valid_typ(1,LOGICAL);
			continue;
		}
		if(opcode==OP_VARF ||opcode==OP_VART) {            //20140512
			Valid_len(3);
			Valid_typ(1,LOGICAL);
			continue;
		}
		if(opcode==OP_COMP) {
			Valid_len(3);
			Valid_typ(1,NUM);
			Valid_typ(2,NUM);
			continue;
		}
		if(opcode==OP_COMPH) {
			Valid_len(4);
			Valid_typ(1,NUM);
			Valid_typ(2,NUM);
			Valid_typ(3,NUM);
			continue;
		}
		if(opcode==OP_COMPA) {
			Valid_len(5);
			Valid_typ(1,NUM);
			Valid_typ(2,NUM);
			Valid_typ(3,NUM);
			continue;
		}
		if(opcode==OP_VAR) {
			Valid_len(4);
			Valid_typ(1,NUM);
			Valid_typ(2,NUM);
			continue;
		}
		if(opcode==OP_NUM) {
			Valid_len(2);
			Valid_typ(1,LOGICAL);
			continue;
		}
		if(opcode==OP_NUMT) {
			Valid_len(3+1);
			Valid_typ(1,LOGICAL);
			Valid_typ(2,NUM);
			continue;
		}
		if(opcode ==OP_FALSE || opcode ==OP_TRUE) {
			Valid_len(1);
			continue;
		}
		if(opcode==OP_CHRPLUS || opcode==OP_CHRMIN) {
			Valid_len(2);
			Valid_typ(1,NUM);
			continue;
		}

		if(opcode==OP_BREAK) {    //only for a coldstart
			Valid_len(1);
			continue;
		}
		if(opcode==OP_SHOWT) {
			Valid_len(2);
			Valid_typ(0,NUM);
			continue;
		}
		if(opcode==OP_SHOW) {
			Valid_len(3);
			Valid_typ(2,NUM);
			continue;
		}
		if(opcode==OP_PLAYT) {
			Valid_len(1);
			continue;
		}
		if(opcode==OP_PLAY) {
			Valid_len(2);
			Valid_typ(1,NUM);
			continue;
		}
		if(opcode==OP_PLAYV) {
			Valid_len(2);
			Valid_typ(1,NUM);
			continue;
		}
		if(opcode==OP_PLAYS) {
			Valid_len(1);
			Valid_typ(0,NUM);
			continue;
		}
		if(opcode==OP_BEEP) {
			Valid_len(1);
			Valid_typ(0,NUM);
			continue;
		}
		// JETI JETIR
		if(opcode==OP_PID) {
			Valid_len(6+4);
			Valid_typ(1,NUM);
			Valid_typ(2,NUM);
			Valid_typ(3,NUM);
			Valid_typ(4,NUM);
			Valid_typ(5,NUM);
			continue;
		}
		if(opcode==OP_PIDA) {
			Valid_len(5+5);
			Valid_typ(1,NUM);
			Valid_typ(2,NUM);
			Valid_typ(3,NUM);
			Valid_typ(4,NUM);
			continue;
		}


	}//end main loop

}