#include "core.h"
/* transmitter computational functions
    load a model file to internal data structures
 */
#ifdef DEBUGP
extern Serial pc;
#endif
extern rccl_data rc; //common data
#ifdef RAM_EXT
#ifdef COMPILER_MBED
short operation_array[OPLENGTH] __attribute__((section("AHBSRAM1")));
#endif
//check the linker.ld script for the name 
#ifdef COMPILER_CS
short operation_array[OPLENGTH] __attribute__((section(".usb_ram")));
//short operation_array[OPLENGTH] __attribute__((section(".ram1")));
#endif
#ifdef COMPILER_ARM  //AHBSRAM1 is the same as  ETH_RAM
short operation_array[OPLENGTH] __attribute__((section("AHBSRAM1")));
#endif
#else
short operation_array[OPLENGTH];
#endif

bool convert(string const& s, short& z, short divider  ) { //convert string s to internal variable short x , return true if ok
	int signePos = 1, idiv=1;
	bool point = false;
	int value = 0;
	string str=s;
	string::iterator it;
	char x;
	for (it=str.begin() ; it < str.end(); it++) {
		x=*it;
		if (x=='+')
			continue;
		if (x=='-') {
			signePos= -1;
			continue;
		}
		if (point)
			idiv *=10;
		if (!point && (x=='.' || x==',')) {
			point=true;
			continue;
		}
		if (x>='0' && x<='9') {
			value = value*10 + (x-'0');
			continue;
		}
		return false;
	}
	value= value * signePos *TOINT;
	while (divider >0){
		divider --;
		value = value /10;
	}
	z = value / idiv;
	return true;
}

short valcheck(short x) {// x = internal variable
	short MAX= 125*TOINT;
	if (x < -MAX)
		return -MAX;
	if (x >MAX)
		return MAX;
	return x;
}
Var::Var(string x, VarOrig t, VarNum u) {
	name=x;
	typeOrig=t;
	typeNum=u;
	if (typeNum==LOGICAL)
		value=false;
	else
	  value=0;
}
VarOrig Var::getTypeOrig() {
	return typeOrig;
}
void Var::putTypeOrig(VarOrig t) {
	typeOrig=t;
}

string Var::valueS(int f) { // conversion to a string  format f=0,1,2
	float x;
	char buffer[10];
	for (int i=0; i<9; i++)
		buffer[i]=' ';
	buffer[9]='\0';
	if (typeNum ==NUM) {
		if (typeOrig==PARAM_PVAR) {

		}
		// numerical formats with 0,1,2 decimals
		x=value;
		if (value == NOT_AVAILABLE )
			return buffer;
		x=x/TOINT;
		if (f==2)
			sprintf(buffer,"%7.2f",x);
		else if (f==1)
			sprintf(buffer,"%6.1f",x);
		else if (f==0)
			sprintf(buffer,"%5.0f",x);
		//chrono format
		else if (f==9) { //chrono
			return lcdTimer(value); //-> calcrun
		}
		return buffer;
	} else {
		if (value ==0)
			return "FALSE";
		else
			return "TRUE";
	}
}

Operation::Operation() {
	code = OP_EOF;
	linenumber=0;
	linenumberM=0;
	vars = vector<short>();
}
Operation::Operation(int n,int l, int lm, char c) {//size vector, linenumer, linenumber/module, op code
	code = c;
	linenumber=l;
	linenumberM=lm;
	vars = vector<short>();
	vars.reserve(n);
}

void Operation::debugPrint() {
#ifdef DEBUGP
	pc.printf("Op  %c (%3i):",code,vars.size());
	vector<short>::iterator ity;
	for (ity=vars.begin(); ity<vars.end(); ity++) {
		short v=*ity;
		if (v>=TOVAR)
			pc.printf(" v%3i",v-TOVAR);
		else
			pc.printf(" %3i",v);
	}
	pc.printf("\n");
#endif
}
//--------------------------------------------------------------------------------
void replace(string& z, string oldvalue, string newvalue) {
	int y = z.find_first_of('#');
	if (y>=0)
		z= z.substr(0,y);     // remove comments after the #
	while (1) {
		int pos=z.find(oldvalue);
		if (pos<0)
			break;
		z.replace(pos,oldvalue.size(),newvalue);
	}
}
//---------------------constructor----------------------------------------------
Calc::Calc(string f) {// constructor: loads the model file (f= filename without extension)
	modelname =f; 
#ifdef DEBUGP
	pc.printf("Constructor Calc  :%s\n",f.c_str());
#endif
/*#ifdef JETI
		if (rc.jetiptr!=NULL)
			rc.jetiptr->explorer_load();// load before procesing the mdx file and the JETIX operations
#endif*/
	ready_to_calc=false;
	iop=0;
	currentLog=-1;
	currentLogTime=0;
	char c1=OP_EOF;
	char c2=0;
	phasename="";
	dtrimArray[0]=Dtrim();
	dtrimArray[1]=Dtrim();
	dtrimArray[2]=Dtrim();
	dtrimArray[3]=Dtrim();
	for (int i=0 ; i<OPLENGTH; i++) //initialize op_code at OP_EOF
		operation_array[i]=(c1<<8)|c2;;
	iopend=0;
	ready_to_calc=true;
	hwinit(); //create the hardware variables
		
	expandfile(f);// expands the "macros" CALL & CALLP, the result is a call to Calc::insert
	Validate();	
	for (int i=0 ; i<8; i++) {
		lcd[i]=" ";
		lcd_protect[i]=0;
	}
	if( !param_restore()){	// important error : no param file
		ready_to_calc=false;
	}
	rc.varSelection=new set<short>;

#ifdef DEBUGP
	printf ("Calcload end %s Available memory  : %d last iop=%d\n",f.c_str(), FreeMem(),iop);
#endif
	freeRam = FreeMem();
	currentBreak= -1;
	kick(); //init the timer for automatic power-off
#ifdef WATCHDOG
	Watchdog::kick(WATCHDOG_TIMER); //watchdog  reload after ... seconds
#endif
	
}

Calc::~Calc(){
	#ifdef WATCHDOG
	Watchdog::kick(WATCHDOG_STOP); //disable the watchdog
#endif
	printf("Calc destructor + save param\n");
	param_save();
	ppm_stop();
}

bool Calc::expandfile(string f) {//read file f , expands each CALL/CALLP
	Filey mfile;
	if (!mfile.fopenread(f+".mdx")) {
		erreur(0,0,"Error open file"+f+".mdx");
		return false;
	}
	string zline;
	int linenumber=-1;
	while (mfile.fgetline(zline)) {
	  // no automatic initial red of RCCL.MDM  
		linenumber++;
		//printf ("Calc expandline %d =%s \n",linenumber ,zline.c_str());
		vector<string> x =cleanline(zline);
		if (x.size()==0) continue;
		if (x.at(0)=="CALL" || x.at(0)=="CALLP") {
			//printf("***CALL %s\n",x[1].c_str());
			vector<string> call_params;
			if (x.size()>2) {
				for (int i=2; i<(int)x.size(); i++) {
					call_params.push_back(x[i]);
					//printf("push %s\n",x[i].c_str());
				}
			}
			vector<string> inits;

			if (x.at(0)=="CALLP") {  //beware the INIT
				string z;
				while (mfile.fgetline(z)) {
					linenumber++;
					//printf("-- %s\n",z.c_str());
					vector<string> y= cleanline(z);
					if (y.size()>0 && y.at(0)=="END") break;
					if (y.size()>1 && y.at(1)=="INIT") {
						inits.push_back(z);
						insert(y,z,linenumber,-1);
						continue;
					}
					if (y.size()==0)
						continue;
					erreur(linenumber,0,"CALLP not followed with INITS +END ");
					break;
				}
			}
			// adapt the module : change the parameters and remove some INITS
			Filey mfile;
			if (!mfile.fopenread(x[1]+".mdm")) { //module =.mdm
				string z ="Module not found:";
				z = z+x[1].c_str();
				erreur(linenumber,0,z);
				return false;
			}
			vector<string> def_params;
			//get the DEF parameters (first non empty line)
			int modline=-1;
			string modz;
			while (mfile.fgetline(modz)) {
				modline++;
				vector<string> x = cleanline(modz);
				if (x.size()==0) continue;
				if (x.at(0)=="DEF") {
					if (x.size()>1) {
						for (int i=1; i<(int)x.size(); i++) {
							def_params.push_back(x[i]);
							//printf("push DEF  %s\n",x[i].c_str());
						}
					}
					if (def_params.size() != call_params.size()) {
						erreur(linenumber,modline,"CALL nb parameters != DEF ");
						break; //??? return
					}
					continue;//only one line with DEF
				}
				for (int i=0; i< (int)def_params.size(); i++)
					replace(modz, def_params[i],call_params[i]);  //replace the parameters within the module
				x = cleanline(modz);
				//printf("expand module after replace : %s\n",modz.c_str());
				if (x.size()==0) continue;
				if (x.at(0)=="DEF") continue;
				if (x.size()==1|| x.at(1)!="INIT") {
					insert(x,modz,linenumber,modline);
					//printf("expand : inserted z=%s\n",z.c_str());
					continue;
				} else {
					bool lineToInsert = true;
					for (int i =0; i< (int)inits.size()&& lineToInsert ; i++) {
						vector<string> y =cleanline(inits.at(i));
						if (y.at(0)==x.at(0)) lineToInsert=false;
					}
					if (lineToInsert) {
						insert(x,modz,linenumber,modline);
						//printf("expand : inserted init=%s\n",z.c_str());
					}
				}
			}// end getline module
			mfile.ftoclose();
		}//end if CALL or CALLP
		if (x[0]=="CALL" || x[0]=="CALLP")
			continue;
		insert(x,zline,linenumber,-1);  // call to Calc::insert
		//printf("expand : inserted zline=%s\n",zline.c_str());
	}//endwhile main model file
	mfile.ftoclose();
	return true;
}//end expand
void Calc::erreur(int linenumber,int linemodule, string message) {
	//pc.printf ("********Error calc at %d/%d : %s\n",linenumber,linemodule, message.c_str());// debug version is a simple printf
	prtOperation("Err=");
	ready_to_calc=false;
	char buffer[200];
	sprintf(buffer,"%2d/%2d: %s",linenumber,linemodule, message.c_str());
	errors.push_back(buffer);
}

short Calc::addVar(string& name, int lineNumber,int lineNumberM,VarNum typeNum,VarOrig typeOrig) {
	short x = getVarIndex(name) ;
	if (x <0 ||(typeOrig==PARAM_VVAR && calc_vars.at(x).getTypeOrig()==PARAM_PVAR)) {
		//not yet existing->add this variable ,accept a pvar==vvar
		Var v = Var(name,typeOrig,typeNum);
		calc_vars.push_back(v);
		//printf(" addVar %s at %i \n",name.c_str(),  calc_vars.size()-1);
		return calc_vars.size()-1;
	} else {//exists
		Var * v = &calc_vars[x];
		if (typeOrig==WORK_MEM &&  v->getTypeOrig()==WORK)
			v->putTypeOrig(WORK_MEM);
		if (typeOrig != v->getTypeOrig())
			erreur(lineNumber,lineNumberM,v->name+" change typeOrig"); 
		if (typeNum != v->typeNum)
			erreur(lineNumber,lineNumberM,v->name+" change typeNum");
		return x;
	}
}
int Calc::getnVar() {// return the size (number of variables)
	return  calc_vars.size();
}

short Calc::getVarIndex(string& name) {//find a var with this name , return index or -1
	for (int i=0; i< (int)calc_vars.size(); i++) {
		if ((calc_vars.at(i).name == name) && (calc_vars.at(i).getTypeOrig() !=PARAM_VVAR))
			return i;
	}
	return -1;
}
short Calc::getVarIndexP(string& name) {//find a vvar with this name , return index or -1
	for (int i=0; i< (int)calc_vars.size(); i++) {
		if ((calc_vars.at(i).name == name) && (calc_vars.at(i).getTypeOrig() ==PARAM_VVAR))
			return i;
	}
	return -1;
}
int Calc::getVarP( string& name) {//find a parameter (vvar)  with this name , return value or value unknown
	short varindex;
	varindex=getVarIndexP(name);
			if (varindex <0) 
				return NOT_AVAILABLE;
			return calc_vars.at(varindex).value ;
}

Var   Calc::getVar(short i) {// get the variable indexed i
	return calc_vars.at(i) ;
}
Var *   Calc::getVarptr(short i) {// get the variable indexed i
	if (i<0 || i>getnVar()-1)
		return NULL;
	return &calc_vars.at(i) ;
}
void  Calc::setVarN(string name, int value) {
	short index = getVarIndex(name);
	if (index >=0) {
		Var * v  = getVarptr(index);
		v->value = value;
	}
}
void  Calc::setVarL(string name, bool value) {
	short index = getVarIndex(name);
	if (index >=0) {
		Var * v  = getVarptr(index);
		if (value)
			v->value = true;
		else
			v->value = false;
	}
}
void Calc::varDump(int select) { // select=0 only Var  select=1 full
#ifdef DEBUGP
	vector<Var>::iterator it;
	int i=0;
	// printf("Statistics  Vars=%i  Op=%i\n",calc_vars.size(),calc_operations.size());
	pc.printf("-------------------- var dump -----------------------------------\n");
	for (it=calc_vars.begin(); it<calc_vars.end(); it++) {
		Var v = *it;
		pc.printf("Var index=%3i type=%i  org=%i  val=%i  %s\n",i,v.typeNum,v.getTypeOrig(),v.value,v.name.c_str());
		i++;
	}
	if (select==0) return;
	pc.printf("-------------------- string dump -----------------------------------\n");
	i=0;
	vector<string>::iterator its;
	for (its=calc_lib.begin(); its<calc_lib.end(); its++) {
		string v = *its;
		pc.printf("Lib index=%i  %s\n",i,v.c_str());
		i++;
	}
	pc.printf("-------------------- operation dump --------------------------------\n");
	setOperationBegin();
	pc.printf("Op nr  code size  iopend=%i \n",iopend);
	while (1) {
		setOperationNext();
		if (getOp_code()==OP_EOF)
			break;
		prtOperation(" ");      //print the current operation
	}
	pc.printf("-------------------- end         -----------------------------------\n");
#endif
}

void Calc::insert(vector<string>& line, string& lineorig,int linenumber, int linenumbermod) {//insert one line of the expanded file into the calc data
	if (line.size()==0) return; //no token (comment line)
	 //printf ("Calc insert %d/%d >>>%s\n",linenumber,linenumbermod,lineorig.c_str());

	// identify the operation code
	code_operation c;
	if (line.size()==1 || get_code_operation(c, line[1])==false) {
		if (get_code_operation(c, line[0])==false) {
			erreur(linenumber,linenumbermod,"Opcode unknown:"+lineorig);
			return;
		}
	}
/*	if (c.codeop==OP_TRAVEL) { //floating point values not compatible
		return;
	}*/
	// create the operation with the known number of variables( line.size() + number of internal variables depending on the codeop)
	Operation currentOperation(line.size()+c.intvar,linenumber,linenumbermod,c.codeop)    ;
	//token_printf(line);  //debug
	// exceptions processing
	if (c.codeop==OP_PHASE) {
		/* inverted processing
		- first variable is the logical condition
		   (if missing = always true -> replace with  the constant 1)
		- next variables are logical variables :the phase name and all the components
		    example name=  vol.aaa.bbbb
		    -> the variables are vol.aaa.bbbb,vol.aaa,vol
		*/
		//varDump(0);
		short conditionvar;
		if (line.size()<2 || line.size()>3) {
			erreur(linenumber,linenumbermod,"Phase incorrect arguments"+lineorig);
			return;
		}
		if (line.size()==2) { //no condition
			currentOperation.vars.push_back(1);
		} else {
			conditionvar = getVarIndex(line.at(2));
			if (conditionvar <0){
				erreur(linenumber,linenumbermod,"Phase variable logique inexistante:+"+line.at(2)+" "+lineorig);
				return;
			}
			currentOperation.vars.push_back(conditionvar+TOVAR);
		}
		string p = line.at(0);//phase name
		while (1) {
			short phasevar = addVar(p, linenumber,linenumbermod,LOGICAL,WORK);
			currentOperation.vars.push_back(phasevar+TOVAR);
			int i = p.find_last_of('.');
			if (i>0) {
				p= p.substr(0,i);//cut the last part
				continue;
			}
			break;
		}
		putOperation(currentOperation); //insert the pointer to this operation
		return;
	}
	if (line.at(0)=="MODUL") { // MODUL 9 22.5
		if (line.size() <3) {
			erreur(linenumber,linenumbermod,"MODUL missing arguments ");
			return;
		}
		int ch = atoi(line.at(1).c_str());
		if (ch <6 || ch >16) {
			erreur(linenumber,linenumbermod,"MODUL number of channels is not correct(6-16) ");
			return;
		}
		double  chw = atof(line.at(2).c_str());
		if (chw <16.0 || chw >22.50) {
			erreur(linenumber,linenumbermod,"MODUL pulse length is not correct (16 - 22.5) ");
			return;
		}
		ppm_init(ch ,chw*1000);
		return;
	}
	// format check
	if (c.codeop==OP_INIT) {//check the format  (a : after the  name) normal processing of outvar follows
		if (line[0].substr(line[0].size()-1,1)!=":"){
			erreur(linenumber,linenumbermod,"parameter name without :");
			return;
		}
	}
	if (c.codeop==OP_JETI ) {
		if (line.size() <2) {
		erreur(linenumber,linenumbermod,"Two arguments : node and variable (or zero) ");
		return;	
		}
	}
	if (c.codeop==OP_JETIX ) {
		if (rc.jetiptr!=NULL){
			if (line.size()!=3){
		    erreur(linenumber,linenumbermod,"node num +  string ");
		    return;	
			}
			string x = line[2];
			if (x.size()!=18){
		    erreur(linenumber,linenumbermod," string len =16 ");
		    return;	
			}
			int node= atoi(line.at(1).c_str());
			if (rc.jetiptr->explorer_exp( node ,line[2]) == false)// change the descriptor of a expander line
				erreur(linenumber,linenumbermod," error processing line ");
		return;	
		}		
	}
	// normal processing
	int current_token =0; //token to process from the line
	if (c.outvar) { // creates the outvar
		VarOrig v = WORK;
		if (c.codeop==OP_INIT) v=PARAM_PVAR;
		/*modifier OUT variable !!!!!!!!!!!!!!!!!!!!!!!!!!!!!
		if (c.codeop==OP_OUT  || c.codeop==OP_OUTN) v=OUT;  // a OUT variable
		*/
		if (line.at(0).substr(0,2)=="@V") v=OUT;  // a OUT variable is identified with @V
		if (c.statevar==true) v=WORK_MEM;
		short outvar = addVar(line[0], linenumber,linenumbermod,c.typenum,v);
		currentOperation.vars.push_back(outvar+TOVAR);//register the outvar in the vector of this current operation
		current_token++;
	}
	current_token++;
	// process the invars
	string current_var;
	for (int i=current_token; i< (int)line.size(); i++) {//process each token
		short value;
		current_var=line.at(i);
		if (current_var.substr(0,1)=="\"") {//followed with a  string variable
			calc_lib.push_back(current_var);
			currentOperation.vars.push_back(calc_lib.size()-1);//register the index
			continue;
		}
		//printf("insert process token %i = %s\n",i,line.at(i).c_str());
		if (convert(current_var,value)) {   //test if numeric
			//printf ("isnumeric %s = %i\n", current_var.c_str(),value);
			if (c.codeop==OP_INIT && i==2) {//this is the value of a vvar name = line[0] + line[3]
				string name = line.at(0); //base name = pvar
				if (line.size()==4)
					name += line.at(3);
				if (line.size()>4) {
					erreur(linenumber,linenumbermod,"To many parameters for a INIT:");
					return;
				}
				short invar = addVar(name, linenumber,linenumbermod,NUM,PARAM_VVAR);  //add this variable as vvar (may have the same name as the pvar)
				calc_vars.at(invar).value=value;
				currentOperation.vars.push_back(invar+TOVAR);
				continue;
			}
			//other  constant registered in the operation
			currentOperation.vars.push_back(value);
			continue;
		}//end if numeric
		short x = getVarIndex(current_var);//if variable exists, register
		if (x>-1) {
			currentOperation.vars.push_back(x+TOVAR);
			continue;
		}
		erreur(linenumber,linenumbermod,current_var+ " non existing input variable");
		return;
	}
	// add the internal variables initialized to zero
	for (int i=0; i< c.intvar; i++)
		currentOperation.vars.push_back(0);
	putOperation(currentOperation); //insert the pointer to this operation
}

bool Calc::putOperation(Operation opvar) {
	/* Put one operation in the non-standart sequential storage short[] op
	        internal format
	  short 0 byte 0 = opcode       byte 1 = number of variables
	  short 1 byte 1 = linenumber   byte 2 = linenumber from the module
	  short [n] = variables
	  ----------usage: model load
	*/
	unsigned char c1,c2;
	c1=opvar.code;
	c2=opvar.vars.size();

	operation_array[iop]   = (c1<<8)|c2;
	//printf ( "calcload putop iop=%i  %c %i %X",iop,c1,c2,operation_array[iop]);
	c1= opvar.linenumber;
	c2= opvar.linenumberM;
	operation_array[iop+1]= (c1<<8)|c2;
	//printf ( "  %i %i %X",c1,c2,operation_array[iop+1]);
	for (int i=0; i< (int)opvar.vars.size(); i++) {
		operation_array[iop+i+2] = opvar.vars.at(i);
		//printf(" %i", operation_array[iop+2+i]);
	}
	//printf("\n");
	if (iop + 2 +  opvar.vars.size()> OPLENGTH) {
#ifdef DEBUGP
		printf("*****big error: overflow op array\n");
#endif
		return false;
	}
	iop = iop + 2 +  opvar.vars.size();
	iopend=iop;
	return true;
}
/* iterator  on the operations
    this is not a standard iterator
*/
void Calc::setOperationBegin() {//set the begin position for a iterator
	iop=-1;
}
void Calc::setOperationNext() {
	if (iop<0) {
		iop=0;
		//printf("iop = 0\n");
		return;
	}
	short x = operation_array[iop];
	unsigned char len = x& 0xFF;
	iop = iop+ len +2; // may be a EOF operation code
	//printf("iop = %d\n",iop);
}
int  Calc::getOperationIndex() {// index of the current operation
	return iop;
}
//-------- other operation arguments
char Calc::getOp_code() {
	short x = operation_array[iop];
	char code= x>>8;
	return code;
}
int  Calc::getOp_linenumber() {
	return operation_array[iop+1]>>8;
}
int  Calc::getOp_linenumberM() {
	return operation_array[iop+1]& 0xFF;
}
int  Calc::getOp_len() { //returns the number of variables
	short x = operation_array[iop];
	unsigned char len = x& 0xFF;
	return len;
}
Var * Calc::getOp_Varptr(int i) { //get a pointer to the variable at index i in the operation
	short x = operation_array[iop+2+i];
	if (x < TOVAR)
		return NULL ; // this argument i is a constant
	return getVarptr(x-TOVAR);
}
int  Calc::getOp_valueN(int i) {//returns the value of the numeric variable at position i in this operation
	short x = operation_array[iop+2+i];
	if (x < TOVAR)
		return x ; // this argument i is a constant
	Var v = getVar(x-TOVAR);
	if (v.getTypeOrig()==PARAM_PVAR)
		v = getVar(v.value-TOVAR);
	return v.value;
}
bool  Calc::getOp_valueL(int i) {//returns the value of the logical variable at position i in this operation
	short x = operation_array[iop+2+i];
	if (x==0) return false;
	if (x==1) return true;
	if (getVar(x-TOVAR).value ==0)
		return false;
	else
		return true;
}
string Calc::getOp_valueS(int i) {//return the value of the string variable at position i in this operation
	short x = operation_array[iop+2+i];
	if(x>=0 &&((unsigned) x <= calc_lib.size()-1)) {
		string z= calc_lib [x];
	  return z.substr(1,z.length()-2);// without the " "
	}else
	return "error";
}
void Calc::setOp_valueN(int i,int value) {  // set the value of the variable or the constant indexed i (numerical )
	short x = operation_array[iop+2+i];
	if (x < TOVAR) { //this value is a constant, not the index of a variable
		operation_array[iop+2+i] = valcheck(value);
		return;
	}
	Var *v = getVarptr(x-TOVAR);
	v->value=value;
}

void Calc::setOp_valueL(int i,bool value) { // set the value of the variable indexed i (logical )
	short x = operation_array[iop+2+i];
	if (x < TOVAR) {
		if (value==true)
			operation_array[iop+2+i] = 1;
		else
			operation_array[iop+2+i] = 0;
		return;
	}
	Var *v = getVarptr(x-TOVAR);
	if (value)
		v->value=1;
	else
		v->value=0;
}
void  Calc::prtOperation(string z) {//print a debug information for the current operation (at iop)
#ifdef DEBUGP
	if (iop <0) return;
	short x = operation_array[iop];
	unsigned char len = x& 0xFF;
	pc.printf("%s iop=%i code=%x len=%i %i/%i;",z.c_str(),iop,  getOp_code(),len, getOp_linenumber(),getOp_linenumberM());
	for (int i =0; i<len ; i++) {
		short x = operation_array[iop+2+i];
		if (x < TOVAR) {// this argument i is a constant
			pc.printf(" %i;",x);
		} else {
			Var v =getVar(x-TOVAR);
			pc.printf("(%d) %s=%i;",x-TOVAR,   v.name.c_str(),v.value);
		}
	}
	pc.printf("\n");
#endif
}
bool Calc::param_restore() {
	// param_save -> menu_calc.cpp
	Filey fichier;
	string line;
	vector<string> tokens;
	string del =" ";
	short varindex, val;
	// restore  the global parameters
	if (!fichier.fopenread("rccl.mdp")){
		char buffer[200];
	    sprintf(buffer,"Missing global param file  rccl.mdp");
		errors.push_back(buffer);
		return false;
	}

	while (fichier.fgetline(line)) { //example 2 26 7000 ail.down:   ou 4 70.00 ail.down:
		tokens.clear();
		Tokenize(line, tokens, del);
		// for (int i=0;i< tokens.size();i++) printf("Token=%s\n",tokens[i].c_str());
		if (tokens[0]=="4" && tokens.size() ==3) { //parameters version 4  format>4 value name !value as  70.00  for a variable =70,00
			varindex=getVarIndexP(tokens.at(2));
			if (varindex <0)
				continue; //skip this obsolete param
			convert(tokens.at(1), val);
			calc_vars.at(varindex).value=val;
			//printf("param_restore i=%i name=%s value=%i\n",varindex,tokens.at(2).c_str(),val);
			continue;
		}
	
	}	
	fichier.ftoclose();
	
	
	//restore the parameters for this model
	if (!fichier.fopenread(modelname+".mdp")){
		char buffer[200];
	    sprintf(buffer,"Missing param file  for %s", rc.current_file.c_str());
		errors.push_back(buffer);
		return false;
	}

	while (fichier.fgetline(line)) { //example 2 26 7000 ail.down:   ou 4 70.00 ail.down:
		tokens.clear();
		Tokenize(line, tokens, del);
		// for (int i=0;i< tokens.size();i++) printf("Token=%s\n",tokens[i].c_str());
		if (tokens[0]=="4" && tokens.size() ==3) { //parameters version 4  format>4 value name !value as  70.00  for a variable =70,00
			varindex=getVarIndexP(tokens.at(2));
			if (varindex <0)
				continue; //skip this obsolete param
			convert(tokens.at(1), val);
			calc_vars.at(varindex).value=val;
			//printf("param_restore i=%i name=%s value=%i\n",varindex,tokens.at(2).c_str(),val);
			continue;
		}
		if (tokens[0]=="2" && tokens.size() ==4) { //parameters version 2  format>2 nn  value name  !value as integer 7000  for a variable =70,00
			// this occurs once, when a V2 param file is first processed in V4
			varindex=getVarIndexP(tokens.at(3));
			if (varindex <0)
				continue; //skip this obsolete param
			val = (short) atoi(tokens.at(2).c_str());
			//convert(tokens.at(2), val);
			calc_vars.at(varindex).value=val;
			//printf("param_restore V2 i=%i name=%s value=%s   %i\n",varindex,tokens.at(3).c_str(),tokens.at(2).c_str(),val);
			continue;
		}
#ifdef DEBUGP
		pc.printf("****error parameter restore line =%s\n",line.c_str());
#endif
	}
	fichier.ftoclose();
	return true;
}
void Calc::kick() { // there is activity -> change the time for a automatic cutoff
	autocutoff_ticker = GetTicker()  + INACTIVITY;
}
