#include "core.h"
#include <queue>
#include<stdlib.h>
/* transmitter computating cycle
     (one cycle / 10ms)
 */
extern rccl_data rc; //common data
extern short operation_array[];
string lcdTimer(int timer) { //timer units= 1/100 minute, returns a string formatted as
	// time < 1 hour :     format as  mm:ss
	//time > 1 hour  : format as  h.mm
	int n,sec,min,heure,x;
	char signe ;
	if (timer>0) {
		x = timer*60;
		signe=' ';
	} else {
		x = -timer*60;
		signe='-';
	}
	n = x/100; // n is a number of seconds
	sec = n % 60;
	n -= sec;
	n /=60;
	min = n % 60;
	n -= min;
	heure = n/60;
	char buffer[15];
	if (heure > 0)
		sprintf(buffer,"%c%1d.%02d",signe,heure,min);
	else
		sprintf(buffer,"%c%02d:%02d",signe,min,sec);
	return buffer;
}
void Calc::run() {
	//lcd messages lines lcd[0...7]
	//lines lcd[0] and lcd[1] display a status, using a small font
	// lines lcd[3...7]
	// line lcd[2] = default transmitter chrono
	// lines[3..7] default
	if ((rc.rccl_task_tick % 60)==0) { //each 1/100 minute updates the lcd messages
		//lcd [] are strings defined in Calc , 4D update occurs in menu_normal
		char buffer[32];  // maximum width 26 char
		sprintf(buffer,"%8s    %4.2f %3.0f %4.0f",rc.current_file.c_str(),rc.batt.vb1,rc.batt.ib1,rc.batt.cb1);
		lcd[0]=buffer;
		int freeiop =((OPLENGTH - iopend)*100)/OPLENGTH;
		sprintf(buffer,"%5d  %3d  %4.2f %3.0f %4.0f",freeRam,freeiop,rc.batt.vb2,rc.batt.ib2,rc.batt.cb2);
		//sprintf(buffer," %5d      %4.2f %3.0f %4.0f",freeRam,rc.batt.vb2,rc.batt.ib2,rc.batt.cb2);
		lcd[1]=buffer; // status
		if (GetTicker()> lcd_protect[2])
			lcd[2]= "Tx=" + lcdTimer(rc.rccl_task_tick /60);  // default chr
		for (int i=3; i<8; i++) {
			if (GetTicker()> lcd_protect[i])
				lcd[i]=string(" ");
		}
	}
	//reset the internal variables
	vector<Var>::iterator it;
	for (it=calc_vars.begin(); it<calc_vars.end(); it++) {
		if (it->getTypeOrig()==WORK) //reset all the work variables to zero
			it->value=0;
	}
	setOperationBegin();//position = before first operation
	int skip=0; //how many IF levels to skip
	int slow=0; //value of the SLOW opcode
	bool phasedetected=false;
#ifdef JETI
	rc.jetiptr->iop_busy_processed=false;
#endif
	if (slow_end>slow_done)
		slow_done++;
	if (slow_end==slow_done) {
		slow_done=0;
		slow_end=0;
	}

	while (1) {
		//prtOperation("?");
		//printf ("calcrun skip=%i\n",skip);
		setOperationNext();//process one operation at iop
		//--- parameters of this operation -------
		char opcode= getOp_code();
		int  oplen = getOp_len();
		if (opcode==OP_EOF) {
			setVarL("@H1",0); //reset the first calc indicator
			ppm_start();

#ifdef JETI
			if (rc.jetiptr->iop_busy ==-1) {
				rc.jetiptr->iop_last =-1; // begin a new "loop" of JETI instructions
				//printf("calcrun new loop  jeti\n");
			}

			if (rc.jetiptr->iop_busy_processed==false && rc.jetiptr->iop_busy !=-1) { //busy but  out of the current IF conditions (=dead)
				//printf("calcrun reset jeti\n");
				rc.jetiptr->iop_last =-1;
				rc.jetiptr->iop_busy=-1;
			}
#endif

			break;
		}
#ifdef JETI
		if (opcode==OP_JETI) {
			/* + 3 internal vars   1 asked to be active  2= last computed variable or unknown  3=validity in ticks
			invars
			X = node
			n = input variable, default 0
			d  = dividor ( default 1)
			L  = logical variable
			f  = format PLAYV
			JETI X            oplen = 5
			JETI X n         oplen =  6
			JETI X n d      oplen = 7
			JETI X n L f    oplen = 8
			JETI X n L f d oplen = 9
			internal variables
			    duration=validity in ticks  => oplen-1
					last computed variable or unknown => oplen -2

			process the two internal variables
			this is always processed because it is before the testing of IF /ENDIF
			*/
			short duration = getOp_valueN(oplen-1);
			short value;
			if (duration >0) {
				duration --;
				value= getOp_valueN(oplen-2);
				setOp_valueN(0,value); //the outvar is the last computed value during the "duration"
				setOp_valueN(oplen-1,duration);
			} else {
				value =  NOT_AVAILABLE ;//outvar is no more availabke after the duration
				setOp_valueN(0,value);
			}
		}
#endif
		if (skip>0) { //---------- IF / ENDIF ------------------------
			if (opcode ==OP_ENDIF) skip--;
			if (opcode ==OP_IF) skip++;
			if (opcode ==OP_IFNOT) skip++;
			continue;
		}
		if (opcode ==OP_IF && oplen==1 && (getOp_valueL(0)==false)) { //condition of the IF is false-> skip
			skip=1;
			continue;
		}
		if (opcode ==OP_IFNOT  && oplen==1 && (getOp_valueL(0)==true)) { //condition of the IFNOT is true -> skip
			skip=1;
			continue;
		}
		if (opcode ==OP_IF ||  opcode == OP_IFNOT ||opcode ==OP_ENDIF) { //condition of the IF is true
			continue;
		}
		//---------- PHASES ------------------------------
		if (opcode ==OP_SLOW) {
			slow =getOp_valueN(0)/10; //slow : unit is 100 msec in the source file, here in "tickers "(10msec)
			//????? inacheve et non teste
			//printf ("slow =%i phasename=%s\n",slow,phasename.c_str());
			continue;
		}
		if (opcode ==OP_PHASE) {
			if (phasedetected==false && getOp_valueL(0)) {  // logical variable or constant 1
				phasedetected=true;
				for (int i=1; i<oplen ; i++) {
					setOp_valueL(i,true);//put each phase name true
				}

				string z = getOp_Varptr(1)->name; //the detected phase name
				if (z==phasename)
					continue; //no phase change
				if (phasename=="") {//this is the first loop, not a phase change
					phasename=z;
					continue;
				}
				//this is a phase change
				phasename=z; //this is the new current phase name
				slow_done=0;
				slow_end=slow;
				//printf("PHASE change -> %s  slow=%i\n",phasename.c_str(),slow);
			}

		}//end processing phase
		if (opcode ==OP_INIT) {
			/*format outvar(N) value(N) (condition x (L))     the other are no more supported*/
			if (oplen==3 && getOp_valueL(2)==false)continue; // optionnal condition is false
			setOp_valueN(0,operation_array[iop+2+1]);     // set the value to the variable index
		}
		if (opcode ==OP_UPD) { // UPD param: value
			short value = getOp_valueN(1);//new value
			Var * pvar = getVarptr(operation_array[iop+2] -TOVAR);//PVAR
			if (pvar==NULL) continue;
			short vvar_index =  pvar->value -TOVAR;
			Var * vvar = getVarptr(vvar_index);//PVAR
			if (pvar==NULL) continue;
			vvar->value=value;
			#ifdef DEBUGP
			printf("Calc UPD value=%d  vvar=%s\n",value, vvar->name.c_str());
			#endif
			continue;
		}

		if (opcode == OP_DTRIM1 ||opcode == OP_DTRIM2 ||opcode == OP_DTRIM3 ||opcode == OP_DTRIM4) {
			int indexDtrim = 0 ;
			if (opcode == OP_DTRIM2)
				indexDtrim = 1;
			else if (opcode == OP_DTRIM3)
				indexDtrim = 2;
			else if (opcode == OP_DTRIM4)
				indexDtrim = 3;
			dtrimArray[indexDtrim].indexPvarNormalTrim=operation_array[iop+2]-TOVAR;
			continue;
		}
		if (opcode ==OP_EQU) {
			setOp_valueN(0,getOp_valueN(1));
			continue;
		}
		if (opcode ==OP_ADD) {
			/* format outvar(N) .1 to n invar(N)    */
			int sum = 0;
			for (int i=1; i<oplen; ++i)
				sum =valcheck(sum +getOp_valueN(i));
			setOp_valueN(0,sum);
			continue;
		}
		if (opcode ==OP_SUB) {
			/* format outvar(N) .1 to n invar(N)    */
			int sum = 0;
			for (int i=1; i<oplen ; ++i)
				sum =valcheck(sum -getOp_valueN(i));
			setOp_valueN(0,sum);
			continue;
		}
		if (opcode ==OP_MULT) { // vvv MULT x p   |variable| < 1
			float x,p,z;
			x=float(getOp_valueN(1))/TOINT;
			p=float(getOp_valueN(2))/TOINT;
			z = x*p;
			setOp_valueN(0, valcheck(z*TOINT));
			continue;
		}
		if (opcode ==OP_MULT_PC) { // multiply with variable in %
			float x,p,z;
			x=float(getOp_valueN(1))/TOINT;
			p=float(getOp_valueN(2))/(TOINT *100);
			z = x*p;
			setOp_valueN(0,valcheck(z*TOINT));
			continue;
		}
		if (opcode ==OP_RAL) { //exponential slow
			float alpha=float(getOp_valueN(2))/(TOINT *10);
			float v = alpha*float(getOp_valueN(1))+(1.0-alpha)*float(getOp_valueN(0));
			//printf ("RAL alpha=%6.4f v=%5.2f\n",alpha,v);
			setOp_valueN(0,v);
		}
		if (opcode ==OP_DELTA) {
			short oldval = getOp_valueN(2);
			short newval = getOp_valueN(1);
			if (oldval==NOT_AVAILABLE || newval==NOT_AVAILABLE ){
				setOp_valueN(0,NOT_AVAILABLE);
				setOp_valueN(2,newval);			
				continue;
			}
			setOp_valueN(0,newval-oldval);
			setOp_valueN(2,newval);
			continue;
		}
		if (opcode==OP_LINE ) {
			Curve mix;
			for (int i=2; i<oplen-1; i=i+2)
				mix.addPoint(getOp_valueN(i),getOp_valueN(i+1));//add a point x,y
			setOp_valueN(0, mix.get(getOp_valueN(1)));
			continue;
		}
		
		
		
		if (opcode==OP_CURVE) {
			Curve mix;
			for (int i=2; i<oplen-1; i=i+2)
				mix.addPoint(getOp_valueN(i),getOp_valueN(i+1));//add a point x,y
			int x =getOp_valueN(1);
			if (x< -60*TOINT || x> 60*TOINT) { //no rounding  for the extreme values
				setOp_valueN(0, mix.get(getOp_valueN(1)));
				//printf("CURVE no rounding \n");
			} else {
				int delta= TOINT*10; //round
				float y1 = mix.get(getOp_valueN(1)+delta);
				float y2 = mix.get(getOp_valueN(1)-delta);
				//printf("CURVE rounding y1=%5.2f y2=%5.2f\n",y1,y2);
				setOp_valueN(0,(y1+y2)/2);
			}
			continue;
		}
		if (opcode==OP_EXPO) {
			int expo=getOp_valueN(2);
			/* "exponential" is a lower sensitivity around x=0
			expo =0-> no exponential, the sensitivity is normal
			expo=30 means 30%  less sensitivity
			implemented here with a symetrical curve with 5 poinst
			2 extreme points  are the normal values (-100,-100) and (+100,+100)
			1 neutral point (0,0)
			2 points at a arbitrary x at +30% and -30%, the y value defines the sensitivity
			*/
			Curve mix;
			mix.addPoint(-100*TOINT,-100*TOINT);
			mix.addPoint(-30*TOINT, -30*TOINT + (expo/100)*30);
			mix.addPoint(0,0);
			mix.addPoint(30*TOINT,  30*TOINT - (expo/100)*30);
			mix.addPoint(100*TOINT, 100*TOINT);
			int x = getOp_valueN(1);
			if (x< -60*TOINT || x> 60*TOINT) //no rounding  for the extreme values
				setOp_valueN(0, mix.get(getOp_valueN(1)));
			else {
				int delta= TOINT*10; //round
				float y1 = mix.get(getOp_valueN(1)+delta);
				float y2 = mix.get(getOp_valueN(1)-delta);
				setOp_valueN(0,(y1+y2)/2);
			}
			continue;
		}
		if (opcode==OP_OUTN || opcode==OP_OUT) {
			float val;
			//typical values   outvar, invar, 155,(1550&#65533;s) , 55 (550&#65533;s)
			//internal value outvar in microsec
			float center = getOp_valueN(2)*10;
			float dev =    getOp_valueN(3)*10;
			float x   =    float(getOp_valueN(1))/(TOINT*100) ; //signal from -1.00 to +1.00
			val = center + x* dev; // output signal in microsec
			//printf("OUT val=%5.2f x=%5.2f center=%5.2f dev=%5.2f\n",val,x,center,dev);
			if (opcode==OP_OUTN || (slow_end==slow_done)) {    //no phase transition
				setOp_valueN(0,val);
				continue;
			}

			if (slow_done==0 && slow_end>0) {  // register the difference in the operation
				int diff = val-getOp_valueN(0) ;//@Vi new - @Vi before
				setOp_valueN(4,diff);         // operation_array[iop+2+4] = diff;
				//printf("OUT diff = %i\n",diff);
				// !!!! ceci est une constante:ne pas utiliser setOp_valueN
			}

			/*phase transition */
			float transition = float(getOp_valueN(4))* (float(slow_end)-float(slow_done))/float(slow_end);
			// printf("OUT tr = %5.2f\n",transition);
			setOp_valueN(0,val- transition);
			continue;
		}
		if (opcode==OP_SERV) {
			Curve mix;
			mix.addPoint(-100*TOINT,getOp_valueN(4));
			mix.addPoint(0,getOp_valueN(3));
			mix.addPoint(100*TOINT, getOp_valueN(2));
			setOp_valueN(0, mix.get(getOp_valueN(1)));

		}
		if (opcode==OP_ADDS) {
			float p  = float(getOp_valueN(2))/(TOINT*100);
			float x  = getOp_valueN(1);
			float vvv  = getOp_valueN(0);
			vvv = vvv + x*p;
			//printf("ADDS x=%5.2f  p=%5.2f  -> vvv=%5.2f\n",x,p,vvv);
			setOp_valueN(0,valcheck(vvv));
			continue;
		}
		if (opcode==OP_SUBS) {
			float p  = float(getOp_valueN(2))/(TOINT*100);
			float x  = getOp_valueN(1);
			float vvv  = getOp_valueN(0);
			vvv = vvv - x*p;
			setOp_valueN(0,valcheck(vvv));
			continue;
		}
		if (opcode==OP_ADDAS) {
			float p1  = float(getOp_valueN(2))/(TOINT*100);
			float p2  = float(getOp_valueN(3))/(TOINT*100);
			float x  = getOp_valueN(1);
			float vvv  = getOp_valueN(0);
			if (x>0)
				vvv = vvv + x*p2;
			else
				vvv = vvv + x*p1;
			setOp_valueN(0,valcheck(vvv));
			continue;
		}
		if (opcode==OP_SUBAS) {
			float p1  = float(getOp_valueN(2))/(TOINT*100);
			float p2  = float(getOp_valueN(3))/(TOINT*100);
			float x  = getOp_valueN(1);
			float vvv  = getOp_valueN(0);
			if (x>0)
				vvv = vvv - x*p2;
			else
				vvv = vvv - x*p1;
			setOp_valueN(0,valcheck(vvv));
			continue;
		}
		if (opcode==OP_AND) {
			bool out = true;
			for (int i=1; i<oplen; ++i) {
				//printf ( "valueL i=%i -> %i\n",i,getOp_valueL(i));
				if (getOp_valueL(i)==false) {
					out = false;
					break;
				}
			}
			setOp_valueL(0,out);
			continue;
		}
		if (opcode==OP_OR) {
			bool out = false;
			for (int i=1; i<oplen; ++i) {
				//printf ( "valueL i=%i -> %i\n",i,getOp_valueL(i));
				if (getOp_valueL(i)==true) {
					out = true;
					break;
				}
			}
			setOp_valueL(0,out);
			continue;
		}
		if (opcode==OP_NOT) {
			setOp_valueL(0,!getOp_valueL(1));
			continue;
		}
		if (opcode==OP_MEM) {
			if (getOp_valueL(1))
				setOp_valueL(0,true);
			continue;
		}
		if (opcode==OP_HOLD) {
			for (int i=0; i<oplen; ++i){
				Var * v = getOp_Varptr( i) ;
				if ( v->getTypeOrig()==WORK){
					v->putTypeOrig(WORK_MEM);
					//printf ("HOLD change to WORK_MEM  %s\n",v->name.c_str());
				}
			}
			continue;
		}
		if (opcode==OP_VART || opcode==OP_VARF) { 	//20140512
			int former = getOp_valueN(2);  // var 2 = internal variable  0=not yet initialized  1=false 2=true
			setOp_valueL(0,false);
			// test the changing value
			if( former == 1 && opcode==OP_VART && getOp_valueL(1))
				setOp_valueL(0,true);
			if( former == 2 && opcode==OP_VARF && !getOp_valueL(1)){
				setOp_valueL(0,true);
				//Var * v = getOp_Varptr( 1) ;
				//printf ("VARF true for %s\n", v->name.c_str());
			}
			//memorize the former value
			if (getOp_valueL(1))
				setOp_valueN(2,2); //set former =2
			else
				setOp_valueN(2,1);  //former=1;
			continue;
		}
		if (opcode==OP_COMP) {
			//printf("COMP  %d > %d \n",getOp_valueN(1),getOp_valueN(2)  );
			if (getOp_valueN(1)> getOp_valueN(2))
				setOp_valueL(0,true);
			else
				setOp_valueL(0,false);
			continue;
		}
		if (opcode==OP_COMPA) {
      if((rc.rccl_task_tick % 60)!=0) { //same as @HS false
				setOp_valueL(0,false); //return false when @HS tick is false
				continue;
			}
			short period   = getOp_valueN(3); //signal every "period"  (unit  x,xx minutes)
			short down= getOp_valueN(4); // down counter (0 the first time ?) same units
			if (down <0 || down >period){ //initialization is not correct
				down=0;
				setOp_valueN(4,0); 
			}
			if (down >0 ) {  
				down--;
				setOp_valueL(0,false);
				setOp_valueN(4,down);
				continue;
			}
			// ___counter down is zero_____
			if (getOp_valueN(1)<= getOp_valueN(2)){//the condition is false with down==0
				setOp_valueL(0,false);
				continue;
			}
			if (getOp_valueN(1)> getOp_valueN(2)){//the condition is true 
				setOp_valueL(0,true); //signal as true
				setOp_valueN(4,period); 
				continue;
			}
		}
		if (opcode==OP_COMPH) {
			if (getOp_valueN(0)&& (getOp_valueN(1)< (getOp_valueN(2)-getOp_valueN(3))))
				setOp_valueL(0,false);
			if (!getOp_valueN(0)&& (getOp_valueN(1)> (getOp_valueN(2)+getOp_valueN(3))))
				setOp_valueL(0,true);
			continue;
		}
		if (opcode==OP_VAR) {
			short value= getOp_valueN(1);
			short delta= getOp_valueN(2);
			short valueSignal= getOp_valueN(3); // last signaled value (or 0 the first time ?)
			if (value < valueSignal-delta  || value > valueSignal+delta) {
				setOp_valueL(0,true);
				setOp_valueN(3,value);
				//printf("Calc VAR value anc = %d delta = %d value=%d\n",valueSignal,delta,value);
			} else {
				setOp_valueL(0,false);
			}
			continue;
		}
		if (opcode==OP_NUM) {
			if (getOp_valueL(1))
				setOp_valueN(0, 100*TOINT);
			else
				setOp_valueN(0, -100*TOINT);
			continue;
		}
		if (opcode==OP_NUMT) {
			/* 0 = output value
			   1 = input value (logical variable)
			   2 = duration (original unit=second, after load and *TOINT....it is 10msec = one loop
			   3 = internal variable - 0 at startup 1 after the first loop (without delay)

			*/
			if (getOp_valueL(3)==0) { //initialization without delay
				//printf("NUMT init\n");
				setOp_valueL(3,1);
				if (getOp_valueL(1))
					setOp_valueN(0, 100*TOINT);
				else
					setOp_valueN(0, -100*TOINT);
				continue;
			}
			//nothing to: the outvar has the correct value
			if (getOp_valueL(1)&& getOp_valueN(0)==  100*TOINT)continue;   //nothing to do
			if (!getOp_valueL(1)&& getOp_valueN(0)== -100*TOINT)continue;  //nothing to do
			//movement during one loop
			int step = (200*TOINT) /getOp_valueN(2) ;//change /loop
			//printf("NUMT 1 v=%i step=%i\n",getOp_valueN(0),step);
			//last step
			if (getOp_valueL(1)&& getOp_valueN(0)>=  100*TOINT -step) {
				setOp_valueN(0, 100*TOINT);
				continue;
			}
			if (!getOp_valueL(1)&& getOp_valueN(0)<=  -100*TOINT +step) {
				setOp_valueN(0, -100*TOINT);
				continue;
			}
			// movement
			//printf("NUMT 2 v=%i step=%i\n",getOp_valueN(0),step);
			if (getOp_valueL(1)) {
				short v = getOp_valueN(0);
				setOp_valueN(0, v+step);
				continue;
			}
			if (!getOp_valueL(1)) {
				short v = getOp_valueN(0);
				setOp_valueN(0, v-step);
				continue;
			}
		}//end NUMT
		if (opcode ==OP_FALSE) {
			setOp_valueL(0,false);
			continue;
		}
		if (opcode ==OP_TRUE) {
			setOp_valueL(0,true);
			continue;
		}
		if (opcode==OP_CHRPLUS || opcode==OP_CHRMIN) { // deprecated
			Var * v =getOp_Varptr(0);
			string name=v->name;
			int    x = getOp_valueN(0) ;
			if (opcode==OP_CHRMIN && x > -15000) {
				x --;
				setOp_valueN(0,x);
			}
			if (opcode==OP_CHRPLUS && x < 15000) {
				x ++;
				setOp_valueN(0,x);
			}
			int pos= (getOp_valueN(1)/100)+1; //index in the table lcd
			if (pos >=0 && pos<=7)
				lcd[pos]=name+"="+ lcdTimer(x);  // format the message ! max name length=5 char
			//printf("CHR outvar=%d  pos=%d   %s\n",x,pos,lcdTimer( x).c_str());
			continue;
		}
		//----display information-----------------------------------
		if (opcode==OP_BREAK) {   //only when @H1 is true  (and only for a coldstart)
			lcd[7]= getOp_valueS(0);
			lcd_protect[7]=GetTicker()+100;
			if (currentBreak != getOperationIndex()) {  // the sounds are generated once
				currentBreak = getOperationIndex();
				rc.sndptr->putFile("2A");  //alarm sound
				rc.sndptr->putFile("2BR");  //message
			}
			break;// not to EOF-> no reset of @H1
		}
		if (opcode==OP_SHOWT) {
			int x = getOp_valueN(0) /TOINT;
			if (x>=2 && x<8) {
				lcd[x]= getOp_valueS(1);
				lcd_protect[x] = GetTicker() +200;
			}
			continue;
		}
		if (opcode==OP_SHOW) {
			Var * v =getOp_Varptr(2);
			string name=v->name;
			if (v->value== NOT_AVAILABLE)
				continue;
			int x = getOp_valueN(0)/TOINT; //line
			int f  = getOp_valueN(1)/TOINT; //format
			if (x>=2 && x<8 &&  v->typeNum==NUM) {
				lcd[x]= name+ " " +v->valueS(f);
				lcd_protect[x] = GetTicker() +200; //protect against erase at the begin of this loop
			}
			continue;
		}
		if (opcode==OP_PLAYT) {
			rc.sndptr->putFile(getOp_valueS(0));
			continue;
		}
		if (opcode==OP_PLAY) {
			int f  = getOp_valueN(0)/TOINT; //format
			short v = getOp_valueN(1);
			if (v==NOT_AVAILABLE)
				continue;
			float x = (float) v/TOINT;
			rc.sndptr->putValue(x,f);
			continue;
		}
		if (opcode==OP_PLAYV) {
			int f  = getOp_valueN(0)/TOINT; //format
			short v = getOp_valueN(1);
			if (v==NOT_AVAILABLE)
				continue;
			float x = (float) v/TOINT;
			string z = getOp_Varptr(1)->name;
			rc.sndptr->putFile(z.c_str());
			//rc.sndptr->zput(200);
			rc.sndptr->putValue(x,f);
			//rc.sndptr->zput(200);
			if (f==9) //the units are generated by format 9
				continue;
			z=z+'X';
			rc.sndptr->putFile(z.c_str());
			rc.sndptr->zput(200);
			continue;
		}
		if (opcode==OP_PLAYS) {
			rc.sndptr->zput(getOp_valueN(0)); //silence unit msec
			continue;
		}
		if (opcode==OP_BEEP) {
			int n = getOp_valueN(0); //silence unit msec
			//2BES = beep 400msec     2BE = beep 100msec
			//printf("calc_run beep %d\n",n);
			while (n >= 400) {
				rc.sndptr->putFile("2BES");
				n = n -400;
			}
			while (n >= 100) {
				rc.sndptr->putFile("2BE");
				n = n -100;
			}
			continue;
		}
#ifdef JETI
		if (opcode==OP_JETI) {
			/* + 3 internal vars   1 asked to be active  2= last computed variable or unknown  3=validity in ticks
				invars
				X = node
				n = input variable, default 0
				d  = dividor ( default 1)
				L  = logical variable
				f  = format PLAYV
				JETI X            oplen = 5
				JETI X n         oplen =  6
				JETI X n d      oplen = 7
				JETI X n L f    oplen = 8
				JETI X n L f d oplen = 9
				internal variables
				    duration=validity in ticks  => oplen-1
						last computed variable or unknown => oplen -2
						asked to be active => oplen-3

				*/
			short sensorID = getOp_valueN(1)/100; //destination node id
			vector<string> z;
			if (oplen<=7 || (oplen>7 && getOp_valueL(3)))
				setOp_valueL(oplen-3,true);
			if (!getOp_valueL(oplen-3))
				continue;
			if (rc.jetiptr->JetiSensor(getOperationIndex(),sensorID, z) == true) {
				short sensorPosition=0;
				if (oplen>=6)
					sensorPosition = getOp_valueN(2)/100;// normal value 0 if there is only one value in this node
				short divider = 0;
				if (oplen==7)
					divider = getOp_valueN(3)/100;
				if (oplen==9)
					divider = getOp_valueN(5)/100;
				short value = NOT_AVAILABLE;
				// normal conversion
				if (z.size()>(unsigned)sensorPosition) { //variable is found
					string y = z[sensorPosition];
					convert(y,value,divider); //returns the value
				}
				setOp_valueN(0,value);
				setOp_valueN(oplen-2,value);
				setOp_valueN(oplen-1,1500);  // duration 100=1sec
				setOp_valueL(oplen-3,false);
				if (oplen>7) { // automatic play
					if (value==NOT_AVAILABLE)
						continue;
					int f  = getOp_valueN(4)/TOINT; //format
					string z = getOp_Varptr(0)->name;
					rc.sndptr->putFile(z.c_str());
					rc.sndptr->zput(200);
					float valueF ;
					valueF =((float) value) /TOINT;
					rc.sndptr->putValue(valueF,f);
					rc.sndptr->zput(200);
					z=z+'X';
					rc.sndptr->putFile(z.c_str());
					rc.sndptr->zput(200);
				}
			
			}
			continue;
		}
		if (opcode==OP_JETIRX) { //string parameter = Rx identification
			/* operation parameters
			  0 = string variable with the two identification numbers
			  1 = internal variable
						0=not yet controled
			      1= ok
			     >1= error  -repeat every ...
			*/
			if (getOp_valueN(1) ==0) {//not yet controlled
				vector<string> z;
				if (rc.jetiptr->JetiSensor(getOperationIndex(),3,  z) == true  && z.size()==2) {
					string x = getOp_valueS(0);
					string y  = z[0]+" "+ z[1];

					if (x.compare(y)==0) {
						setOp_valueN(1,1);
						printf("JETIRX valide=%s   \n",y.c_str());
					} else {
						setOp_valueN(1,2);
						printf("JETIRX error lu=%s   voulu=  %s\n",y.c_str(),x.c_str());
						lcd[7]= "JETIRX error";
						lcd_protect[7]=GetTicker()+300;
						rc.sndptr->putFile("2A");  //alarm sound
					}
				}
			}
		} //end JETIRX
#endif      //Jeti
		if (opcode==OP_LOG) {
			char buffer[10];
			if(rc.jetiptr->rx_count == rx_count_logged ) // do not log  without refreshed telemetry data 
				continue; //do not process 
			if (iop != currentLog  || rc.rccl_task_tick > (currentLogTime + 200)){ 
				currentLog=iop;
				string s ("#*"); //prepare a separator message with variable names
				// when rtc available:  timestamp	
				//s+="\t*";
				for (int i=0; i<oplen; ++i){
					s+='\t';
					s += getOp_Varptr( i)->name; 
			    }
			s+="\n" ; 
			logger_put(s);		
			}
				
			currentLogTime = rc.rccl_task_tick;
			// prepare a string
			sprintf(buffer,"%d",rc.rccl_task_tick ); //timestamp  (unit= tick 10 msec)
			string s (buffer);
			for (int i=0; i<oplen; ++i){
				s+='\t';
				s += getOp_Varptr( i)->valueS(2); 
			}
			s+="\n" ; 
			logger_put(s);
		}//end OP_LOG
		
		if(opcode==OP_PID || opcode==OP_PIDA){
			float co; //controller output
			float pv; //process variable
			short pv_s;// idem
			float sv ; //setup variable
			float e  ; //error
			float Kp,Ki,Kd; //PID parameters !time unit Ki = x/sec Kd=x*sec 
			//internal variables
			float Ci;  //Integral PID component
			float pv1;  // former value of the proces varaible pv 
			short rx_processed;//rx_count processed (if < rx_current -> this PID is turned off)
			 

			 // read the variables of the operation
			co = ((float) getOp_valueN(0) )/ TOINT; //this is the pilot control
			pv_s= getOp_valueN(1);
			if (pv_s==NOT_AVAILABLE)
				continue; //do nothing, let co unchanged  and the next call is a start of this PID
			int pid ;
			pv =((float)pv_s)/TOINT;
			//printf("PID pv=%5.2f co=%5.2f\n",pv,co); 

			if (opcode==OP_PID){//offset for the parameters of PID /PIDA
				pid=1; 
				// 0=co  PID    1=pv 2=sv 3=Kp  4=Ki  5=Kd  ( 6=esum 7=pv1  8=rx_processed 9=former PID correction)
				sv=((float)getOp_valueN(2)) /TOINT;
			}else{
				pid=0; //offset for PID)/TOINT
				// 0=co  PIDA    1=pv  2=Kp  3=Ki  4=Kd  ( 5=esum 6=pv1  7=rx_processed   8=sv 9=former PID correction)
				sv=((float)getOp_valueN(8)) /TOINT;
			}
			Kp=(float)getOp_valueN(2+pid) / TOINT;
			Ki=((float)getOp_valueN(3+pid)) * 0.080/TOINT;
			Kd=((float)getOp_valueN(4+pid))/(0.080*TOINT);
			Ci=((float)getOp_valueN(5+pid))/TOINT;
			pv1=((float)getOp_valueN(6+pid))/TOINT;
			rx_processed=getOp_valueN(7+pid);
			if(rc.jetiptr->rx_count == rx_count_logged ){ // do not update  without refreshed telemetry data 
				if (rx_processed > rc.jetiptr->rx_count -5){ //PID running 
					setOp_valueN(0,valcheck(getOp_valueN(0)+getOp_valueN(9) )); //9=former PID correction to co
				}
				continue; //do not process 
			}
			//printf("PID Kp=%5.2f Ki=%5.3f Kd=%5.3f Ci=%5.2f pv=%5.2f co=%5.2f\n",Kp,Ki,Kd,Ci,pv1,co); 
			// detection of the start (first call of the PID intruction 
			if (rx_processed < rc.jetiptr->rx_count -5){ //start this PID  after a delay of 5 (minimum is 1)
				//printf("PID start at %d\n",rc.jetiptr->rx_count);
				pv1=pv; //set the derivative component to 0
				Ci=0; //set the integral component to 0
				if (pid==0) {//PIDA
					sv = pv; // automatic initialization of the setpoint
					setOp_valueN(8,(short)sv*TOINT);
				}
				e = pv - sv; //error signal;
			}	else{ //this PID is running	
				e = pv - sv; //error signal;
				Ci += e *Ki;
				if (Ci < -100)
					Ci=-100;
				if (Ci >100)
					Ci=100;
			}				
			// PID computation
			float corr =  Kp*e + Ci - Kd*(pv-pv1);// PID control 
			co += corr; // combine the pilot control and the PID control
			pv1=pv;
			rx_processed =  rc.jetiptr->rx_count;
			//printf("PID co=%5.2f e=%5.2f  Ci=%5.2f  rx=%6d            \n",co,e,Ci,rx_processed);
			//update operation
			setOp_valueN(0,valcheck( (short)(co*TOINT)));
			setOp_valueN(5+pid,(short)(Ci*TOINT));
			setOp_valueN(6+pid,(short)(pv1*TOINT));
			setOp_valueN(7+pid,rx_processed);
			setOp_valueN(9,valcheck( (short)(corr*TOINT)));
			continue;
		}
		
		//-----------end--------------------------------------------
		slow_start=false;
	}//end main loop
	rx_count_logged= rc.jetiptr->rx_count; 
}
