/* folder core  = core logic of RCCL
  NOT depending on the hardware choice for the processor (with a 32 bits processor)
  BUT depending on the 4D serial acces (!!!! to be replaced with glcd.h)
  depends on printf for debug messages
  DEPENDING on the processor
  folder hwxxx and the main.cpp
*/
/*============================================================================
 Copyright (C) 2008 -2012 Robert Spilleboudt

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
============================================================================*/
#ifndef CORE_H
#define CORE_H
#include "mbed.h"
#include "calc.h"
#include <string>
#include <vector>
#include <queue>
#include <set>

/*==========    OPTIONS   ===========================================================*/
/*------langue: choisir une option---------------------------------------------------*/
//#define SPEAK_LAN_FR   //francais de france
#define SPEAK_LAN_FR_BE  //francais parle en Belgique (septante, nonante..)
#define JETI     // to include the  support of JETI Duplex; simulation of JetiBox , telemetry, alarms...
#define VBLOADED  6.80     // if voltage > VBLOADED then reset the counters - battery loaded
#define DTRIM_MAX 30 // maximum value of a trim (maximum= 100, normal value is 30 - trim = 30% of the travel)
//#define DISP_LANDSCAPE      //if the DISP program changed the hardware config to landscape
#define LCD_RESET_IN_OUT   // if a 4.7k resistor between VCC and Reset pin J2.5  (recommended but not needed)
/*------compiler choice  : select one option----------------------------------------*/
//#define COMPILER_MBED    // the normal on-line compiler of mbed.org
#define COMPILER_CS    // off-line compiler (codesourcery gcc compiler)
//#define COMPILER_ARM    // off-line compiler (arm  gcc compiler)
/*------other compiling options --------------------------------------*/
#define WATCHDOG  
#define WATCHDOG_TIMER 2 // watchdog timer (seconds, float) during the normal operation
#define WATCHDOG_STOP 600 // watchdog timer (float) during the load  operation
#define INACTIVITY   150000                        // 36000 = 360 sec = 6 minutes  ->autommatic power-off after this time without activity of the sticks (100 ticks /second -)
#define POWEROFF   1000     // a short time (1000= 10 sec) to cancel the power-off sequence with a movement of the sticks
//--------debugging options ------------------------------------------------------------------------------------------------------------------------------------------------------------------------
#define DEBUGP  //DEBUGP= activate the UART output for debugging with printf messages //
#define NOSTART_V   //do not start when there is no alimentation ( useful to debug with a terminal) 
//#define DEBUG_LCD_START  
/*===================================================================================*/
#define RAM_EXT  // RAM_EXT if the operation array is in "outside" RAM , outside of the 32k
#define OPLENGTH 2000 // length of the storage for operations (short[OPLENGTH] )
#define LOGMAX 20 //maxsize for the LOG message queue
/*-----serial display 4D uLCD-24PT(SGC) ----------------------------------*/
//see hwmbed/lcdhw Fast for the uart speed
// lcd dimension (after rotation->landscape)
#define LCDX  320
#define LCDY  240
// Defined basic colors (RGB)
#define WHITE  0xFFFFFF
#define BLACK  0x000000
#define RED     0xFF0000
#define GREEN 0x00FF00
#define BLUE    0x0000FF
#define YELLOW 0xFFFF00
#define CYAN    0x00FFFF
#define LGREY  0xBFBFBF
#define DGREY 0x5F5F5F
enum colorCode {BACKGROUND_TEXT,FOREGROUND_TEXT,BACKGROUND_CMD,FOREGROUND_CMD};
enum colorSet {LCD_RS ,LCD_BW,LCD_WB};
/* arrows */
#define ARROW_UP '\x00'
#define ARROW_DOWN '\x01'
#define ARROW_LEFT '\x02'
#define ARROW_RIGHT '\x03'
#define ACK  '\x06'
#define NAK  '\x15'

#define TICKER 10   // TICKER = duration in milliseconds of one tick (see GetTicker)
int FreeMem(void);
int GetTicker(); // returns the own systicker
void startOwnSysTicker();//start the systicker
void printTime(string text ,int t1);
//LCD color choices
int getColor(colorCode); //returns a RGB
int getColory(int y); //returns the color of variable y  in a graph
int getColorx(); //returns the color of a line in a graph
class Lcd4D {
	struct lcdCalibrateData {       /* calibration data for the touchscreen -> note AVR341 5.1
         XT,YT = coordinates from the touchscreen
         XD,YD = coordinates from the display  XD = A*XT + B*YT +C     YD= D*XT + E*YT + F
         file RCCLTS.TXT
         extended to other TX parameters
         */
		float A,B,C,D,E,F;
		short volume; //7..127
	};
    //needed for the sd card access
    int file_length; //how many bytes to read from this file
    int file_length_transfer;


public:
	string yfilename;
	int last_touch; //last touchscreen     (unit=GetTicker() and startup indicator if >0
	struct lcdCalibrateData calibrate;//shared by lcdcalibrate , lcdInit, lcdGetTouch
	colorSet colorChoice;
	Lcd4D(void);
	void ChangeColor();
	void Cls(void);
	void Control(char mode, char value);  //display control functions
	void Cmd(char *cmd,int n);  //write a command and check ACK
	void Int(char *cmd,int command_index,int x, int y); //translate a int(x,y) to 4 command bytes
	void Textg(string s, int x, int y, char font, int color, char width, char height);//text string in graphic mode
	void Textcmd(string s, int x, int y);    // draw a text string in the "command line"
	//void Text(string s, int x, int y, char font, int color);//text string in text mode
	void TextOpaque(bool); //true=opaque false=transparent
	void Color(char *command,int command_index, int color);//conversion of a RGB code to the 2bytes code for 4D
	void Pen(char mode);// set pen to SOLID or WIREFRAME
	void Rectangle(int x1, int y1 , int x2, int y2, int color);
	void Line(int x1, int y1 , int x2, int y2, int color);
	void Point(int x,int y, int color);  //display a small test point
	void Pixel(int x,int y, int color);  //display a pixel
	void ArrowCmd(char direction,int x , int y);//arrow within the 16*16  x,y= left top direction up=0 down=1 left=2 right=3
	void GetTouch(int *x, int *y); //get the calibrated output of the touchscreen
	bool GetTouchNB(int *x, int *y); //non blocking: true if touch
	bool GetTouchNBC(int *x, int *y); //non blocking continuous : true if touch
	bool GetTouchSlider(int *x, int *y); //non blocking: true if moving
	void Calibrate();// recalibrate the touchscreen
	void CalibrateCtrl();// control the calibration
	void CalibrateWrite();
	void CalibrateREAD();
	void CmdS(char *cmd,int n); //write a command or a part of a command
	void putString(string s); //write a string and appends a \0
	void Ack(void); //check ACK
	bool Reset(); //initial reset reeturns true if done
	int static font_x(char f);
	int static font_y(char f);
	   //sd card file access
	bool fexist(string filename);//return true if file exists
    bool fopenread(string filename);//return true if openend
    bool fopenwrite(string s);
    bool fgetline(string& s);
    void fputline(string& s);
    void fputclose();
    vector<string> fgetlist();
	// digital io
	void PinRead() ; //physical read of the 16 GPIO and transfer to the array Pin[ ]  - save to RPC register (watchdog) (seelcdhw)
	void PinRestore() ; //after watchdog restart, restore the register -> array Pin[ ] (see lcd hw)
	bool PinGet(int pin);//get the value of a pin from the internal array Pin[ ]
	void PinDump(); //debug message with the 16 pins
	//sound
	void mute(bool m);// digital IO to the pin 7
	void play(string filename);  // full filename
	int getSoundLen(string filename);
	void sleepSound();
	void setVolume();//adjust the volume to calibrate.volume
	void soundMove(string suffix);//transfer the .w* files froms the local file system to the 4D sd card
	//image
	bool Image(string filename); //returns true if success  - filename without extension (.GCI implicit)
	//static functions
	static int xpix(char font);  //returns the width of this font
	static int ypix(char font);
private:
	void putc(char x); //interface with the uart , non blocking (calls rccl_task
	char getc();
	char getcB(); //blocking interface
	void flush();
	void Fast(); //fast baud rate
	void Init1(); // first step: hardware reset
	void GetTouchBrut(int *x, int *y); //get the uncalibrated output of the touchscreen
    int GetFileSize();
    char Receive_buf(void); //buffered receive

};


/*---- Class mdx = a model file-------------------------------------------- */
#define MAXLINE 100  //maximum length of a input line
class Mdx {
	
public:
	string filename;
	vector<string> lignes ;
	Mdx(string filename)  ;//constructor from filename 8.3
	Mdx();  // constructor :a empty model
	int getnblignes();
	void debugprintf();
	string getline(int i);
	vector<string> gettokens(string z); //extract the tokens
	void save(); //save using the same name
	//void replace (int line ,string oldvalue, string newvalue);//replace in line  each occurence of old with new

};
/*-------duplicate  not in a class ---------------------------*/
vector<string> cleanline(string z);  //extract the tokens  -> mdx.cpp
void replace(string& line ,string oldvalue, string newvalue); //replace in line  each occurence of old with new (-> calcload)

/*----Editor  : graphic parameters -----------------------------------------*/
class Ed { //show a part of the file
	Mdx modele; //the file
	int marge;  // height of the "command line" below the text
	char font; // font code as defined by 4D, here for instance /x02 = 8*12
	int font_x, font_y; // dimension of a character, in pixels (  x=8,y=12 for the font 8*12)
	int n_x,n_y ;//number of characters for this font n_x =40 char/line n_y = 20 lines
	int pix_x, pix_y;  // 320,240 with this display
	int selectedline;
	bool readonly;
public:
	Ed(string f, bool readonly);// f= filename 8.3
	void  setfont(char font);
	void  display(); //display the file from line y, column x (top left corner)
	void  displayLine();
	void  displayHelp();
    void saveAs(); 
};
/*-----PPM signal generator --------------------------------------------------------*/
#define PPM_MAX_CH 16 //max channels for a ppm signal (depending on the transmitter module, here for Jeti)
void  ppm_init(int nb_channels, int frame_pulse); // example (6,20000) 6 channels frame=20 milliseconds
void  ppm_set(int channel ,int pulse_value);  // set the servo value for a  channel from 0 to nb_channels-1 ; pulse in usec)
void  ppm_start();
void  ppm_stop();


class Filey {// file operation with the mbed local file system
	FILE *fy;
public:
	/*
	filename format 8.3
	s = string to transfer
	the return code is true (operation done) or false (not done or EOF for a read)
	*/
	bool fopenread(string filename);//return true if openend
	bool fopenwrite(string s);
  void fremove(string s);
  bool fgetone(char& x) ; //get one char , return true if found, false at eof
	bool fgetline(string& s);
	int getlen(); //returns the length of the file (only open in red mode )
	void fputline(string& s);
	void ftoclose();
	vector<string> fgetlist();
	vector<string> fgetlist(string prefix,string suffix);
};


//---------------------------------------------------------------------------------
void Tokenize(const string& str, vector<string>& tokens, const string&);
void token_printf(vector<string> tokens);
/*------  watchdog   power------------------------------------------------*/
class Watchdog  {
public:
	static void kick(float s);  //set the watchdog duration = s
	static void kick();  //reset the watch dog
	static void setBU(char x);//save a backup register, available after the restart
	static  char getBU();
	static char getRSID();//returns the code for rc.startup
};
void power(int power_on);
//void power_off();

//task control and dialogs
void rccl_test(void);//a special test program called from the normal menu (model file loaded)
void rccl_task(int);//real time dispatcher for the rc loop each 10 msec
// 0 = waiting on lcd serial interface
// 1 = waiting on human touch

void rccl_dialog();//human interface (dialogs) dispatcher ; each dialogs dispatch the next dialog x with a call rccl_d(x)
enum dialog { MENU_SELECT,MENU_LOAD,MENU_ERROR,MENU_NORMAL,MENU_START,MENU_ONLINE,MENU_OFF,
        PARAM_UPD,PARAM_ASSIGN,VAR_DISPLAY,VAR_DISPLAY_SEL,VAR_GRAPH,MENU_EDITOR,MENU_EDITOR_RO,
        MENU_SETUP,TX_PARAM,TX_CALIBRATE,MENU_JETI,MENU_JETI_FOLLOW,MENU_SD4D,
	    MENU_LOGVIEW,MENU_LOGEXP,MENU_LOGDEL
            };
void menu_main(void);//the main menu
dialog menu_select(void);// menu when there is no current file
dialog menu_1oad(void);//there is a current file but not loaded in a calc objet
dialog menu_error(void);//idem but loaded with error
dialog menu_normal(void);//normal update menu
dialog menu_online(void);//normal update menu
dialog menu_off(void);//
dialog param_upd(void);//browse the parameters INIT values (vvar) and update
dialog param_assign(void);//browse the parameters (PVAR) to aaign a parameter to a digital trim
dialog menu_ed(void);// editor for the current file
dialog menu_ed_ro(void);// edisplaythe current file						
dialog var_display(void);//browse the variable and choose some variables to display "life"
dialog var_display_sel(void);//display only the selected variables
dialog var_graph();// display x var along the x axis and all the other varList along y axis
dialog menu_setup(); //all the transmitter parameters
dialog menu_sound();//the sound files on SD 4D
dialog menu_logview(); //menu to explore the logging data
dialog menu_logexp();//menu to export the looging file to the mbed filesystem
dialog menu_logdel();			
#define GRAPH_N  6  //maximum number of y variables in a graph		
/*-----JETI  interface -> /hwjeti  ----------------------------------------*/
#ifdef JETI
//jetibox commands  bit 4 =Right , Bit 5 = Up (or Back) , Bit 6 = Down or Select und Bit 7 Left
#define JETI_NOTHING 0xf0
#define JETI_RIGHT  0xe0
#define JETI_LEFT    0x70
#define JETI_UP       0xd0
#define JETI_DOWN  0xb0
#define JETI_ERASE 0x60

class Jeti {// the communication with the Jeti Tu module
public:
	Jeti(); //constructor of the class and start the serial communication
	void putJeti(char x) ;  //returns false if busy, true if processed  x = format ACK 0x
	string get1();
	string get2();
  void jeti_replace(); //replace strange characters   = <   ~=> ,  before displaying in the menu
	volatile int rx_count ; //count the received messages, from the start 
	volatile int status ; // 0 = normal (accept a Tx instruction) >0 processing of this  Tx instruction 
/* 0 = normal status 
   1 = tx instruction recorded (normal one shot instruction)
   2  = tx instruction sent to Jeti module
   3...   count the received messages to have a delay
      return to 0 after recceiving the following Rx message
   
*/
	volatile char jx[33] ; //message in the original format + \0
//performance
//Jeti_performance stat;
//explorer
 int connect_status; // control the sensor exploration by explorer_connect 0= connected (=done)
//--------- opration JETI : data acquisition from the sensors-----------------
	bool jeti_box_busy ; // true when the manual menu controls the dialog with Jeti -> cancel any automatic dialog
	short iop_busy; // >=0  iop of the Rccl JETI operation which is  currently processed    <0 when no JETI operation is busy -> process the next one
	short iop_last; //iop of the last processed JETI operation , 0 when back to the first
  bool   iop_busy_processed;
  int iop_rx_count; //rx count last processed message
	int iop_busy_time; // watchdog 

// functions
	bool JetiSensor(short iop_current,short sensorID, vector<string> &valueString); // process the JETI opcode processed=true when the busy iop is processed
  void explorer_connect() ;//search the connected sensors and update the explorer array
  void getVar(short sensorIndex, vector<string>& z );//extract the variables as strings from the  node at sensorIndex
	int    node_find() ;//returns the node id of the current node

	// single step within JetiSensor, between the nodes "from" and "next", direction fromnext
 short next;//next node, or ==0 when there is no "step"
 short from; 
 int  fromnext;
 char longtouch;
 //explorer
	void explorer_load() ; //load the descriptors file  ( after the load of the model .mdx file)
	bool explorer_exp(short node, string line);//change the descriptor of a expander line
	void explorer_dump() ; //debug dump of the explorer data
	void statReset(); // reset the performance counters	
	void statDump(); // debug print of the performance counters	
		private:
	bool isnode(int x);//return true if the current position is the node x
	int node_index(int nodeId);//find the index of the node with nodeID
	void link(int from, int direction, int to); //link from node Id , direction => node to
	void linkL(int from, int direction, int to); //link from node Id , direction => node to
  int readMap(short from , short to, short &next ,char& longtouch); // navigate from to ; current node = from, returns the direction to go for the end node to
  void putdir(int dir) ; // same as put, with the numerical direction code

 

};
dialog menu_jeti();//jeti box	dialog
dialog menu_jeti_follow();//follow the automatic navigation
//dialog menu_normal_jeti(void);//normal update menu with jeti sensors extensions

#endif

//----------lcd graphic components ------------------------------------------------
class Gr { //graphic component for any free graphic...
public:
	int x1,y1,x2,y2; //coordinates 1=left top point, 2=bottom right
	int color;//foreground color
	bool selectable;
	Gr(void);
	Gr(int x1,int y1,int x2,int y2);
	bool selected(int x,int y);//true if selectable and touch x,y within the component
	void erase();// draw a rectangle with the background color = erasure

};
class Grt : public Gr { //text graphic component
	char font;
	int mult; //font size multiplier (text in graphic mode)
	int textsize;
public:
	string text;
	Grt(int x1,int y1,int textsize,int multiplier, char font);
	Grt(int y); //full line
	void setText(string text);//without replacement if text does not change (-> no flicker )
	void setColor(int color);

};
class Gra : public Gr { //arrow graphic component
public:
	Gra(int x1,int y1);
	void setArrow(char direction);
};

//-----------------sound ----------------------------------
#define SNDQMAX 100    //max size sound queue before a new put
class Snd { //sound class
	
	/*  sound queue
	    normal string x -> sound played is x.wav
	    string begins with ? ?nnn -> stop playing during nnn ticks */
	int nextTick; //
	
public:
	bool muteOn;  //true when the amplifier is cut - suspend the lcd update too!
	queue <string> sndq;
   //void put(string filename); //put a sound file in the sound queue
	//void clear();//clear the queue
	void getNext(int tick); //real-time processing - if the curent sound is processed, play the next one tick=current ticker
	void putFile(const string filename) ; //put a sounf file (without the suffix .wav)
	void putValue(float value,int format);// spell this value (format *TOINT) and put the sounds in the queue
	//void init();
	Snd(void);
private:
	void conv3(int nombre) ;//put a integer +-999 in the queue !! depending on the langage -> sound_xx
	void conv2(int nombre) ;//put the decimal part 2 decimals
	void conv1(int nombre) ;//put the decimal part 1 decimal
	void xput(string f); // generer le son definit par le string f  (filename witout the suffiw .wav)
	void nput(int i) ;  //generer le son definit par le nombre i   exemple i=12 -> 12 (12.wav)
public:
	void zput(int i) ;  // put a silence of i msec
};
class AdcFilter {//filter for the adc, independant of the pin read
public:
	AdcFilter(void);
	float delta ; //maximum difference between two samples (periodically sampled)
	float smooth; //expo filter 1.= no filter
	float filter(float x);
private :
	float y;//current smoother value
	bool init; // false at start
};
class Debounce {// debouce for one button
public:
	Debounce();
	bool get(bool x); // call each 10 msec - provide ONE true response when a button is pushed
	unsigned char state;
	int ticker; //ticker when the pushed state is recognized 

};
//-------loggging 
void logger_export(string s);//copy a log file to the mbed
//see logging.cpp

/*-------global control data common to all the programs---------------------------------------------*/
#define COLDSTART '0'
#define WARMSTART '3'
#define LCD_NO_READY '0'
#define LCD_PIN_RESTORED '1'
#define LCD_READY '2'
struct battery { /* last voltage and accumulation of mAh */
	float vb1,vb2,ib1,ib2,cb1,cb2;
};
struct rccl_data {
	string current_file; //current model file name ,without the extension
	Calc* calcptr;  // pointer to the current calc object (NULL if not loaded)
	Lcd4D* lcdptr;   // pointer to the lcd object
	Snd*  sndptr; //pointer to the sound object
#ifdef JETI
	Jeti*    jetiptr; //pointer to the JetiBox object..or NULL
#endif
	set<short> *varSelection; //selection of index to the variables to display in some menus (normal empty)
	int rccl_task_tick;
	char startmode; //  see watchdog  '0'=coldstart  '3'=warmstart (watchdog)
	char lcdstate ; // '2' = fully available     0=not started
	//char poweroffconfirm ; //P = confirm  not needed with new poweroff
	battery batt;
};
void InitSave(); // save data to RCCLINIT


#endif