#include "core.h"
extern rccl_data rc; //common data
/* lcd = application logic for communicating with the SERIAL lcd 4D
 */
int Lcd4D::xpix(char font) {
    switch (font) {
    case '\x00':
        return 5;
    case '\x01':
        return 8;
    case '\x03':
        return 12;
    default:
        return 8;
    }
}
int Lcd4D::ypix(char font) {
    switch (font) {
    case '\x00':
        return 8;
    case '\x01':
        return 8;
    case '\x03':
        return 16;
    default:
        return 8;
    }
}
Lcd4D::Lcd4D() {
	#ifdef DEBUG_LCD_START
		printf("Lcd4D constructor -a\n");
	#endif
    last_touch=0; //-> not ready
	  if (rc.startmode==WARMSTART ){
      //PinRestore(); //restore the digital io from the RTC - real values after a Warmstart, 
			rc.lcdstate= LCD_PIN_RESTORED;
			#ifdef DEBUG_LCD_START
		   printf("Lcd4D PinRestore \n");
			 PinDump();
	    #endif
		}
    Init1(); // lcdhw : initial reset
    char cmd[5];
    cmd [0]='U';
    Cmd(cmd,1);
		#ifdef DEBUG_LCD_START
		printf("Lcd4D constructor cmd U-a\n");
	#endif
    Fast();//switch to a higher transmission speed
		PinRead();//first read to initialize the array Pin correction20120429
    cmd[0]= '\x59';
    cmd[1]='\x04';
    cmd[2]='\x01';
    Cmd(cmd,3); //orientation landscape 
		cmd[0]= '\x59';
    cmd[1]='\x02';
    cmd[2]='\x0F';
    Cmd(cmd,3); //contrast max
    cmd[0]= '\x46';
    cmd[1]='\x02';
    Cmd(cmd,2); //default font 8*12
    //calibrate default (if no file )
    calibrate.A= 1.0;
    calibrate.B= 0.0;
    calibrate.C= 0;
    calibrate.D= 0.0;
    calibrate.E= 1.0;
    calibrate.F= 0;
    colorChoice=LCD_RS;
    //read the last calibrate file
    CalibrateREAD();
    Control('\x05','\x00');//enable touchscreen
    last_touch=GetTicker();
    //init SD
    cmd[0]= '\x40';
    cmd[1]='\x69';
    Cmd(cmd,2);
 //   rc.sndptr = new (nothrow) Snd(); moved to rccl init
		#ifdef DEBUG_LCD_START
    printf("Lcd4D  init ok at %d\n",GetTicker());
		#endif

}
void Lcd4D::ChangeColor() {
    char cmd[5];
    cmd[0]= '\x42';
    Color(cmd,1,getColor(BACKGROUND_TEXT));
    Cmd(cmd,3); //background !! 2,1 sec
}

void Lcd4D::CmdS(char *cmd,int n) {//write a command or a part of command
    for (int i=0; i<n; i++) {
        putc(cmd[i]);
    }
}
void Lcd4D::putString(string s) {//write a string
    for (int i=0; i<(int)s.length(); i++) {
        putc(s.at(i));
        // wait_ms(1);
    }
    putc('\0');
}
void Lcd4D::Ack() { // check the ACK answer or stop
    char resp; // printf("Wait ACK command\n");
    resp=getc();
    if (resp==ACK)
        return;
    if ( resp == NAK)  {
        error("Error NAK after the command\n");
        while (1) //replace with a log + abort
            {}
    }
    error("Error ??? after the command\n");
    while (1) {}
}

void Lcd4D::Cmd(char *cmd,int n) {//write a command  AND check the answer ACK
    CmdS(cmd,n); // "short" command without ack control
    Ack();
}

void Lcd4D::Cls(void) {//clear the lcd
    char cmd[1];
    cmd[0]= '\x45';
    Cmd(cmd,1);
    TextOpaque(true);
    //printf("lcd.Cls()\n");
}
void Lcd4D::Control (char mode, char value) {//control mode and value
    char cmd[3];
    cmd[0]= '\x59';
    cmd[1]=mode;
    cmd[2]=value;
    Cmd(cmd,3);
}
void Lcd4D::Color(char *command,int command_index, int color) {//convert RGB value to 2 bytes for 4D at command_index
    int red5   = (color >> (16 + 3)) & 0x1F;              // get red on 5 bits
    int green6 = (color >> (8 + 2))  & 0x3F;              // get green on 6 bits
    int blue5  = (color >> (0 + 3))  & 0x1F;              // get blue on 5 bits
    command[command_index] = ((red5 << 3)   + (green6 >> 3)) & 0xFF;  // first part of 16 bits color
    command[command_index+1] = ((green6 << 5) + (blue5 >>  0)) & 0xFF;  // second part of 16 bits color
}
void Lcd4D::Textg(string s, int x, int y, char font, int color, char width = 1, char height = 1) {   // draw a text string
    char cmd[10];
    cmd[0] ='\x53'; //draw string in graph format
    Int(cmd,1,x,y);
    cmd[5] = font;
    Color(cmd,6,color);
    cmd[8] = width;
    cmd[9] = height;
    CmdS(cmd,10); //send theses command bytes
    putString(s); //send the string and append with \0
    Ack();
}
void Lcd4D::Textcmd(string s, int x, int y ) {   // draw a text string
    char cmd[10];
    cmd[0] ='\x53'; //draw string in graph format
    Int(cmd,1,x,y);
    cmd[5] = '\03'; //font
    Color(cmd,6,getColor(FOREGROUND_CMD));
    cmd[8] = '\01';
    cmd[9] = '\01';
    CmdS(cmd,10); //send theses command bytes
    putString(s); //send the string and append with \0
    Ack();
}
void Lcd4D::TextOpaque(bool opaque) {
    char cmd[2];
    cmd[0]= '\x4F';
    if (opaque)
        cmd[1]='\x00';
    else
        cmd[1]='\x01';
    Cmd(cmd,2); //transparent
}
void Lcd4D::ArrowCmd(char direction,int x , int y) {//arrow within the 16*16  x,y= left top direction up=0 down=1 left=2 right=3
    char cmd[15];
    cmd[0]='\x47' ;//draw a triangle
    switch (direction) {
    case ARROW_UP:
        Int(cmd,1,x+8,y);
        Int(cmd,5,x+4,y+16);
        Int(cmd,9,x+12,y+16);
        break;
    case ARROW_DOWN:
        Int(cmd,1,x+8,y+16);
        Int(cmd,5,x+12,y);
        Int(cmd,9,x+4,y);
        break;
    case ARROW_RIGHT:
        Int(cmd,1,x,y+4);
        Int(cmd,5,x,y+12);
        Int(cmd,9,x+16,y+8);

        break;
    default:
        Int(cmd,1,x+16,y+4);
        Int(cmd,5,x,y+8);
        Int(cmd,9,x+16,y+12);
    }
    Color(cmd,13,getColor(FOREGROUND_CMD));
    Cmd(cmd,15);
}
/*void Lcd4D::Text (string s, int x, int y, char font, int color) {   // draw a text string in text mode
//inutilis
    
    char cmd[10];
    cmd[0] ='\x73'; //draw string in graph format
    cmd[1]=x;
    cmd[2]=y;
    cmd[3] = font;
    Color(cmd,4,color);
    CmdS(cmd,6); //send theses command bytes
    putString(s); //send the string and append with \0
    Ack();
}*/
void Lcd4D::Int (char *cmd,int command_index,int x,int y) {//translate a int x,y value to 4 command bytes
    cmd[command_index] = (x >> 8) & 0xFF;
    cmd[command_index+1] = x & 0xFF;
    cmd[command_index+2] = (y >> 8) & 0xFF;
    cmd[command_index+3] = y & 0xFF;
}
void Lcd4D::Pen(char mode) {   // set pen to SOLID or WIREFRAME
    char cmd[2];
    cmd[0]= '\x70';
    cmd[1]=mode;
    Cmd(cmd,2); //set pen size 00 for a solid rectangle
}
void Lcd4D::Rectangle(int x1, int y1 , int x2, int y2, int color) {
    char cmd[20];
    cmd[0]= '\x72';
    Int(cmd,1,x1,y1);
    Int(cmd,5,x2,y2);
    Color(cmd,9,color);
    Cmd(cmd,11);
}
void Lcd4D::Line(int x1, int y1 , int x2, int y2, int color) {
    char cmd[20];
    cmd[0]= '\x4C';
    Int(cmd,1,x1,y1);
    Int(cmd,5,x2,y2);
    Color(cmd,9,color);
    Cmd(cmd,11);
}
void Lcd4D::Point( int x,int y, int color) { //display a small circle
    char cmd[10];
    cmd[0]='\x43';
    Int(cmd,1,x,y);
    cmd[5]='\x00';
    cmd[6]='\x02'; //diameter
    Color(cmd,7,color);
    Cmd(cmd,9);
}
void Lcd4D::Pixel( int x,int y, int color) { //display a small test point
    char cmd[10];
    cmd[0]='\x50';
    Int(cmd,1,x,y);
    Color(cmd,5,color);
    Cmd(cmd,7);
}

bool Lcd4D::GetTouchNB( int  *x, int *y) {//true=returns the coordinates  false=return without touch
    char cmd[2];
    char answer[4];
    int tick;
    cmd[0]='\x6F';
    cmd[1]='\x04'; //command get touch status
    CmdS(cmd,2);
    for (int i=0; i<4; i++) {
        answer[i]=  getc(); // 2 bytes x + 2 bytes y
    }
    int sx;
    sx = ((answer[0]<<8)+ answer[1]) * (answer[0] != 0xFF) ;
    if (sx !=1) { //testing if touch press
        rccl_task(1);//the rc loop with the processing each 10 msec
        return false;
    }
    tick=GetTicker();
    //printf("touch status= 01 tick=%d last_touch=%d\n",tick,last_touch);
    if (tick < last_touch+10) { //avoid double touch within 100 msec (adapt to detect a double touch !)
        return false;
    }
    last_touch=tick;
    cmd[0]='\x6F';
    cmd[1]='\x05';
    CmdS(cmd,2); //command get touch coordinates
    for (int i=0; i<4; i++) {
        answer[i]=  getc(); // 2 bytes x + 2 bytes y
    }
    // brut
    int xt,yt;
    xt = ((answer[0]<<8)+ answer[1]) * (answer[0] != 0xFF) ;
    yt = ((answer[2]<<8)+ answer[3]) * (answer[2] != 0xFF) ;
    //calibrate
    *x=int(calibrate.A * xt + calibrate.B*yt + calibrate.C);
    *y=int(calibrate.D * xt + calibrate.E*yt + calibrate.F);
    if (rc.calcptr!= NULL)
        rc.calcptr->kick();
    return true;
}
bool Lcd4D::GetTouchNBC( int  *x, int *y) {//true=returns the coordinates  false=return without touch ;
	//NB=non block C=continuous, for the Jeti Box
    char cmd[2];
    char answer[4];
    int tick;
    cmd[0]='\x6F';
    cmd[1]='\x04'; //command get touch status
    CmdS(cmd,2);
    for (int i=0; i<4; i++) {
        answer[i]=  getc(); // 2 bytes x + 2 bytes y
    }
    int sx;
    sx = ((answer[0]<<8)+ answer[1]) * (answer[0] != 0xFF) ;
    if (sx ==0) { //testing if touch press
        rccl_task(1);//the rc loop with the processing each 10 msec
        return false;
    }
    tick=GetTicker();
    //printf("touch status= 01 tick=%d last_touch=%d\n",tick,last_touch);
     last_touch=tick;
    cmd[0]='\x6F';
    cmd[1]='\x05';
    CmdS(cmd,2); //command get touch coordinates
    for (int i=0; i<4; i++) {
        answer[i]=  getc(); // 2 bytes x + 2 bytes y
    }
    // brut
    int xt,yt;
    xt = ((answer[0]<<8)+ answer[1]) * (answer[0] != 0xFF) ;
    yt = ((answer[2]<<8)+ answer[3]) * (answer[2] != 0xFF) ;
    //calibrate
    *x=int(calibrate.A * xt + calibrate.B*yt + calibrate.C);
    *y=int(calibrate.D * xt + calibrate.E*yt + calibrate.F);
    if (rc.calcptr!= NULL)
        rc.calcptr->kick();
    return true;
}
bool Lcd4D::GetTouchSlider( int  *x, int *y) {//true=returns the coordinates  false=return if not moving
    char cmd[2];
    char answer[4];
    cmd[0]='\x6F';
    cmd[1]='\x04';
    CmdS(cmd,2); //command get touch status
    for (int i=0; i<4; i++) {
        answer[i]=  getc(); // 2 bytes x + 2 bytes y
    }
    int sx;
    sx = ((answer[0]<<8)+ answer[1]) * (answer[0] != 0xFF) ;
    if (sx !=3) { //testing if touch moving
        rccl_task(1);//the rc loop with the processing each 10 msec
        return false;
    }

    cmd[0]='\x6F';
    cmd[1]='\x05';
    CmdS(cmd,2); //command get touch coordinates
    for (int i=0; i<4; i++) {
        answer[i]=  getc(); // 2 bytes x + 2 bytes y
    }
    // brut
    int xt,yt;
    xt = ((answer[0]<<8)+ answer[1]) * (answer[0] != 0xFF) ;
    yt = ((answer[2]<<8)+ answer[3]) * (answer[2] != 0xFF) ;
    //calibrate
    *x=int(calibrate.A * xt + calibrate.B*yt + calibrate.C);
    *y=int(calibrate.D * xt + calibrate.E*yt + calibrate.F);
    if (rc.calcptr!= NULL)
        rc.calcptr->kick();
    return true;
}
void Lcd4D::GetTouchBrut( int  *x, int *y) {//returns the coordinates after wait/release
    char cmd[2];
    char answer[4];
    int tick;
    while (1) {
        cmd[0]='\x6F';
        cmd[1]='\x04';
        CmdS(cmd,2); //command get touch status
        for (int i=0; i<4; i++) {
            answer[i]=  getc(); // 2 bytes x + 2 bytes y
        }
        int sx;
        sx = ((answer[0]<<8)+ answer[1]) * (answer[0] != 0xFF) ;
        if (sx !=1) { //testing if touch press
            rccl_task(1);//the rc loop with the processing each 10 msec
            continue;
        }
        tick=GetTicker();
        //printf("touch status= 01 tick=%d last_touch=%d\n",tick,last_touch);
        if (tick > last_touch+50) { //avoid double touch within 500 msec (adapt to detect a double touch !)
            last_touch=tick;
            break;
        }
    }

    //printf("touch coordinates\n");
    cmd[0]='\x6F';
    cmd[1]='\x05';
    CmdS(cmd,2); //command get touch coordinates
    for (int i=0; i<4; i++) {
        answer[i]=  getc(); // 2 bytes x + 2 bytes y
    }
    *x = ((answer[0]<<8)+ answer[1]) * (answer[0] != 0xFF) ;
    *y = ((answer[2]<<8)+ answer[3]) * (answer[2] != 0xFF) ;
    //printf("touchBrut coordinates x=%d y=%d\n", *x, *y);
    if (rc.calcptr!= NULL)
        rc.calcptr->kick();

}
void Lcd4D::GetTouch( int  *x, int *y) {//returns the coordinates after calibrate
    int  xt;
    int  yt;
    GetTouchBrut(&xt,&yt);
    *x=int(calibrate.A * xt + calibrate.B*yt + calibrate.C);
    *y=int(calibrate.D * xt + calibrate.E*yt + calibrate.F);
    //printf("touch coordinates x=%d y=%d\n", *x, *y);
}

float distance(float x1,float y1,float x2,float y2){
	float d2 =((x1-x2)*(x1-x2) )+((y1-y2)*(y1-y2));
 return sqrt(d2);	
}

void Lcd4D::CalibrateCtrl() { //check the calibration
    Control('\x05','\x00');//enable touchscreen
  /*  float xd[3]={LCDX*0.1 , LCDX*0.9, LCDX*0.5}; //display a point xd,yd and read the touchscreen xt,yt
    float yd[3]={LCDY*0.1 , LCDY*0.5 ,LCDY*0.9};
    int xts=0;
    int yts=0;

	Cls();
    for (int j=0; j<3 ; j++) { //point 1 to 3
        Point (xd[j],yd[j],BLUE);
        GetTouch(  &xts,  &yts);
        Point (xts,yts,RED);
			  //printf("Calibrate residual err=%5.0f \n",distance(xd[j],yd[j],xts,yts));
    }
	wait(1);	*/
	int xts =0;
	int yts=0;
	Cls();
    for (int j=0; j<10 ; j++) {
		GetTouch(  &xts,  &yts);  // choose where  to control
		Point (xts,yts,BLUE);       
		GetTouch(  &xts,  &yts);
		Point (xts,yts,RED);
	}
	
}


void Lcd4D::Calibrate() { //calibration following avr note 341
    Control('\x05','\x00');//enable touchscreen
    float xd[3]={LCDX*0.15 , LCDX*0.85, LCDX*0.5}; //display a point xd,yd and read the touchscreen xt,yt
    float yd[3]={LCDY*0.15 , LCDY*0.5 ,LCDY*0.85};
    float xt[3]={0,0,0};
    float yt[3]={0,0,0};
    /* show ncal times 3 black points,
        touch the screen zt each point
        the detected coordinates give  is a red point
    */
    int ncal=4; //measure ncal times to average
    float ndiv= 1.0/ ncal;
    int xts=0;
    int yts=0;
    for (int i=0; i<ncal; i++) { //do some loops
        Cls();
        for (int j=0; j<3 ; j++) { //point 1 to 3
            Point (xd[j],yd[j],BLACK);
            GetTouchBrut(  &xts,  &yts);
            Point (xts,yts,RED);
					  float d =distance(xd[j],yd[j],xts,yts);
					  printf("Calibrate err=%5.0f \n",d);
            xt[j] += xts*ndiv;
            yt[j] += yts*ndiv;
        }
        wait(1);    //short wait after the point 3
    }
    /*  solve this equation to compute the correction coefficients (see note avr 341)  */
    float xd1=xd[0];  //as the variable names of avr341
    float xd2=xd[1];
    float xd3=xd[2];
    float yd1=yd[0];
    float yd2=yd[1];
    float yd3=yd[2];
    float xt1=xt[0];
    float xt2=xt[1];
    float xt3=xt[2];
    float yt1=yt[0];
    float yt2=yt[1];
    float yt3=yt[2];

    calibrate.A = ((xd1*(yt2-yt3)) +(xd2*(yt3-yt1)) + (xd3*(yt1-yt2))) /
                  ( (xt1*(yt2-yt3))+(xt2*(yt3-yt1))+(xt3*(yt1-yt2)) );
    calibrate.B = ( calibrate.A*(xt3-xt2)+xd2-xd3) /  (yt2-yt3);
    calibrate.C = xd3 - calibrate.A*xt3 - calibrate.B*yt3;
    calibrate.D = (yd1*(yt2-yt3)+ yd2*(yt3-yt1) + yd3*(yt1-yt2))/
                  (xt1*(yt2-yt3) +xt2*(yt3-yt1) + xt3*(yt1-yt2));
    calibrate.E = (calibrate.D *(xt3-xt2)+yd2-yd3) / (yt2-yt3);
    calibrate.F = yd3 - calibrate.D*xt3 - calibrate.E*yt3;
    printf ("calibrate A=%5.3f B=%5.3f C=%5.3f\n",calibrate.A,calibrate.B,calibrate.C);
    printf ("calibrate D=%5.3f E=%5.3f F=%5.3f\n",calibrate.D,calibrate.E,calibrate.F);
}
//------------font
int Lcd4D::font_x(char f) {
// calculates the number of line/colums... to display the text
    switch (f) {
    case '\x00':
        return 5;
        //font_x=5; font_y=8;font=f;break;// !!!error in the documentation; height is 8
    case '\x01':
        return 8;
        //font_x=8; font_y=8;font=f;break;
    case '\x03':
        return 12;
        // font_x=12; font_y=16;font=f;break; // 15 lignes * 26 col
    default:
        return 8;
        // font_x=8; font_y=12;font='\x02';break;
    }
}
int  Lcd4D::font_y(char f) {
// calculates the number of line/colums... to display the text
    switch (f) {
    case '\x00':
        return 8;
        //font_x=5; font_y=8;font=f;break;// !!!error in the documentation; height is 8
    case '\x01':
        return 8;
        //font_x=8; font_y=8;font=f;break;
    case '\x03':
        return 16;
        // font_x=12; font_y=16;font=f;break; // 15 lignes * 26 col
    default:
        return 12;
        // font_x=8; font_y=12;font='\x02';break;
    }
}
/*
//----------digital inputs---------------------------------------------------
bool Lcd4D::PinGet(int n) {
    return Pin[n];
}
void Lcd4D::PinDump(){
	printf("PinDump at %d  ",GetTicker());
	for (int i=0;i<=15;i++){
		if (Pin[i])
	   printf ("1");
		else
			printf ("0");
	}
	
	unsigned char uc1 = (unsigned char)(LPC_RTC->GPREG1 >> 24);
	unsigned char uc2 = (unsigned char)(LPC_RTC->GPREG1 >> 16);
	unsigned char uc3 = (unsigned char)(LPC_RTC->GPREG1 >>  8);
	unsigned char uc4 = (unsigned char)(LPC_RTC->GPREG1 >>  0);
	printf("  hex bytes: %02X %02X %02X %02X\n",uc1,uc2,uc3,uc4);	
}
*/
//-----------sounds -> sound.cpp
//--------graphic components-----------------------------------------------------
Gr::Gr() {
    x1=0;
    y1=0;
    x2=0;
    y2=0;
    color=getColor(FOREGROUND_TEXT);
}
Gr::Gr(int a1,int b1,int a2, int b2) {
    x1=a1;
    y1=b1;
    x2=a2;
    y2=b2;
    color=getColor(FOREGROUND_TEXT);
    selectable=false;
}
bool Gr::selected(int x,int y) {//x,y = coordinates of the touch point
    if (!selectable) return false;
    if (x>= x1 && x<=x2 && y>=y1 && y<= y2) return true;
    return false;
}
void Gr::erase() {
    rc.lcdptr->Lcd4D::Rectangle(x1,y1,x2,y2,getColor(BACKGROUND_TEXT));

}
Grt::Grt(int x,int y,int n, int m,char f) {//text component at (x,y) for n characters, m=size multiplier
    x1=x;
    y1=y;
    color=getColor(FOREGROUND_TEXT);
    font=f;
    //font='\x03';
    mult=m;
    x2=x1+n*Lcd4D::xpix(font)*mult;
    if (x2>LCDX) x2=LCDX;
    y2=y1+Lcd4D::ypix(font)*mult;
    if (y2>LCDY) y2=LCDY;
    textsize=(x2-x1)/(Lcd4D::xpix(font)*mult);
    //printf("Grt x1=%d y1=%d x2=%d y2=%d textsize=%d\n",x1,y1,x2,y2,textsize);
    text=string(textsize,'\x20');
}
Grt::Grt(int y) { // text component at y (x=0, full line)
    x1=0;
    y1=y;
    color=getColor(FOREGROUND_TEXT);
    font='\x03';
    mult=1;
    x2=LCDX;
    y2=y1+Lcd4D::ypix(font)*mult;
    if (y2>LCDY) y2=LCDY;
    textsize=(x2-x1)/(Lcd4D::xpix(font)*mult);
    //printf("Grt x1=%d y1=%d x2=%d y2=%d textsize=%d\n",x1,y1,x2,y2,textsize);
    text=string(textsize,'\x20');
}
void Grt::setText(string t) {
    //printf(" Old=%s new=%s size=%d  max=%d\n",text.c_str(),t.c_str(),text.size(),textsize);
    if (t.compare(text)==0) return;
  	if (t.substr(0,textsize).compare(text)==0) return;
    if (t.compare(" ")==0)
        color=getColor(FOREGROUND_TEXT);
    erase();// repaint the background

    if ((int)t.size()<= textsize)
        text=t;
    else
        text=t.substr(0,textsize);
    rc.lcdptr->Textg (text, x1,y1,font, color,mult,mult);
}
Gra::Gra(int a1,int b1) {
    x1=a1;
    y1=b1;
    x2=x1+16;
    y2=y1+16;
    color=getColor(FOREGROUND_CMD);
    selectable=true;
}
void Gra::setArrow(char direction ) {
    rc.lcdptr->ArrowCmd(direction,x1 ,y1);
}
//-------images
bool Lcd4D::Image(string filename){//string without extension returns true when successfull 
	//printf("Lcd4D Image =%s \n",filename.c_str());
	char cmd[10];
	#ifndef DISP_LANDSCAPE
	//the hardware disp is portrait - display now a rotated image
	 //printf("Image -> portrait\n");
	 cmd[0]= '\x59';
   cmd[1]='\x04';
   cmd[2]='\x03';
   Cmd(cmd,3); //orientation native  
	Cls();
	#endif
	cmd[0]='\x40';
	cmd[1]='\x6D';
	CmdS(cmd,2); 
	putString(filename+".Gci"); //returns NAK if file not found
	cmd[0]='\x00';  //x
	cmd[1]='\x00';
	cmd[2]='\x00';  //y
	cmd[3]='\x00';
	cmd[4]='\x00';
	cmd[5]='\x00';
	cmd[6]='\x00';
	cmd[7]='\x00';
	CmdS(cmd,8); 
	//Ack();
	char resp =getc();  //Ack or Nak ...
	#ifndef DISP_LANDSCAPE
	 // printf("Image -> landscape\n");
	 cmd[0]= '\x59';
   cmd[1]='\x04';
   cmd[2]='\x01';
   Cmd(cmd,3); //orientation landscape 
	 #endif
	
	if (resp==ACK)
		return true;
	else
		return false;
	 
 
}
