// logging and log data explorer
#include "core.h"
extern rccl_data rc; //common data
// ========data for the logview logic================================
class LogView {
		public:
	// data for the selected segment
	//short segment_nr ; //number of the selected segment (0...n)
	int segment_debut  ; //timestamp for the first record (format ticks of GetTicker() , 1/100 sec)
	int segment_fin ; 
	vector <string> varname; //variable names
	// graphical display
	int screen_var ; //index 0..n of the variables to display 
	//scaling X axis
	int screen_debut ; //display records from this timestamp 
	int screen_scale_X    ; //index to the scale_X -> defines the compression of the time along X axis
	//scaling Y
	float Y_min, Y_max;	
	bool coordinates ;	
	//point value
	float Yval [LCDY];
	float  Xval [LCDX];
	//internal 
	int select();  //returns 0 if nothing to view ; if a selection, returns number of records in the mbed file RCCL.LOG
    void scaleX(); //choose the X scale 
	void scaleY(); //choose the Y  scale 	 after having choosed a X scale 
	void spellVar(); //spell the variable name
	void spellValue(int xt);//value at one point in the graph
	void showgraph();//display the graph
};
LogView* lv; 
const short scale_X [ ]={1,2,5, 10,20,50, 100,200,500, 1000,2000,5000, 10000,20000};
// scale_X = divider  to compress several ticks (10 msec) within one pixel
//example 1000 -> 1000 ticks or 10 seconds /pixel
const int scale_X_size = 14;
//==== fill the logging file on the sd card (Lcd4D)
void Calc::logger_put(string s) { //put a string in the queue to the lcd sd card  - see calcrun
	//calc_logger is the message queue
	if(rc.startmode !=COLDSTART)
		return; //no logging after a "accident"
	if(logq.size()< LOGMAX)
		logq.push(s);
}
void Calc::logger_sd() { // get a string from the queue and send this to the lcd
	if(logq.size()==0)
		return;
	rc.lcdptr->yfilename = rc.current_file + ".LOG"; //this is the filename for the logger
	rc.lcdptr->fputline(logq.front());
	logq.pop();
}
//======menu logging___________________________________
// delete calc object to avoid active logging 
dialog menu_logexp() { //export the log file to the mbed filesystem
	printf("menu_logexp \n");
	rc.lcdptr->Cls();
	delete rc.calcptr; //destroy the Calc object !!! -> stop the watchdog and free memory (somme of the operations are too long)
	rc.calcptr =NULL;
	string fname =rc.current_file+".LOG";
	logger_export(fname); //copy the file		
	return MENU_START ; //back ------------------------------------------
}

dialog menu_logview() { //graphical display of the logging
	printf("menu_logview \n");
	rc.lcdptr->Cls();
	delete rc.calcptr; //destroy the Calc object !!! -> stop the watchdog and free memory (somme of the operations are too long)
	rc.calcptr =NULL;
	lv = new(nothrow) LogView();
	while (true){
		int nb= lv->select();
		//printf("Menu_logview1=%d\n",nb);
		if (nb<0)
			break; 
		lv->scaleX();// select the range, display ...
	}
	delete lv;
	return MENU_START ; 
}
dialog menu_logdel(){
	printf("menu_logdel \n");
	rc.lcdptr->Cls();
	rc.lcdptr->fopenwrite(rc.current_file+".LOG");//erase from the SDcard
	return MENU_START ; 
}



//======copy the file to the Mbed filesystem
void logger_export(string fname) {
	if(rc.lcdptr->fopenread(fname)) {
		string line;
		Filey mfile; //output file in the mbed local file system
		if(! mfile.fopenwrite(fname))
			return; //error
		int len =0;
		while(rc.lcdptr->fgetline(line)) {
			line +="\n";
			len+= line.size();
			mfile.fputline(line);
			if (len > 500000){
				string error = "# error / too big file";
				mfile.fputline(error);
				mfile.ftoclose();
				return;				
			}
		}
		mfile.ftoclose();
		return;
		
	}
	printf("logger_export no file\n");
}
// logview logic
int LogView::select() { //first level = choose a log segment, return 0 when nothing selected , returns number of data recors copied to RCCL.LOG
	/* format log record:see calcrun OP_LOG
	- first line # with timestamp RTC(if available)  and variables names, separated with \t
	  next lines  timestamp (in 1/100 sec) and variables, separated with \t */

	vector <string> segment ;
	string fname =rc.current_file+".LOG"; //filename of the log file
	if(!rc.lcdptr->fopenread(fname)) {
		rc.sndptr->putFile("2A");  //alarm sound  no log file
		return -1 ;
	}
	string line;
	//fill the segment vector with the descriptor of each segment
	while(rc.lcdptr->fgetline(line)) {
		if(line.substr(0,1) =="#") {
			string x = line;  //replacing tabulator with space
			unsigned found = x.find_first_of("\t");
			while (found!=std::string::npos)
			  {
				x[found]=' ';
				found=x.find_first_of("\t",found+1);
			  }
			segment.push_back(x);
		}
	}
	//------display , browse ----------------------------------------------
	char font ='\x03';
	int font_x,font_y,n_x,n_y;
	const int marge=25; //height of the command line
	int x=0, y=0; //current position
	int selectedline =-1 ;
	int xt,yt;
	int step_x,step_y;//pagination step
	//tabs 12pixels/char
	const int ar_back=0,ar_plusm=80, ar_u=160, ar_d=190 , ar_l=220, ar_r=260;//tabs on the command line, without selected line
	const int  as_n=265;
	while(1) {  //loop on the display
		//printf("x=%d y=%d \n",x,y);
		font_x=Lcd4D::font_x(font);
		n_x= LCDX/font_x;
		step_x=5 ;
		font_y=Lcd4D::font_y(font);
		n_y= (LCDY-marge)/font_y;
		step_y= n_y -2;
		rc.lcdptr->Cls();
		// bottom command line
		rc.lcdptr->Pen('\x00');   //set pen size 0 for a solid rectangle
		rc.lcdptr->Rectangle(0,LCDY-marge,LCDX,LCDY, getColor(BACKGROUND_CMD));
		int yb=LCDY-marge+5; //upper alignement of font 3
		rc.lcdptr->Textcmd(string("BACK"),ar_back,yb);
		if(selectedline<0) { //pagination  arrows
			rc.lcdptr->ArrowCmd(ARROW_UP  ,ar_u,yb);
			rc.lcdptr->ArrowCmd(ARROW_DOWN,ar_d,yb);
			rc.lcdptr->ArrowCmd(ARROW_LEFT,ar_l,yb);
			rc.lcdptr->ArrowCmd(ARROW_RIGHT,ar_r,yb);
			rc.lcdptr->Textcmd(string(" +/- "),ar_plusm,yb);
		} else //commands if a line is selected
			rc.lcdptr->Textcmd(string("VIEW"),as_n,yb);
		//display the text
		for(int  i =0; (i<n_y)&&(i<(int)segment.size()-y); i++) {
			string z;
			z=  segment[i+y];
			if((int)z.length() >x)
				rc.lcdptr->Textg(z.substr(x,n_x),0,i*font_y, font,((i+y==selectedline)?RED: getColor(FOREGROUND_TEXT)),1,1);
		}
		rc.lcdptr->GetTouch(&xt,  &yt);
		if(yt>LCDY-marge) {   // selection within the command line
			if(selectedline<0) {
				int nblines= segment.size();
				if(xt> ar_r) {
					x=min(MAXLINE-n_x ,x+step_x);     //shift right
					continue;
				}
				if(xt> ar_l) {
					x=max(0,x-step_x);    //shift left
					continue;
				}
				if(xt> ar_d) {
					y=min(max(0,nblines - n_y+2),y+step_y);
					continue;
				}
				if(xt> ar_u) {
					y=max(0,y-step_y);
					continue;
				}
				if(xt> ar_plusm) {
					font=(font=='\x02'?'\x03':'\x02');    //change the font size
					continue;
				}
				rc.lcdptr->Cls();
				return -1 ; //back ------------------------------------------
			} else { //commands when a line is selected
				if(xt> as_n) { //------------- a segment is selected-------------------
					rc.lcdptr->Cls(); // confirm
					rc.lcdptr->fopenread(fname);
					//printf("logview1fopenread sel=%d\n ",selectedline);
					int sel =-1;
					int nb=0; //number of records in the selected segment
				    Filey fichier;
                    if (!fichier.fopenwrite("RCCL.LOG")){
						//printf("logview1 error openwrite");
						return 0; //error
					}
					while(rc.lcdptr->fgetline(line)) {
						if(line.substr(0,1) =="#") 
							sel++; // number of this segment
						if (selectedline ==sel){
							nb++;
							line +="\n";
							fichier.fputline(line);
						}
					}
					//printf("logview1 ftoclose\n ");
			        fichier.ftoclose();
					return nb; //selection done, data copied to RCCL.LOG
				}
				//back
				selectedline=-1;
				continue;
			}
		}
		//selection in the text zone above the command line
		selectedline=(yt/font_y)+y; // index of the selected line 0...n
		if ( selectedline> (signed)segment.size())
			selectedline = segment.size();
		continue;
	} //end of the loop
}
void LogView::scaleX(){ //second level dialog to change the X scale and origin and display the graph
	Filey fichier;
	string line;
	fichier.fopenread("RCCL.LOG");
	vector <string> tokens;
	//get the variables
	fichier.fgetline(line);
	Tokenize(line,tokens," \t");
	for (unsigned int i=2;i< tokens.size();i++) 
	   lv->varname.push_back(tokens[i]);
	//get the timestamps 
	lv->segment_debut= 999999999;
	lv->segment_fin=0;
	coordinates = false;
	while (fichier.fgetline(line)){
		tokens.clear();
		Tokenize(line,tokens," \t");
		int ts = atoi(tokens[0].c_str());
		if (ts < segment_debut)
			segment_debut = ts;
		if (ts > segment_fin)
			segment_fin = ts;
	}
	//printf ("logview::scaleX segment_debut=%d segment_fin=%d\n",segment_debut,segment_fin);
	//select initial scaling X
	screen_debut = segment_debut ; //display from the first 
	for (screen_scale_X=0; screen_scale_X< scale_X_size ;screen_scale_X++){
		if( LCDX*(int)scale_X[screen_scale_X]> (segment_fin- segment_debut))
			break; // initial display = the full segment
	}
	screen_var = 0; // display the first variable
	//------display , browse ----------------------------------------------
	const int marge=25; //height of the command line
	int xt,yt;
	//tabs 12pixels/char
	const int ar_back=0, ar_u=60, ar_d=90 , ar_l=130, ar_r=165, ar_w= 200, ar_vu=225,ar_vd=285     ;//tabs on the command line
	while(1) {  //loop on the display
		//printf("x=%d y=%d \n",x,y);
		rc.lcdptr->Cls();
		// bottom command line
		rc.lcdptr->Pen('\x00');   //set pen size 0 for a solid rectangle
		rc.lcdptr->Rectangle(0,LCDY-marge,LCDX,LCDY, getColor(BACKGROUND_CMD));
		int yb=LCDY-marge+5; //upper alignement of font 3
		rc.lcdptr->Textcmd(string("BACK"),ar_back,yb);
        //pagination  arrows
		rc.lcdptr->Textcmd(string(" +"),ar_u,yb);
		rc.lcdptr->Textcmd(string(" -"),ar_d,yb);
		rc.lcdptr->ArrowCmd(ARROW_LEFT,ar_l,yb);
		rc.lcdptr->ArrowCmd(ARROW_RIGHT,ar_r,yb);
		rc.lcdptr->Textcmd(string(" ? "),ar_w,yb);
		rc.lcdptr->Textcmd(string(" Var+"),ar_vu,yb);
		rc.lcdptr->Textcmd(string(" - "),ar_vd,yb);
		rc.lcdptr->Textcmd(varname[screen_var],200,yb-20);
		
		scaleY();
		
		showgraph();//========display the graph===============
		rc.lcdptr->GetTouch(&xt,  &yt); 
		if(yt>LCDY-marge) {   // selection within the command line
			if(xt> ar_vd  && screen_var >0)  {//var --
				screen_var --;
				continue;
			}
			if(xt> ar_vu  && screen_var < (signed)varname.size()) { //var --
				screen_var ++;
				continue;
			}
			if(xt> ar_w) { //ask the coordinates
				if (!coordinates)
					coordinates = true;
				else
					coordinates = false;
				continue;
			}
			if(xt> ar_r) { //shift right
				screen_debut += LCDX*scale_X[screen_scale_X] /3 ;
				continue;
			}
			if(xt> ar_l) { //shift left
				screen_debut -= LCDX*scale_X[screen_scale_X] /3 ;
				continue;
			}
			if(xt> ar_d) {  //zoom -
				if (screen_scale_X > 0)
					screen_scale_X--;
				continue;
			}
			if(xt> ar_u) { //zoom +
				if (screen_scale_X < scale_X_size)
					screen_scale_X++;
				continue;
			}
			fichier.ftoclose();
			rc.lcdptr->Cls();
			return  ; //back ------------------------------------------	
		}
		else{ //selection within the graph
			spellValue(xt);
		}
	} //end of the loop
}
void LogView::spellVar(){//spell name of the selected variable
	rc.sndptr->putFile(varname[screen_var]);  
}
void LogView::spellValue(int xt){
	while (Xval[xt]==0)
		xt++;
	//printf("logview.spellValue  xp=%d  x=%6.2f  y =%6.2f\n",xt, Xval[xt],Yval[xt]);
	// spell xt as xxx,xx seconds 
	rc.sndptr->putValue(Xval[xt],2); 
	rc.sndptr->putFile("1C");
	rc.sndptr->putFile("1S");
	// spell variable
	rc.sndptr->putFile(varname[screen_var]);  
	rc.sndptr->putValue(Yval[xt],2); 
	rc.sndptr->putFile(varname[screen_var]+"X");  
	
	
}
void LogView::scaleY(){ //knowing the X scale , choose a Y scale 
	// find the minimum/maximum Y values displayed within the X scale, for he choosen Var
	Filey fichier;
	string line;
	fichier.fopenread("RCCL.LOG");
	vector <string> tokens;
    Y_min= 200;
     Y_max =-200; 	
	fichier.fgetline(line); //skip the first record ( the variables)
	while (fichier.fgetline(line)){
		//printf("Logview.scaleY line=%s \n",line.c_str());
		tokens.clear();
		Tokenize(line,tokens," \t");
		int ts = atoi(tokens[0].c_str());
		if(ts < screen_debut || ts > screen_debut+ LCDX*scale_X[screen_scale_X])  // out of the range
          continue;
		float v = atof(tokens[ screen_var +1].c_str()); //value
		//printf("Logview.scaleY ts=%d v=%6.2f \n",ts, v);
		if (v<Y_min)
			Y_min = v;
		if(v > Y_max)
			Y_max=v;
	}
	if (Y_max - Y_min < 2)
		Y_max = Y_min+2;
	fichier.ftoclose();
	//printf("Logview.scaleY min=%6.2f max=%6.2f\n",Y_min, Y_max);
	
}
void LogView::showgraph(){
	printf("Logview.showgraph  index=%d scale=%d debut=%d  Ymin=%5.2f  Ymax=%5.2f  var=%d\n ",screen_scale_X, scale_X[screen_scale_X],screen_debut ,Y_min,Y_max,screen_var);
	Filey fichier;
	string line;
	fichier.fopenread("RCCL.LOG");
	vector <string> tokens;
 	fichier.fgetline(line); //skip the first record ( the variables)
	int xpp=-1;
	float yvalue=0;
	int nb=0;
	for (int i =0 ; i< LCDY;i++)
		Yval[i]=0;
	for (int i =0 ; i< LCDX;i++)
		Xval[i]=0;	
	while (fichier.fgetline(line)){
		tokens.clear();
		Tokenize(line,tokens," \t");
		int ts = atoi(tokens[0].c_str()); //timestamp of the log record
		if(ts < screen_debut || ts > screen_debut+  LCDX*scale_X[screen_scale_X])  // ? within the range
          continue;
		float v = atof(tokens[ screen_var +1].c_str()); //value
		int xp = (ts-screen_debut)/scale_X[screen_scale_X]; // X pixel
		//printf("logview.showgraph value=%6.2f ts=%d xp=%d \n",v,ts,xp);
		if (xp==xpp){ //same x
			yvalue += v;
			nb++;
		}else{
			// put on the lcd
			yvalue = yvalue/nb; //mean value
			int yp = (Y_max-yvalue) * (200.0/(Y_max-Y_min));
			//printf("logview.showgraph value=%6.2f  yp=%d  xp=%d \n",yvalue,yp,xpp);
			rc.lcdptr->Point(xpp,  yp  ,RED);
			if(xpp>=0){
				Yval[xpp]=yvalue;
				Xval[xpp]=ts/100.0;
			}
			//reset
			nb=1;
			yvalue =v;
			xpp=xp;
		}

	}
	fichier.ftoclose();
	if(coordinates){
			// values
		char buffer[10];
		float pos= screen_debut/100.0;
		sprintf(buffer,"x=%5.0f",pos);
		rc.lcdptr->Textcmd(buffer,0,100);	
		
		float dur = LCDX*((float) scale_X[screen_scale_X] )/100.0;
		sprintf(buffer,"dur  %5.0f",dur);
		rc.lcdptr->Textcmd(buffer,180,100);
		
		sprintf(buffer,"min=%5.1f",Y_min);
		rc.lcdptr->Textcmd(buffer,50,10);	
		
		sprintf(buffer,"max=%5.1f",Y_max);
		rc.lcdptr->Textcmd(buffer,50,LCDY-50);	
	}
	
}
//-------