/* dialogs related to a calc object: parameters, variables, operations ...
 functions referenced in core.h:
 - param_upd = browse the parameters and edit the value of a parameter
 - var_display =  browse the variables and choose some variables to display "life
    -> calls var_display_sel to display only the selected variables

all the other functions in this file are for internal use  and are not available outside
*/
#include "core.h"
#include <set>
#include <algorithm>
#include <stdlib.h>
extern rccl_data rc; //common data
//various  components  used within this file ---
bool comp_var(short i, short j) { // alpha compare of 2 variable (given by their index)
    return ( rc.calcptr->getVar(i).name < rc.calcptr->getVar(j).name);
}
int valToPixel(int xmin, int xmax,Var* v) {//scale the var to pixel xmin-xmax
    float x = v->value;
    x = (x/TOINT); // value between -100 / +100
    float y= ((xmax+xmin)/2 ) +((x/200) * (xmax-xmin));
    return y;
}
void pixelToVal(int y,int xmin, int xmax,Var* v) {//scale the var to pixel xmin-xmax
    if (y<xmin) {
        v->value= -100*TOINT;
        return;
    }
    if (y>xmax) {
        v->value= 100*TOINT;
        return;
    }
    float x = ((200*float(y))/float(xmax-xmin)) - 100.0*(float(xmax+xmin)/float(xmax-xmin));
    v->value= (int)x*TOINT;
}

void paramEdit(Var * v ) {//edit the value of one selected vvar parameter (called by param_upd)
    rc.lcdptr->Cls();
    char font =  '\x3';
    int marge =25;
    int xt,yt;
    string name= v->name;
	string valueInit= v->valueS(2);
    rc.lcdptr->Pen('\x00');   //set pen size 0 for a solid rectangle
    rc.lcdptr->Rectangle(0,0,LCDX,25, getColor(BACKGROUND_CMD));
    rc.lcdptr->Textg (name,2,2,font,getColor(FOREGROUND_TEXT),1,1);
    rc.lcdptr->Textg ("Value = ",10,40, '\x3',getColor(FOREGROUND_TEXT),1,1);
	rc.lcdptr->Textg (" init = "+valueInit,10,60, '\x3',getColor(FOREGROUND_TEXT),1,1);
    int ar = 250;
    Gra gaup(ar,40);
    gaup.setArrow(ARROW_UP);
    Gra gadown(ar+20,40);
    gadown.setArrow(ARROW_DOWN);
    char buffer[50];
    float inc=1; //increment step or decrement step
    rc.lcdptr->Textg ("inc/dec=",10,80,font,getColor(FOREGROUND_TEXT),1,1);
    Gra stepU(ar,80);
    stepU.setArrow(ARROW_UP);
    Gra stepD(ar+20,80);
    stepD.setArrow(ARROW_DOWN);
    //slider
    int ys= 120;// center of the slider line
    Gr slider(50,ys -6,LCDX-50,ys+6);//the slider
    Gr sliderTouch (45,ys -6,LCDX-45,ys+6);//the slider
    sliderTouch.selectable=true;
    // var selection
    //build the list of the selected variables
    vector<short> varList ;
    set<short>::iterator it;
    int yvar= 150;
    int unsigned selectVar=0;
    Grt selectName(10,yvar,12,1,'\x03'); // name of the selected variable
    Grt selectValue(100,yvar+20,10,1,'\x03'); // name of the selected variable
    Gra selectU(ar,yvar); //select one variable with the arrows
    Gra selectD(ar+20,yvar);
    if (rc.varSelection!=NULL) {
        for (it=rc.varSelection->begin(); it!=rc.varSelection->end(); it++) {
            varList.push_back(*it);
            //printf("paramEdit selected vars %d  %s\n",*it ,rc.calcptr->getVar(*it).name.c_str());
        }
        selectU.setArrow(ARROW_UP);
        selectD.setArrow(ARROW_DOWN);
        sort(varList.begin(),varList.end(),comp_var);//sort /name
        //rc.lcdptr->Textg ("var=",50,yvar,font,FOREGROUND_TEXT,1,1);
    }
    // bottom command line
    rc.lcdptr->Rectangle(0,LCDY-marge,LCDX,LCDY, getColor(BACKGROUND_CMD));
    int yb=LCDY-marge+5; //upper alignement of font 3
    rc.lcdptr->Textcmd(string("BACK"),10,yb);
    bool touchmode=true;// true =press false=moving
    while (1) { // online display of the values
        rc.lcdptr->TextOpaque(true);//needed to erase the former text
        rc.lcdptr->Textg ("        ",100,40, '\x3',getColor(FOREGROUND_TEXT),1,1);
        rc.lcdptr->Textg ("        ",100,80,font,getColor(FOREGROUND_TEXT),1,1);
        rc.lcdptr->TextOpaque(false);
        string valueS= v->valueS(2);
        rc.lcdptr->Textg (valueS,100,40, '\x3',getColor(FOREGROUND_TEXT),1,1); // value of the parameter to update
        sprintf(buffer,"%6.2f",inc);
        rc.lcdptr->Textg (buffer,100,80,font,getColor(FOREGROUND_TEXT),1,1);
        rc.lcdptr->Rectangle(slider.x1,slider.y1,slider.x2,slider.y2,getColor(BACKGROUND_CMD));
        int xs= valToPixel(50,LCDX-50,v);
        rc.lcdptr->Point(xs,ys,RED);
        if (!varList.empty()) {
            Var select = rc.calcptr->getVar(varList[selectVar]);
            selectName.setText(select.name);
            selectValue.setText(select.valueS(2) );
        }
        if (touchmode) {
            if (!rc.lcdptr->GetTouchNB(  &xt,  &yt))
                continue;

            if (selectU.selected(xt,yt) && selectVar < varList.size()-1) {
                selectVar ++;
                continue;
            }
            if (selectD.selected(xt,yt) && selectVar > 0) {
                selectVar --;
                continue;
            }
            if (stepU.selected(xt,yt)&& inc<10 ) {
                inc = inc * 10;
                continue;
            }

            if (stepD.selected(xt,yt)&& inc>0.01 ) {
                inc = inc / 10;
                continue;
            }
            if (gaup.selected(xt,yt)) {
                v->value= valcheck(v->value + inc*TOINT);
                continue;
            }
            if (gadown.selected(xt,yt)) {
                v->value= valcheck(v->value - inc*TOINT);
                continue;
            }
            if (yt > yb) {
                rc.lcdptr->TextOpaque(false);
                return;
            }
            if (sliderTouch.selected(xt,yt)&& abs(xt-xs)<10) { // touch in the slider...change to moving touch
                touchmode=false;
                continue;
            }
        } else {//touchmode=false
            touchmode=! rc.lcdptr->GetTouchSlider(  &xt,  &yt);
            //rc.lcdptr->Point(xt,ys,GREEN);
            pixelToVal(xt,50,LCDX-50,v);
        }
    }
}
// main functions called from outside this file
//---------parameter browse and update the VVAR---------------------------------
dialog param_upd() {
    //build a alphabetical sorted full list = paramList
    vector<short> paramList ;
    for (int i = 0; i<rc.calcptr->getnVar(); i++) {
        Var v = rc.calcptr->getVar(i);
        if (v.getTypeOrig()==PARAM_VVAR)
            paramList.push_back(i);
    }
    sort(paramList.begin(),paramList.end(),comp_var);
    //------display , browse ----------------------------------------------
    char font ='\x03';
    int font_x,font_y,n_x,n_y;
    const int marge=25; //height of the command line
    int x=0, y=0; //current position
    int selectedline =-1 ;
    int xt,yt;
    int step_x,step_y;//pagination step
    //tabs 12pixels/char
    const int ar_back=0,ar_plusm=80, ar_u=160, ar_d=190 , ar_l=220, ar_r=260;//tabs on the command line, without selected line
    const int as_r=220, as_n=265;
    while (1) { //loop on the display
        //printf("x=%d y=%d \n",x,y);
        font_x=Lcd4D::font_x(font);
        n_x= LCDX/font_x;
        step_x=5 ;
        font_y=Lcd4D::font_y(font);
        n_y= (LCDY-marge)/font_y;
        step_y= n_y -2;
        rc.lcdptr->Cls();
        // bottom command line
        rc.lcdptr->Pen('\x00');   //set pen size 0 for a solid rectangle
        rc.lcdptr->Rectangle(0,LCDY-marge,LCDX,LCDY, getColor(BACKGROUND_CMD));
        int yb=LCDY-marge+5; //upper alignement of font 3
        rc.lcdptr->Textcmd(string("BACK"),ar_back,yb);
        if (selectedline<0) {//pagination  arrows
            rc.lcdptr->ArrowCmd(ARROW_UP  ,ar_u,yb);
            rc.lcdptr->ArrowCmd(ARROW_DOWN,ar_d,yb);
            rc.lcdptr->ArrowCmd(ARROW_LEFT,ar_l,yb);
            rc.lcdptr->ArrowCmd(ARROW_RIGHT,ar_r,yb);
            rc.lcdptr->Textcmd(string(" +/- "),ar_plusm,yb);
        } else{ //commands if a line is selected
            rc.lcdptr->Textcmd(string("EDIT"),as_n,yb);
			rc.lcdptr->Textcmd(string("REV"),as_r,yb); //reverse
		}
        //display the text
        for (int  i =0; (i<n_y)&&(i<(int)paramList.size()-y); i++) {
            Var v = rc.calcptr->getVar(paramList[i+y]);
            string z;
            if (v.name.size()<18)
                z=v.name+string(18-v.name.size() ,'\x20')+v.valueS(2);
            else
                z=v.name+"  "+v.valueS(2);
            //---------------
            if ((int)z.length() >x)
                rc.lcdptr->Textg (z.substr(x,n_x),0,i*font_y, font,((i+y==selectedline)?RED: getColor(FOREGROUND_TEXT)),1,1);
        }
        rc.lcdptr->GetTouch(  &xt,  &yt);
        if ( yt>LCDY-marge) { // selection within the command line
            if (selectedline<0) {
                int nblines= paramList.size();
                if (xt> ar_r) {
                    x=min(MAXLINE-n_x ,x+step_x);     //shift right
                    continue;
                }
                if (xt> ar_l) {
                    x=max(0,x-step_x);    //shift left
                    continue;
                }
                if (xt> ar_d) {
                    y=min(max(0,nblines - n_y+2),y+step_y);
                    continue;
                }
                if (xt> ar_u) {
                    y=max(0,y-step_y);
                    continue;
                }
                if (xt> ar_plusm ) {
                    font=(font=='\x02'?'\x03':'\x02');    //change the font size
                    continue;
                }

                rc.lcdptr->Cls();
                return MENU_START ; //back ------------------------------------------
            } else { //commands when a line is selected
                Var * varptr = rc.calcptr->getVarptr(paramList[selectedline]);
                if (xt> as_n) {//EDIT he parameter
                    paramEdit(varptr); //EDIT the object indexed by i
                    selectedline=-1;
                    continue;
                }
				if (xt> as_r) {//reverse the parameter
					//==============================
					varptr->value= -varptr->value ;
	                //selectedline=-1; garder la ligne slectionne
                    continue;				
				}
                //back
                selectedline=-1;
                continue;
            }
        }
        //selection in the text zone above the command line
        selectedline=(yt/font_y)+y; // index of the selected line 0...n
        continue;
    } //end of the loop
}
/* --------display some variables "life"--------------------------
        display a list of all the variables (not the parameters)  and clic to select 1...n variables (selected = displayed as red)
        deselect with clic on a selected variable
        back -> main menu , reload,  go-> display only the selected variables
*/
dialog var_display() {
    //build a alphabetical sorted full list = varList
    vector<short> varList ;
    for (int i = 0; i<rc.calcptr->getnVar(); i++) {
        Var v = rc.calcptr->getVar(i);
        if (v.getTypeOrig()==PARAM_VVAR ||v.getTypeOrig()==PARAM_PVAR)
            continue; // do not display
        varList.push_back(i);
    }
    sort(varList.begin(),varList.end(),comp_var);//sort /name
    // selected lines = set varSelection
    if (rc.varSelection==NULL)
        rc.varSelection=new set<short>;
    char font ='\x03';
    int font_y,n_y;
    const int marge=25; //height of the command line
    int  y=0; //current position
    int xt,yt;//touch coo.
    int step_y;//pagination step
    //tabs 12pixels/char
    const int ar_back=0, ar_u=60, ar_d=90,  as_n=280;//tabs on the command line, without selected line
    font_y=Lcd4D::font_y(font);
    n_y= (LCDY-marge)/font_y;
    while (1) { //loop on the display
        step_y= n_y -2;
        rc.lcdptr->Cls();
        // bottom command line
        rc.lcdptr->Pen('\x00');   //set pen size 0 for a solid rectangle
        rc.lcdptr->Rectangle(0,LCDY-marge,LCDX,LCDY, getColor(BACKGROUND_CMD));
        int yb=LCDY-marge+5; //upper alignement of font 3
        int ylines= yb/font_y;//nb text lines
        rc.lcdptr->Textcmd(string("BACK"),ar_back,yb);
        //pagination  arrows
        rc.lcdptr->ArrowCmd(ARROW_UP  ,ar_u,yb);
        rc.lcdptr->ArrowCmd(ARROW_DOWN,ar_d,yb);
        rc.lcdptr->Textcmd(string("GO"),as_n,yb);
        // text objects (fixed font)
        vector <Grt> tl;
        for (int i =0; i<ylines; i++)
            tl.push_back(Grt(i*font_y));//one Grt object / line
        do {
            set<short>::iterator it;
            for (int i =0; (i<n_y); i++) {
                if (i<(int)varList.size()-y) {
                    Var v = rc.calcptr->getVar(varList[i+y]);
                    tl[i].selectable=true;
                    string z,name;
                    if (v.getTypeOrig()==PARAM_PVAR) {
                        Var vp=rc.calcptr->getVar(v.value-TOVAR);
                        name=vp.valueS(2);
                    } else {
                        name=v.valueS(2);
                    }
                    if (v.name.size()<18)
                        z=v.name+string(18-v.name.size() ,'\x20')+name;
                    else
                        z=v.name.substr(0,17)+" "+name;
                    it=rc.varSelection->find(varList[i+y]); //if selected : display with RED color
                    if (it!=rc.varSelection->end())
                        tl[i].color= RED;
                    tl[i].setText(z);
                } else {//empty lines
                    tl[i].selectable=false;
                    tl[i].erase();
                }
            } //end for loop
        }
        while (rc.lcdptr->GetTouchNB(  &xt,  &yt)==false); //end of the do statement repainting the text waiting for a touch
        // process the touch
        if ( yt>LCDY-marge) { // selection within the command line
            if (xt> as_n)
                return VAR_DISPLAY_SEL;
            if (xt> ar_d) {
                y=min(max(0,int(varList.size()) - n_y+2),y+step_y);
                continue;
            }
            if (xt> ar_u) {
                y=max(0,y-step_y);
                continue;
            }
            //back
            rc.lcdptr->Cls();
            return MENU_START ; //back ------------------------------------------
        }
        //selection in the text zone above the command line: update varSelection set
        for (int i =0; (i<n_y); i++) {
            if (tl[i].selected(xt,yt)) {
                set<short>::iterator it;
                it=rc.varSelection->find(varList[i+y]);
                if (it!=rc.varSelection->end())//already selected , now deselect
                    rc.varSelection->erase(it);
                else
                    rc.varSelection->insert(varList[i+y]);
                break;
            }
        }
        continue;
    } //end of the loop
}
dialog var_display_sel() {
    //printf("var_display_sel\n");
    //build the selection list = varList
    vector<short> varList ;
    set<short>::iterator it;
    for (it=rc.varSelection->begin(); it!=rc.varSelection->end(); it++) {
        varList.push_back(*it);
        //printf("var_display_sel %d  %s\n",*it ,rc.calcptr->getVar(*it).name.c_str());
    }
    sort(varList.begin(),varList.end(),comp_var);//sort /name
    char font ='\x03';
    int font_y,n_y;
    const int marge=25; //height of the command line
    int xt,yt;//touch coo.
    //tabs 12pixels/char
    const int ar_back=0,ar_graph =LCDX-70;//tabs on the command line, without selected line
    font_y=Lcd4D::font_y(font);
    n_y= (LCDY-marge)/font_y;
    while (1) { //loop on the display
        rc.lcdptr->Cls();
        // bottom command line
        rc.lcdptr->Pen('\x00');   //set pen size 0 for a solid rectangle
        rc.lcdptr->Rectangle(0,LCDY-marge,LCDX,LCDY, getColor(BACKGROUND_CMD));
        int yb=LCDY-marge+5; //upper alignement of font 3
        int ylines= yb/font_y;//nb text lines
        rc.lcdptr->Textcmd(string("BACK"),ar_back,yb);
        if (varList.size() <= GRAPH_N)
            rc.lcdptr->Textcmd(string("GRAPH"),ar_graph,yb);
        // text objects (fixed font)
        vector <Grt> tl;
        for (int i =0; i<ylines; i++)
            tl.push_back(Grt(i*font_y));//one Grt object / line
        do {
            set<short>::iterator it;
            for (int i =0; (i<n_y); i++) {
                if (i<(int)varList.size()) {
                    Var v = rc.calcptr->getVar(varList[i]);
                    string z;
                    if (v.name.size()<18)
                        z=v.name+string(18-v.name.size() ,'\x20')+v.valueS(2);
                    else
                        z=v.name.substr(0,17)+" "+v.valueS(2);
                    tl[i].setText(z);
                } else {//empty lines
                    tl[i].erase();
                }
            } //end for loop
        }
        while (rc.lcdptr->GetTouchNB(  &xt,  &yt)==false); //end of the do statement repainting the text, waiting for a touch
        // process the touch
        if ( yt>LCDY-marge) { // selection within the command line
            if (xt> ar_graph  &&  varList.size() <= GRAPH_N) {
                return VAR_GRAPH;
            }
            //back
            rc.lcdptr->Cls();
            return VAR_DISPLAY ; //back ------------------------------------------
        }
    }
}
// -----------graphical display of some variables----------------------------------------
dialog var_graph() {// display x var along the x axis and all other rc.varSelection along y axis
    float toPix = float(LCDY)/float(250) ; //scale value (-125..125) to pixel
    int x0 = LCDY/2; //origin x=0 y=0
    int y0 = LCDY/2;
    int xt,yt;//touch coo.
    int delta = int(100*toPix) ;
    int  font_y=Lcd4D::font_y('\x03');
    //printf("var_graph begin  %i , %i , %i   \n",x0,y0,delta);
    int x =-1; //index of the x var
    int colorxy = getColorx() ; //axis
    while (1) { //clear the graph
        rc.lcdptr->Cls();
        vector <Grt> vy;// variables names - select one as x and clear
        set<short>::iterator it;
        int i=0;
        for (it=rc.varSelection->begin(); it!=rc.varSelection->end(); it++)  {
            Var v =    rc.calcptr->getVar(*it);
            //printf("y [%i] = %s   \n",i,v.name.c_str());
            vy.push_back(Grt(230,15+2*i*font_y,7,1,'\x03'));//one Grt object / line
            vy[i].color= getColory(i) ;
            if (i!=x)
                vy[i].setText(v.name);
            else
                vy[i].setText("x="+v.name);
            vy[i].selectable=true;
            i++;
        }
        Grt back(230, LCDY-25,8,1,'\x03');
        back.color=getColor(FOREGROUND_CMD);
        back.setText("BACK");
        back.selectable=true;
        rc.lcdptr->Pen('\x01');
        rc.lcdptr->Rectangle(x0 - delta, y0- delta,x0 + delta, y0+ delta,colorxy);
        rc.lcdptr->Line  (x0 - delta, y0           ,x0 + delta, y0            ,colorxy);
        rc.lcdptr->Line  (x0            , y0- delta,x0             , y0+ delta,colorxy);

        do {
            int xp,yp; //pixel
            xp=0;
            if (x>=0) { //paint the points
                float val[GRAPH_N];
                int i=0;
                for (it=rc.varSelection->begin(); it!=rc.varSelection->end(); it++)  {
                    Var v =rc.calcptr->getVar(*it);
                    val[i] = float(v.value)/TOINT*toPix; //value in pixels from origin
                    if (i==x)
                        xp=val[i]+x0;
                    i++;
                }
                for (int i =0; i< (int)rc.varSelection->size() 	; i++) {
                    if (i!=x) {
                        yp= val[i]+y0;
                        rc.lcdptr->Point(xp,yp,getColory(i));
                    }
                }
            }
        }
        while (rc.lcdptr->GetTouchNB(  &xt,  &yt)==false); //end of the do statement repainting the graph, waiting for a touch
        if (back.selected(xt,yt)) {
            return VAR_DISPLAY_SEL;
        }
        for (int i =0; i< (int)rc.varSelection->size() 	; i++) {
            if (vy[i].selected(xt,yt)) {
                x=i; //new x variable
                //printf ("var_graph x=%i\n",x);
            }
        }
    }
    //return VAR_DISPLAY_SEL;
}
//---------parameter browse and assign the PVAR to a dtrim---------------------------------
dialog param_assign() {
    //build a alphabetical sorted full list = paramList
    vector<short> paramList ;
    for (int i = 0; i<rc.calcptr->getnVar(); i++) {
        Var v = rc.calcptr->getVar(i);
        if (v.getTypeOrig()==PARAM_PVAR)
            paramList.push_back(i);
    }
    sort(paramList.begin(),paramList.end(),comp_var);
    //------display , browse ----------------------------------------------
    char font ='\x03';
    int font_x,font_y,n_x,n_y;
    const int marge=25; //height of the command line
    int x=0, y=0; //current position
    int selectedline =-1 ;
    int xt,yt;
    int step_x,step_y;//pagination step
    //tabs 12pixels/char
    const int ar_back=0,ar_plusm=80, ar_u=160, ar_d=190 , ar_l=220, ar_r=260;//tabs on the command line, without selected line
    const int as_1=100 , as_2=140 , as_3=180  ,as_4= 220;
    while (1) { //loop on the display
        //printf("x=%d y=%d \n",x,y);
        font_x=Lcd4D::font_x(font);
        n_x= LCDX/font_x;
        step_x=5 ;
        font_y=Lcd4D::font_y(font);
        n_y= (LCDY-marge)/font_y;
        step_y= n_y -2;
        rc.lcdptr->Cls();
        // bottom command line
        rc.lcdptr->Pen('\x00');   //set pen size 0 for a solid rectangle
        rc.lcdptr->Rectangle(0,LCDY-marge,LCDX,LCDY, getColor(BACKGROUND_CMD));
        int yb=LCDY-marge+5; //upper alignement of font 3
        rc.lcdptr->Textcmd(string("BACK"),ar_back,yb);
        if (selectedline<0) {//pagination  arrows
            rc.lcdptr->ArrowCmd(ARROW_UP  ,ar_u,yb);
            rc.lcdptr->ArrowCmd(ARROW_DOWN,ar_d,yb);
            rc.lcdptr->ArrowCmd(ARROW_LEFT,ar_l,yb);
            rc.lcdptr->ArrowCmd(ARROW_RIGHT,ar_r,yb);
            rc.lcdptr->Textcmd(string(" +/- "),ar_plusm,yb);
        } else { //commands if a line is selected
            rc.lcdptr->Textcmd(string("1"),as_1,yb);
            rc.lcdptr->Textcmd(string("2"),as_2,yb);
            rc.lcdptr->Textcmd(string("3"),as_3,yb);
            rc.lcdptr->Textcmd(string("4"),as_4,yb);
        }
        //display the text
        for (int i =0; (i<n_y)&&(i<(int)paramList.size()-y); i++) {
            Var v = rc.calcptr->getVar(paramList[i+y]);
            char dtnr=' ';
            char dt[4] ={'1','2','3','4'};
            for (int j=0; j<=3; j++) {
                if  (rc.calcptr->dtrimArray[j].indexPvarTuneParam == paramList[i+y])
                    dtnr=dt[j];
            }
            string z;
            if (v.name.size()<18)
                z=v.name+string(18-v.name.size() ,'\x20')+ dtnr;
            else
                z=v.name+"  "+dtnr;
            //---------------
            if ((int)z.length() >x)
                rc.lcdptr->Textg (z.substr(x,n_x),0,i*font_y, font,((i+y==selectedline)?RED: getColor(FOREGROUND_TEXT)),1,1);
        }
        rc.lcdptr->GetTouch(  &xt,  &yt);
        if ( yt>LCDY-marge) { // selection within the command line
            if (selectedline<0) {
                int nblines= paramList.size();
                if (xt> ar_r) {
                    x=min(MAXLINE-n_x ,x+step_x);     //shift right
                    continue;
                }
                if (xt> ar_l) {
                    x=max(0,x-step_x);    //shift left
                    continue;
                }
                if (xt> ar_d) {
                    y=min(max(0,nblines - n_y+2),y+step_y);
                    continue;
                }
                if (xt> ar_u) {
                    y=max(0,y-step_y);
                    continue;
                }
                if (xt> ar_plusm ) {
                    font=(font=='\x02'?'\x03':'\x02');    //change the font size
                    continue;
                }

                rc.lcdptr->Cls();
                return MENU_START ; //back ------------------------------------------
            } else { //commands when a line is selected
                //Var * varptr = rc.calcptr->getVarptr(paramList[selectedline]);
                if (xt> as_4) {//ASSIGN the parameter to a dtrim
                    rc.calcptr->dtrimArray[3].indexPvarTuneParam=paramList[selectedline];
                    selectedline=-1;
                    continue;
                }
                if (xt> as_3) {//ASSIGN the parameter to a dtrim
                    rc.calcptr->dtrimArray[2].indexPvarTuneParam=paramList[selectedline];
                    selectedline=-1;
                    continue;
                }
                if (xt> as_2) {//ASSIGN the parameter to a dtrim
                    rc.calcptr->dtrimArray[1].indexPvarTuneParam=paramList[selectedline];
                    selectedline=-1;
                    continue;
                }
                if (xt> as_1) {//ASSIGN the parameter to a dtrim
                    printf("menu_calc assign param-> dtrim1 = %i\n",paramList[selectedline]);
                    rc.calcptr->dtrimArray[0].indexPvarTuneParam=paramList[selectedline];
                    selectedline=-1;
                    continue;
                }
                //back
                selectedline=-1;
                continue;
            }
        }
        //selection in the text zone above the command line
        selectedline=(yt/font_y)+y; // index of the selected line 0...n
        for (int j=0; j<3; j++) {
            if  (rc.calcptr->dtrimArray[j].indexPvarTuneParam == paramList[selectedline])
                rc.calcptr->dtrimArray[j].indexPvarTuneParam =-1; //deassign
        }
        continue;
    } //end of the loop
}
bool Calc::param_save() { //save the parameter file for the current model
    if (!ready_to_calc)
		return false;
	Filey fichier;
   //printf("Menu_calc param_save filename=%s\n",modelname.c_str());
    if (!fichier.fopenwrite(modelname+".mdp")) 
        return false;
    //build a alphabetical sorted full list = paramList
    vector<short> paramList ;
    for (int i = 0; i<rc.calcptr->getnVar(); i++) {
        Var v = rc.calcptr->getVar(i);
        if (v.getTypeOrig()==PARAM_VVAR)
            paramList.push_back(i);
    }
    sort(paramList.begin(),paramList.end(),comp_var);
    for (int i =0; i<(int)paramList.size(); i++) {
        Var v = rc.calcptr->getVar(paramList[i]);
		if( v.name[0]== '@')
			continue; //skip this global parameter
        string z;
        z="4  "+v.valueS(2)+"  "+v.name+"\n";
        fichier.fputline(z);
        //printf("   param_save    >%s",z.c_str());
    }
    fichier.ftoclose();
	//printf("Menu_calc param_save_glob ");
	if (!fichier.fopenwrite("rccl.mdp")) 
        return false;
    for (int i =0; i<(int)paramList.size(); i++) {
        Var v = rc.calcptr->getVar(paramList[i]);
		if( v.name[0]!= '@')
			continue; //skip this model parameter
        string z;
        z="4  "+v.valueS(2)+"  "+v.name+"\n";
        fichier.fputline(z);
        //printf("   param_save    >%s",z.c_str());
    }
    fichier.ftoclose();	
	
	
    return true;
}

