#include "core.h"
#include <algorithm>
/* transmitter sound functions usig the 4D sound and sd card
    for any langage
 */
extern rccl_data rc; //common data
DigitalOut soundLED(LED2); // on when mute
void Lcd4D::play(string filename) {  //play a sound file with immediate return
	//printf("Lcd4D play =%s \n",filename.c_str());
	char cmd[4];
	cmd[0]='\x40';
	cmd[1]='\x6C';
	cmd[2]='\x01' ;
	CmdS(cmd,3); //01= return immediatly  00=return when playing complete
	putString(filename); //returns NAK if file not found
	Ack();
}
int Lcd4D::getSoundLen(string filename) {//get the size  and returns the duration of a canonical wav file in msec
	char cmd[3];
	cmd[0]='\x40';
	cmd[1]='\x61';
	cmd[2]='\x01' ;
	CmdS(cmd,3);
	putString(filename); //send the string and append with \0
	unsigned char answer[4];
	for (int i=0; i<4; i++) {
		answer[i]=  getcB(); // 4 bytes bg endian
	}
	putc(NAK); //stop with read !
	getc(); //receive the ACK answer
	/* answer helpdesk :If you send NAK to the filelength you should then receive an ACK and the display should be ready for the next command  */
	int len =  answer[1] *256*256 +answer[2]*256 + answer[3];
	int duration = (len-44-8)/32; // wav file 16khz 16 bits mono
	return duration; //msec
}
void Lcd4D::setVolume() {
	char cmd[2];
	cmd[0]= '\x76';
	cmd[1] = (char) calibrate.volume;
	Cmd(cmd,2); //background !! 2,1 sec
}
void Lcd4D::mute(bool m) { //control the mute pin
	char cmd[3];
	cmd[0]='\x79';
	cmd[1]='\x07';
	if (m)
		cmd[2]='\x01' ;
	else
		cmd[2]='\x00' ;
	Cmd(cmd,3);

}

/*void Lcd4D::sleepSound() {
    char cmd[3];
    cmd[0]='\x5A';
    cmd[1]='\x40'; //cut sound
    cmd[2]='\x01' ; //max 1 sec
    CmdS(cmd,3); // !! no ACK
}*/

Snd::Snd() {
	//printf("Snd constructor\n");
	nextTick=0;
	muteOn=false;
}
//-------put a sound in the queue (internal fonctions)-----------------------------------------------
void Snd::xput(const string filename) {
	if (sndq.size()< SNDQMAX)
		sndq.push(filename); //put this filename in the queue (without the fixed suffix .WAV
}
void Snd::nput(int i) {  //convert a number  i  in  a sound file
	char buffer[10];
	sprintf(buffer ,"%i",i); //itoa not supported by mbed compiler
	if (sndq.size()< SNDQMAX)
		sndq.push(buffer); //put this filename in the queue (without the fixed suffix .WAV
}
void Snd::zput(int i) {  //insert a silent pause  i msec
	char buffer[10];
	sprintf(buffer ,"?%i",i);
	//itoa(i, buffer,10); itoa not suppoted by mbed compiler
	if (sndq.size()< SNDQMAX)
		sndq.push(buffer); //put this string in the queue (this is not a filename !)
}
//--------- output a sound ( in rccl_task ) ------------------------------------
void Snd::getNext(int tick) {
	soundLED = muteOn;
	if (sndq.empty() && !muteOn && tick > (nextTick+100)) {
		//rc.lcdptr->sleepSound( ); //ne fonctionne pas
		muteOn=true;
		rc.lcdptr->mute(muteOn);
		return;
	}

	if (sndq.empty() || tick < nextTick)
		return;

	muteOn=false;
	rc.lcdptr->mute(muteOn);
	string x = sndq.front();
	if (x[0]== '?') { // this string defines the length of a silence
		string y = x.substr(1,20);
		nextTick += atoi(y.c_str())/10;
		sndq.pop();
		return;
	}
	string f = x+".WAV";
	rc.lcdptr->play(f);
	int len = rc.lcdptr->getSoundLen(f)/10;
	nextTick =tick + len;
	if (len==0) {
#ifdef DEBUGP
		printf("Snd:: play :file not found %s\n",f.c_str()); //this is not an error -
#endif
		//rc.calcptr->lcd[7]= "err:"+f ;
	}

	sndq.pop();
}
//---------- put a number / sound file in the queue -----------------------------------------
void Snd::putFile(const string filename) {
	//printf ("Snd::putFile %s\n",filename.c_str());
	xput(filename);
}
void Snd::putValue(float value,int format) {// spell this variable and put the sounds in the queue format = 0,1,2 decimals, 9=chrono , 8=vario
	//printf("Snd::putValue %f format=%i\n",value,format);
	if (format==8) {  //output  vario: sons de tonalit variable ; la variable reue est la frquence /100
		// les fichiers sont VVfff.WAV, pour fff de 400  2200
		if (sndq.size()>0)
			return; // vario sound has a lower priority than other sounds and may not be delayed by the queue 20120727
		// find the tone
		int x = floor((value *100/40)+0.5)*40 ;  // tone 0=400Hz  1=440Hz ...
		if (x <400 || x > 2200)
			return;
		char str[12] ;
		sprintf(str,"VV%d",x);
		//printf (" vario sound= %s\n", str);
		xput(str);
	}
	xput("1S"); //a short silence to activate slowly the audio amplifier
	if (value <0) {
		xput("1M"); //"moins"
		value=-value;
	}
	if (format==0) {  //output without decimal
		float valueR = floor(value+0.5);
		int entier = int(valueR) ;
		conv3(entier);
	}
	if (format==1) {  //output with 1 decimal
		int valueR = int( floor((value*10.0)+0.5));
		int entier = valueR/10;
		conv3(entier);
		int fract  =valueR - entier*10;
		if(fract>0){
			xput("1V");
			conv1(fract);
		}
	}
	if (format==2) {  //output with 2 decimal
		int valueR = int(floor(value*100.0+0.5));
		int entier = valueR/100 ;
		conv3(entier);
		int fract  =valueR - entier*100;
		if(fract>0){
			xput("1V");
			conv2(fract);
		}
	}
	if (format==9) {  //output chrono timer units=  minute
		// see lcdTimer in calcrun
		int n,sec,min,heure,x;
		x = value*60*100;
		n = x/100; // n is a number of seconds
		sec = n % 60;
		n -= sec;
		n /=60;
		min = n % 60;
		n -= min;
		heure = n/60;
		//printf("Snd::putValue value= %d h =%d min=%d sec=%d\n",x,heure,min,sec);
		if (heure > 0) {
			conv3(heure);
			xput("1A");
		}
		if (min > 0) {
			conv3(min);
			xput("1B");
		}
		conv3(sec);
		xput("1C");
	}

}
//==============menu  for the sound files on SD 4D=========================
dialog menu_sound() { //sound and the files on the SD card
	//printf("menu_sound\n");
	rc.lcdptr->soundMove("WAV"); //tranfer the .w* files from the local file system to the 4D sd (see lcdhw)
	rc.lcdptr->soundMove("GCI");
	//rc.lcdptr->soundMove("DAT");
	//build a alphabetical sorted full list = filelist
	vector<string> dirlist; // directory with the full filename of the files
	dirlist =  rc.lcdptr->fgetlist();
	char font ='\x03';
	sort(dirlist.begin(),dirlist.end());
	//------display , browse ----------------------------------------------
	int font_y,n_y;
	const int marge=25; //height of the command line
	int  y=0; //current position
	int selectedline =-1 ;
	int xt,yt;
	int step_y;//pagination step
	//tabs 12pixels/char
	const int ar_back=0, ar_u=80, ar_d=120, ar_del=150, ar_play=220 ;//tabs on the command line, without selected line
	while (1) { //loop on the display
		//printf("x=%d y=%d \n",x,y);
		//font_x=Lcd4D::font_x(font);
		//n_x= LCDX/font_x;
		// step_x=5 ;
		font_y=Lcd4D::font_y(font);
		n_y= (LCDY-marge)/font_y;
		step_y= n_y -2;
		rc.lcdptr->Cls();
		// bottom command line
		rc.lcdptr->Pen('\x00');   //set pen size 0 for a solid rectangle
		rc.lcdptr->Rectangle(0,LCDY-marge,LCDX,LCDY, getColor(BACKGROUND_CMD));
		int yb=LCDY-marge+5; //upper alignement of font 3
		rc.lcdptr->Textcmd(string("BACK"),ar_back,yb);
		if (selectedline<0) {//pagination  arrows
			rc.lcdptr->ArrowCmd(ARROW_UP  ,ar_u,yb);
			rc.lcdptr->ArrowCmd(ARROW_DOWN,ar_d,yb);
		} else { //commands if a line is selected
			rc.lcdptr->Textcmd(string(" Del "),ar_del,yb);
			rc.lcdptr->Textcmd(string("Play"),ar_play,yb);
		}
		//display the text
		for (int  i =0; (i<n_y)&&(i<(int)dirlist.size()-y); i++) {
			rc.lcdptr->Textg(dirlist[i+y]     ,0,i*font_y, font,((i+y==selectedline)?RED: getColor(FOREGROUND_TEXT)),1,1);
		}
		rc.lcdptr->GetTouch(&xt,  &yt);
		if (yt>LCDY-marge) {  // selection within the command line
			if (selectedline<0) {
				int nblines= dirlist.size();
				if (xt> ar_d) {
					y=min(max(0,nblines - n_y+2),y+step_y);
					continue;
				}
				if (xt> ar_u) {
					y=max(0,y-step_y);
					continue;
				}
				rc.lcdptr->Cls();
				return MENU_START ; //back ------------------------------------------
			} else { //commands when a line is selected
				string fname= dirlist[selectedline];

				if (xt> ar_play) {//play the file
					vector<string> token;
					Tokenize(fname,token,".");
					if (token[1].compare("WAV")==0)
						rc.sndptr->putFile(token[0]);
					if (token[1].compare("GCI")==0) {
						rc.lcdptr->Image(token[0]);
						rc.lcdptr->GetTouch(&xt,  &yt);
					}
					/*if (token[1].compare("LOG")==0) {
						logger_export(fname); //copy the file
						dirlist.erase(dirlist.begin()+selectedline);  //erase from the list
					}*/
					selectedline=-1;
					continue;
				}
				if (xt> ar_del) {//delete the file
					dirlist.erase(dirlist.begin()+selectedline);  //erase from the list
					rc.lcdptr->fopenwrite(fname);//erase from the SDcard
					selectedline=-1;
					continue;
				}

				//back
				selectedline=-1;
				continue;
			}
		}
		//selection in the text zone above the command line
		selectedline=(yt/font_y)+y; // index of the selected line 0...n
		continue;
	} //end of the loop
}