#include "core.h"
#include <stdio.h>
#include<stdlib.h>
//#define DEBUGJETI_CONNECT // special debug of the connection
//#define DEBUGJETI_SENSOR // special debug of the sensor acquisition
//#define DEBUGJETI_READMAP
//#define DEBUGJETI     //simple message in the link
/* jeti sensors explorer
The descriptor file
	the file explorer.txt is a decription of the  nodes in the menu
	for each node
	1) a line with $ followed with 5 tokens
	- node identification nummer
	- the 4 directions up, right, down, left
	each numerical value xx is a node nummer ; if Lxx it means "long touch" needed (repeat the instruction as long as needed)
	some are alphabetic and need  a replacement. this is used for the expander configuration
	2) line 1 and line 2 are 2*16 characters. The normal characters are the signature of the message.
	The special characters are
	@ = any character
	& = any character for the signature, but this defines a field for the data acquisition

	3) lines with a # in first position or empty lines are not processed

Expander configuration is a array exp [5] filled automatically by explorer_search()
- no expander exp[0]=node connected to Mx  others =0
- expander exp[0]=21 other = node connected or 0




 */
extern rccl_data rc; //common data
#define JETI_NODES  100
struct node {
	short id ; //identification of this node (1....255)
	short next [4] ; //ident. of the next node, or 0 when there is no connection  ; index 0=up, 1=right, 2=down, 3=up
	short exp[4] ; // normal value  0, 1= this direction has been used  +1=in +2=out
	char longtouch[4] ; //space for a normal unique touch
	char line1[17]; //\0 terminated
	char line2[17];
};
//the data are static reserved outside the main memory
#ifdef RAM_EXT
#ifdef COMPILER_MBED
node node_array[JETI_NODES] __attribute__((section("AHBSRAM1")));
#endif
#ifdef COMPILER_CS
node node_array[JETI_NODES] __attribute__((section(".usb_ram")));
#endif
#ifdef COMPILER_ARM
node node_array[JETI_NODES] __attribute__((section("AHBSRAM1")));
#endif
#else
node node_array[JETI_NODES];
#endif
int rx_count_sensor;
bool is(string x) { //return true if string x in the first sensorId line
	return (rc.jetiptr->get1().find(x) != string::npos);
}
bool is2(string x) { //return true if string x in the 2nd sensorId line
	return (rc.jetiptr->get2().find(x) != string::npos);
}
void setLcd(int i, string txt) {
	rc.calcptr->lcd[i+2]=txt;
	rc.calcptr->lcd_protect[i+2]=GetTicker()+1000;
}

void Jeti::explorer_load() {//load the descriptors explorer , after the load of the model .mdx file
#ifdef DEBUGJETI_CONNECT
	printf("explorer load \n");
#endif
	Filey fichier;
	string line;
	string spaces = string(16 ,' ');//16 spaces
	fichier.fopenread("RCCLEXP.txt");
	for (int iexp =0; iexp<JETI_NODES; iexp++) { //reset the table
		node_array[iexp].id = 0;
		for (int i = 0; i<4; i++) {
			node_array[iexp].next[i]=0;
			node_array[iexp].exp[i]=0;
			node_array[iexp].longtouch[i]=0x20;
		}
		for (int i = 0; i<15; i++) {
			node_array[iexp].line1[i]=0x20;
			node_array[iexp].line2[i]=0x20;
		}
		node_array[iexp].line1[16]='\0';
		node_array[iexp].line2[16]='\0';
	}
	int iexp=0; // current index in the node_array
	while (fichier.fgetline(line) && iexp< JETI_NODES) {
		//printf("explorer load %s\n",line.c_str());
		if (line.find('#')==0  ||line.compare(spaces)==0) {
			//printf("Explorer skip %s\n",line.c_str());
			continue;
		}
		//read the 3 lines
		if (line.find('$')==0) {
			vector<string> x =cleanline(line);
			if (x.size() != 6) {
				printf("Explorer load error number of token %d  > %s\n",x.size(),line.c_str());
				break;
			}
			node_array[iexp].id = atoi(x[1].c_str());
			for (int i = 0; i<4; i++) {
				string z;
				if (x[2+i].find('L')==0) {
					z=x[2+i].substr(1,30);
					node_array[iexp].longtouch[i]='L';
				} else
					z=x[2+i];
				node_array[iexp].next[i]=atoi(z.c_str());
			}
			//---line 1
			fichier.fgetline(line);
			strcpy(node_array[iexp].line1 , line.c_str());
			//---line 2
			fichier.fgetline(line);
			strcpy(node_array[iexp].line2 , line.c_str());
			iexp++;
		}
	}
	fichier.ftoclose();
	connect_status=1; // to explore the sensors
	#ifdef DEBUGJETI_CONNECT
	rc.jetiptr->explorer_dump();
   #endif
}
bool Jeti::explorer_exp(short node, string line) {
	//printf ("JETIX %s\n",line.c_str());
	short iexp= node_index(node);
	strcpy(node_array[iexp].line2 , line.c_str());
	//printf ("JETIX update node %d\n",node);
	//explorer_dump();
	return true;
}
void Jeti::explorer_dump() { //debug dump of the explorer data
	for (int iexp =0; iexp<JETI_NODES; iexp++) { //reset the table
		if (node_array[iexp].id ==0)
			continue;
		printf("explorer_dump iexp= %d node id=%d",iexp, node_array[iexp].id);
		for (int i = 0; i<4; i++) {
			printf("-----%d  %c   ",node_array[iexp].next[i],node_array[iexp].longtouch[i]);
		}
		printf("\n");
		printf("Line 1=%s\n",node_array[iexp].line1);
		printf("Line 2=%s\n\n",node_array[iexp].line2);
	}
}
/*bool Jeti::isnode(int x) {//return true if the current position is the node x
	int iexp=-1;
	for (int i=0; i<JETI_NODES; i++) { //find the node x
		if (node_array[i].id == x) {
			iexp=i;
			break;
		}
	}
	//printf("isnode  id=%d iexp=%d \n",x, iexp);
	if (iexp==-1) return false; //!!! node x is not existing
	string s;
	s = get1();
	for (int i =0; i<15; i++) {
		char z;
		z=node_array[iexp].line1[i];
		if (z=='&' || z=='@')
			continue;
		if (z<=' ' &&  s[i]<= ' ') //sometimes strange char in the Jeti display !!!
			continue;
		if (z!=s[i]) {
			//printf("false line 1 at %d %c %c\n",i,z,s[i]);
			return false;

		}
	}
	s = get2();
	for (int i =0; i<15; i++) {
		char z;
		z=node_array[iexp].line2[i];
		if (z=='&' || z=='@')
			continue;
		if (z<=' ' &&  s[i]<= ' ')
			continue;
		if (z!=s[i]) {
			//printf("false line 2 at %d %c %c\n",i,z,s[i]);
			return false;

		}
	}
	return true;
}*/
bool Jeti::isnode(int x) {//return true if the current position is the node x
	// version qui accepte plusieurs lignes pour le meme noeud (exemple PUI 30 definit autrement que MUI 30 EX)
	for (int iexp=0; iexp<JETI_NODES; iexp++) { //find the node x
		if (node_array[iexp].id != x) 
			continue;
		// node id ==x , control the descriptor line 1
		bool err = false; 
		string s;
		s = get1();
		for (int i =0; i<15; i++) {
			char z;
			z=node_array[iexp].line1[i];
			if (z=='&' || z=='@')
				continue;
			if (z<=' ' &&  s[i]<= ' ') //sometimes strange char in the Jeti display !!!
				continue;
			if (z!=s[i]) 
				err=true;
		}
		s = get2(); // control the descriptor line 1
		for (int i =0; i<15; i++) {
			char z;
			z=node_array[iexp].line2[i];
			if (z=='&' || z=='@')
				continue;
			if (z<=' ' &&  s[i]<= ' ')
				continue;
			if (z!=s[i]) 
				err=true;
		}
		if (!err)
			return true;
	}
	return false;
}
int distance (int x, int y){ //x y are >0
	if (x>y)
		return x-y;
	else
		return y-x;
}

int Jeti::node_find() {//returns the node id for the current display; if not unique, choose the node near "from"
	int answer=0;
	int node_id;
	for (int i=0; i<JETI_NODES; i++) { //find the node x
		if (node_array[i].id ==0)
			continue;
		if (!isnode(node_array[i].id)) 
			continue;
		node_id=node_array[i].id;
		if (answer==0)
			answer=node_id;
		else {
			if (distance(answer,from) > distance(node_id,from))
				answer=node_id;
		}
	}
	return answer;
}
int Jeti::node_index(int nodeId) {//find the index of the node with nodeID
	for (int i=0; i<JETI_NODES; i++) { //find the node x
		if (node_array[i].id == nodeId) {
			return i;
		}
	}
	return 0;
}
void Jeti::link(int from, int direction, int to) {//link from node Id , direction => node to
	node_array[node_index(from)].next[direction]=to;
	node_array[node_index(from)].longtouch[direction]=0x20;
#ifdef DEBUGJETI
	printf("Explorer link %d (%d) -> %d\n",from,direction,to);
#endif
}
void Jeti::linkL(int from, int direction, int to) {//link from node Id , direction => node to
	node_array[node_index(from)].next[direction]=to;
	node_array[node_index(from)].longtouch[direction]='L';
#ifdef DEBUGJETI
	printf("Explorer link LONG %d (%d) -> %d\n",from,direction,to);
#endif
}
void Jeti::explorer_connect() {//search the connected sensors and update the explorer array runs evey 10 msec ,returns the new status
	/*
	#ifdef DEBUGJETI_CONNECT
		//if (connect_status !=0) {
		if (connect_status >=5) {
			printf("connect_status=%d\n",connect_status);
			printf("%s \n%s\n1234567890123456789012\n",  get1().c_str(), get2().c_str());
		}
	#endif
		*/
	if (connect_status ==1) {//initialization
		explorer_load();
		connect_status=2;
		iop_busy=-1;
		iop_last=-1;
		return;
	}
	if (connect_status ==0) //connected
	return;
#ifdef  DEBUGJETI_CONNECT
	char buffer[20];
	sprintf(buffer,"%d",connect_status);
	setLcd(5,buffer);
	sprintf(buffer,"%s",get1().c_str());
	rc.calcptr->lcd[1] = buffer;
#endif

	if (jeti_box_busy)
		return;
	if (connect_status ==2) {//go to top
		if (is("Tx ->")||is("Rx ->")||is("<- Mx")) {
			connect_status =3;
		} else {
			putJeti(JETI_UP);
			return;
		}
	}
	if (connect_status ==3) {//check Rx loop with message if not connected
		if (is("Tx ->")) {
			putJeti(JETI_RIGHT);
			return;
		}
		if (is("<- Mx")) {
			putJeti(JETI_LEFT);
			return;
		}
		if (is("Rx ->")) {
			if (is2("v")) {//ok rx connected, goto Mx
				setLcd(5,"Rx ok");
				if (rx_count < iop_rx_count+100)  //let stabilize
					return;
				connect_status =4;
				iop_rx_count = rx_count;
				putJeti(JETI_RIGHT);
				return;
			}
			//Rx but not yet connected:loop
			iop_rx_count = rx_count;
			setLcd(5,"Rx ???");
			return; //loop on this connect_status
		}
	}
	if (connect_status ==4  && 		!is("<- Mx")) {
		putJeti(JETI_RIGHT);
		return;
	}
	if ((connect_status ==4 || connect_status ==5 )&& 		is("<- Mx")) {//check sensor connected to Mx
		if (rx_count < iop_rx_count+50)  //let stabilize
			return;
		if (is2("v")) {//ok something is connected - search
			connect_status =5; //try to connect
			iop_rx_count = rx_count;
			putJeti(JETI_DOWN);
			return;
		}
		setLcd(0,"no Mx ");
		connect_status =0;
	}

	if (connect_status ==5  && 		!is("<- Mx")) { //something is connected
		if (rx_count < iop_rx_count+50)  //let stabilize
			return;
		connect_status =6;
		iop_rx_count = rx_count;
	}
	if (connect_status ==6) { //checking the sensor connected at the receiver 
		//-long wait for the initializing of RCCL_Instr
		if (rx_count > (iop_rx_count+ 200 ) ) { //nothing is connected
			setLcd(0,"no Mx?");
			connect_status =7;
			iop_rx_count = rx_count;
			return;
		}
		int node= node_find();
		if (node==0 || is("<- Mx")) {
			//putJeti(JETI_DOWN); //????
			//iop_rx_count = rx_count+10;
			return; // waiting for the sensor
		}
		if (node >= 21 && node <=24) { //expander connected to the receiver
			link(20,2,21);  // down direction from Mx(20) ->  Expander (21)
			linkL(21,0,20);// up direction from Expander port 1 (21) -> Mx (20)
			connect_status =21; // connection of sensors to the ports of the Expander
			setLcd(0,"Expander");
			return;
		}
		if (node >=30) { //sensor found, connected to the receiver
			// only connect to the top of the sensor
			if ((node % 10) >0) {
				if (rx_count < iop_rx_count+10)  //let stabilize
					return;
				putJeti(JETI_UP);  // not a general solution to return to the top - need ded: the jetiSensor navigation
				iop_rx_count = rx_count;
				return;
			}
			link(20,2,node);  //from Mx (20), direction down (2) to sensor
			linkL(node,0,20);//from sensor to Mx with a long touch
			setLcd(0,get1().c_str());
			connect_status =0;
			return;
		}
		if (node==20) { //too far
			putJeti(JETI_DOWN);
			return;
		}
	}
	if (connect_status==7) {
		if ( is("<- Mx")){
		setLcd(0,"no Mx");
		connect_status=0;
		return;
		}
		putJeti(JETI_UP); //long up
			//setLcd(6,"jeti up");
			return;
	}
	if (connect_status >=21 && connect_status <=24) { //checking the sensor connected at each  expander port
		int id = node_find();
#ifdef DEBUGJETI_CONNECT
		printf("connect_status= %d id=%d %s  %s \n",connect_status,id, get1().c_str(),get2().c_str()) ;
#endif
		if (id==connect_status &&  !is2("...")) { // connect_status is the port  nummer (21..24)
			putJeti(JETI_RIGHT);
			iop_rx_count = rx_count+10;
			//printf("JETI_RIGHT\n");
			return;
		}
		if (id==0) {
			putJeti(JETI_NOTHING);
			return;
		}
		if (id==connect_status &&  is2("...")) { // connect_status is the port  nummer (21..24)
			if (connect_status < 24)
				connect_status++;
			else {
				connect_status=0;
#ifdef DEBUGJETI_CONNECT
				explorer_dump();
#endif
			}
			return;
		}
		if (id> connect_status && id >=21 && id<=24) {
			putJeti(JETI_UP);
			iop_rx_count = rx_count+10;
			return;
		}
		if (id< connect_status && id >=21 && id<=24) {
			putJeti(JETI_DOWN);
			iop_rx_count = rx_count+10;
			return;
		}
		if (id>=30) {//the connected sensor is found
			link(connect_status, 1, id);
			link(id, 3,connect_status);
			setLcd(connect_status-20,get1().c_str());
			connect_status +=10; // 31..34
			putJeti(JETI_LEFT);
			return;
		}
	}
	if (connect_status >=31 &&  node_find() >=30) {
		putJeti(JETI_LEFT);
		return;
	}
	if (connect_status >=31 &&   node_find() >=21 && node_find() <=24) {
		connect_status -=10;
		if (connect_status < 24)
			connect_status++;
		else {
			connect_status=0;
#ifdef DEBUGJETI_CONNECT
			explorer_dump();
#endif
		}
		return;
	}
}
void Jeti::getVar(short sensorId, vector<string>& z) {//extract the variables as strings from the  node id
	int sensorIndex=node_index(sensorId);
	string s = get1()+get2(); //message from the display
	string r = string(node_array[sensorIndex].line1)+string(node_array[sensorIndex].line2) ;//reference with the place of the variables
	string t; //only the variables
	for (int i =0; i<32; i++) {
		if (r[i]!='&')
			t.push_back(' ');
		else {
			if (s[i]>0x20 && s[i]<0x40)
				t.push_back(s[i]);
		}
	}
	/*printf("id = %d index=%d\n",sensorId,sensorIndex);
	printf("r =%s\n",r.c_str());
	printf("s= %s\n",s.c_str());
	printf("t= %s\n",t.c_str());
	for(int i=0; i< (signed)z.size();i++)
	printf(" %d = %s\n",i, z[i].c_str()); */
	//extract the tokens
	Tokenize(t,z," ");
}
int Jeti::readMap(short from , short to, short &next, char& longtouch) { // current node = from, returns the direction to go for the end node to
	/* this function search the the path from "from" to "to" only using the  node_array table as a navigation map
	there is no interaction with the Jeti
	this runs  fast

	returns -1 if found, -2 if error
	returns a direction 0..3 for the next step
	returns &next, the node id of the first next step
	*/
	int loop=0;
	// nothing is explored
	for (int iexp =0; iexp<JETI_NODES; iexp++)  //reset the table
		for (int i = 0; i<4; i++) {
			node_array[iexp].exp[i]=0;
		}
	int dirfrom =-1; // last direction tested from the initiel node "from"
	int current_node_id = from;


	while (loop < 500) { // explore the whole tree, with this againts a permanent loop
		loop++;
#ifdef DEBUGJETI_READMAP
		printf("readMap Current node= %d   from=%d to=%d \n",current_node_id,from,to);
		for (int i = 0; i<4; i++)
			printf(" ->%d node=%d  exp=%d \n",i,  node_array[node_index(current_node_id)].next[i],node_array[node_index(current_node_id)].exp[i]);
#endif
		int index =node_index(current_node_id); //index in the node_array
		int delta =100000;
		int i=-1;
		for (int j = 0; j<4; j++) {// select a free road
			if (node_array[index].next[j]>0) {
				int deltanew= abs(to-node_array[index].next[j]) + node_array[index].exp[j]*1000;
				if (deltanew<delta) {
					i=j;
					delta=deltanew;
#ifdef DEBUGJETI_READMAP
					printf("found_a i=%d  \n",i);
#endif
				}
			}
		}
		if (i==-1) {
			//#ifdef DEBUGJETI_READMAP
			printf("return no path %d -> %d\n",from,to);
			//#endif
			return -2;//error, no path   from -> to
		}

		if (current_node_id == from) {
			dirfrom=i;
			next=node_array[index].next[i];
			longtouch= node_array[index].longtouch[i];
		}
		node_array[index].exp[i] +=2; //going out of this node
		short former_node_id=current_node_id;
		current_node_id =	node_array[index].next[i];
		index =node_index(current_node_id);
		for (int j = 0; j<4; j++) {
			if (node_array[index].next[j]==former_node_id) {
				node_array[index].exp[j]+=1;
			}
		}
		if (current_node_id== to) {
#ifdef DEBUGJETI_READMAP
			printf("------------------------found path from=%d dir=%d\n",from,dirfrom);
#endif
			return dirfrom;
		}
	}
	return -2;
}
void Jeti::putdir(int dir) {
	switch (dir) {
	case 0:
		putJeti(JETI_UP);
		break;
	case 1:
		putJeti(JETI_RIGHT);
		break;
	case 2:
		putJeti(JETI_DOWN);
		break;
	case 3:
		putJeti(JETI_LEFT);
		break;

	}
	//printf("putDir dir:%d rx_count:%d\n",dir,rx_count);
}
bool Jeti::JetiSensor(short iop_current,short sensorID,vector<string> &valueString) {  // process the JETI opcode  return true when a new value exists returns a vector of string variables
	/*
	Parameters
	iop_current identifies the current operation
	sensorID
	active is true for a normal continuous JETI operation (oplen <=6)
	active is L :the logical control varaible when oplen >6

	the answer is a vector of strings  ! call this  function with a EMPTY vector
	 error condition returns true (= > sensor found) but vector empty
	--------State variables for navigation between  JETI operation-------------------
	iop_busy = identity of this JETI operation ( = index in the operation array)
	              < 0 when no JETI operatio is active
	iop_last  = identity of the last active operation
	iop_busy_processed =true when iop_busy has been "touched" during a calc ( does not  occurs after some change in the logical variables)
	--------State variables for one move   "from" -> "next" -----------------
	from = node id of the departure node
	next  = node id of the next node (directly next )
	fromnext= direction from->next
	next =0 when there is no active move
	--------State variable for the Jeti communication
	status = 0 when the last instruction (putDir) has been sent and the answer has been received
	*/
	int rx_count_begin= rx_count; //the rx data can change at any moment
	if (iop_busy==iop_current)
		iop_busy_processed=true; //the locked iop_busy is processed
	if (iop_busy>=0 && (iop_busy !=iop_current))
		return false; //skip :another JETI instruction is busy
	if (iop_busy<0   && (iop_current <= iop_last))
		return false ; //skip , this was processed
	if (iop_busy<0 && (iop_current > iop_last))  {//begin processing this operation, this is the new busy instruction
#ifdef DEBUGJETI_SENSOR
		printf("JetiSensor begin-------------------  iop=%d   iop_last=%d \n",iop_current,iop_last);
#endif
		iop_busy = iop_current;
		iop_busy_processed=true;
		iop_last  = iop_current;
		iop_busy_time= GetTicker();
		next=0; // not moving during a single step ??????
		//execute the next instructions
	}
	if ((status>0 && longtouch == 0x20)|| connect_status !=0 ||  jeti_box_busy)
		return false; // wait until ready
	int node_current_id =  node_find(); 
	if (rx_count != rx_count_begin) // the interrupt routine changed the data
		return false;
	if (is2("Connecting"))
		return false;
	if (node_current_id==0) {  //? error or Connecting
		//stat.node_err++;
#ifdef DEBUGJETI_SENSOR
		//printf("JetiSensor error next=%d \n%s\n%s     count=%d\n",next,get1().c_str(),get2().c_str(),rx_count);
#endif
		return false;
	}
	if (next >0 && node_current_id==next) {  //the single step is done
		//printf("JetiSensor iop=%d  next=%d is done\n",iop_current,next);
		//stat.wait += rx_count- rx_count_sensor;
		next=0;
	}
	if (next >0 && node_current_id==from && longtouch =='L') {  //the single step is  not done but longtouch
		putdir(fromnext);
		return false;
	}
	if (next >0 && node_current_id==from && (rx_count- rx_count_sensor)< 6) {   //the single step is  not done
		//wait a moment
		return false;
	}
	if (iop_busy==iop_current)	{  //process this instruction
		if (node_current_id ==sensorID) {// --------------arrived at  destination------------------
			// find the variable and return
			getVar(sensorID,valueString);  //= the variables
#ifdef DEBUGJETI_SENSOR
			printf("JetiSensor iop=%d   is done  iop_last=%d   zsize =%d \n",iop_current,iop_last,valueString.size());
#endif
			iop_busy=-1;
			return true;
		}
		// move one single step or retry the present move
		fromnext = readMap(node_current_id,sensorID, next,longtouch); //search the direction of the next step
		if (fromnext== -2) {//path not found
			//printf("JetiSensor path not found iop=%d move from:%d to:%d  \n",iop_current,node_current_id,next);
			iop_busy=-1;
			return true;
		}
		if (GetTicker() > iop_busy_time + 3000) {//destination not found after 15" not found = a simple watchdog
			printf("JetiSensor timeout iop=%d   move from:%d to:%d \n",iop_current,node_current_id,next);
			iop_busy=-1;
			return true;
		}
		putdir(fromnext);
		rx_count_sensor=rx_count; // starting a move from->next
		from=node_current_id;
#ifdef DEBUGJETI_SENSOR
		printf("JetiSensor iop=%d move from:%d to:%d  dir=%d\n",iop_current,node_current_id,next,fromnext);
#endif
		//next is now > 0
		return false;
	}
	return false;
}
/*
//=======  performance statistics =======================
void  Jeti::statReset(){
	#ifdef DEBUGP
	printf("----Jeti statistics   reset-----\n");
	#endif
	stat.time_from=GetTicker();
	stat.UART_err=0;
	stat.node_err=0;
	stat.rx_count=0;
	stat.wait=0;
	stat.repeat=0;
}
void  Jeti::statDump(){

	printf("----Jeti statistics-----\ntime_from=%d  duration=  %d \n",stat.time_from, GetTicker()-stat.time_from );
	printf("  UART_err= %d\n  node_err= %d\n  rx_count= %d\n  wait=     %d\n  repeat=   %d\n------\n",stat.UART_err, stat.node_err,stat.rx_count,stat.wait,stat.repeat    );


}
*/


