/* communication with a Jeti Duplex transmitter module
_____Jeti Data communication protocol______________________
The communication from the TX module to the JetiBox
1. It is 1-wire serial communication (9600 Bauds)
2. The Jeti box is a dumb Terminal, the Jeti components control the whole process (components are the Tx module, the RX, a sensor or expander)
3. 1 received message  consists of 34 byte, 2 control character (start=0xfe finish=0xff) and 32 bytes text between these control characters.
4. 1 Byte consists of 13 bits in the format: T-D-D-D-D-D-D-D-D-I-P-S-S    (LSB first= T-0-1-2-3-4-5-6-7-I-P-S-S)
		where: T = startbit, D =  8 data bits, I control characters or text ( 1=text, 0=control), P = parity based odd , S = stop bit (two).
5. The box answers each message (34 byte)  with one byte  ACK message (with I=0: control byte)
	0xF0 =11110000 if no button (four bits with value 1 if the button is NOT pushed
	bit 4 =Right , Bit 5 = Up (or Back) , Bit 6 = Down or Select und Bit 7 Left
6.a short delay  is needed between the received message and the ACK message
     from the end of 0xff to begin of ACK :3,9 msec (mesured with Saleae)
		 -TU module waits 21 msec and  reply with a new 34 bytes message =>	total length = 79 msec

	If there is no ACK, the TU module waits 25 msec and resend the message

____soft uart__________________________-
	data acquisition with 2 interrupts (see  AVR 274)
	each "character" is written in a  buffer 32 bytes
____starting___________________________
when the jetibox and the module are starting, there is
- a firts message 32 spaces from the module
-a first answer 0x00 from the Jeti Box


*/
#include "core.h"
#define UART_PIN  p29    //pin for tx or rx , connect to the TU module through a 4.7k resitance
#define UART_W15 146   //1.5 pulse 
#define UART_W1 98        // 1 pulse, tuned with the logic analyzer saleae
#define UART_BITS  12 //define here the TOTAL number of bits (data bits + parity + stop bits)
//#define UART_DEBUG p21 //debugpin to debug with a real JetiBox connected -> the ACK from RCCL is on this pin
extern rccl_data rc; //common data
/*----define the buffer */
volatile unsigned char  UART_buffer[33]; //temporary buffer filled char / char
volatile uint32_t  UART_count_char ; // number of items in the buffer

// input  /output
#ifdef UART_DEBUG 
DigitalOut debugpin(UART_DEBUG);
#endif
DigitalInOut UART_pin(UART_PIN); // pin connected to the signal from Tx module
Timeout      UART_nextBit;
Timeout      UART_Tx_nextBit;
InterruptIn  UART_startBit(UART_PIN) ;
//char processing   
volatile int   UART_counter;//next bit position value 0..8 (9 bits)+ 9=parity  10,11=stop  FOR THE RX
volatile char unsigned    UART_Rx_data ; //character transmitted
bool textchar ; //true if text, false if control
bool UART_errorchar ;// true if error (parity, stop bit)
int    UART_parity;
char unsigned   UART_Tx_buttons ;//ACK to transmit
char unsigned   UART_Tx_data ; //ACK currently transmitted and shifted -> transmitter module
int   UART_Tx_counter;//bit counter for the Tx
void ISRstartBit() ;
void ISRdataBitRx();
void ISRdataBitTx();
//----interrupt routines ---------------------------------------------------------------------------------------
void ISRstartBit() { //detect the start bit, start timeout to read the first data bit
	UART_nextBit.attach_us(&ISRdataBitRx , UART_W15); // delai = 1.5 pulse
	UART_counter= -1;//preparing for the first data bit
	UART_Rx_data=0 ;//received data
	UART_errorchar = false;
	UART_parity=0;
	UART_startBit.fall(NULL);  // detach the  ISRstartBit
}
void ISRdataBitRx() { //nextbit timeout in Rx mode
	UART_counter++;
	if (UART_counter< UART_BITS) {//this is not the last bit of a character		- immediatly start the next timer interrupt
		UART_nextBit.attach_us(&ISRdataBitRx , UART_W1);  // next bit
	} else
		UART_startBit.fall(&ISRstartBit); //listen to the next char ( cancel later if end of message)
	int pinread = UART_pin.read();
	if (UART_counter<8)	{ 	// process  the first 8 databits
		UART_Rx_data = (UART_Rx_data >> 1);   //Right shift RX_data so the new bit can be masked into the Rx_data byte.
		if (pinread==1) {
			UART_Rx_data |= 0x80;               //Set MSB of RX data if received bit == 1.
			UART_parity++;
		}
		return;
	}
	if (UART_counter==8)	{ 	// control bit
		textchar = pinread;
		UART_parity +=pinread;
		return;
	}
	if (UART_counter==9)	{//parity
		UART_parity +=pinread;
		if (UART_parity%2 ==0) {//UART_errorchar=true;
			UART_errorchar=true;
		}
		return;
	}
	if (UART_counter==10 || UART_counter==11)	{ //stop bits
		if (pinread==0) {
			UART_errorchar=true;
		}
		return;
	}
	//the bits of one UART character are received -> transfer to the buffer
	if (UART_errorchar){
		//rc.jetiptr->stat.UART_err ++;
		rc.jetiptr->status=0;
		return; //do not process a byte with errors
	}
	if (!textchar) {//control char
		if (UART_Rx_data== 0xFE) {//start of message
			UART_count_char=0;
			return;
		}
		if (UART_Rx_data== 0xFF && UART_count_char==32 && !UART_errorchar) {//end of message -
			for (int i=0; i<32; i++) {// copy the 32 characters received
				rc.jetiptr->jx[i]=UART_buffer[i];
			}
			
			if (rc.jetiptr->status==2)
				rc.jetiptr->status=0;
			if (rc.jetiptr->status==0){
				rc.jetiptr->rx_count++;
				//rc.jetiptr->stat.rx_count++;
			}
				
			//----prepare the transmit of ACK-------------------------------------------------
			UART_Tx_nextBit.attach_us(&ISRdataBitTx , 3800);  // prepare the ACK
			UART_Tx_counter=-1;
#ifndef UART_DEBUG
			UART_startBit.fall(NULL);
			UART_pin.output();
			UART_pin=1;
#endif
		}
	} else { //text char
		if (UART_count_char < 32)
			UART_buffer[UART_count_char]=UART_Rx_data;
		UART_count_char++;  //value 32 is correct for a full buffer
	}
}


void ISRdataBitTx() { //nextbit timeout in Tx mode
	int txpin = 1;
	if (UART_Tx_counter== -1) {//start bit
		txpin=0;
		UART_parity=0;
		UART_Tx_data=UART_Tx_buttons;
	}
	if	(UART_Tx_counter >=0 && UART_Tx_counter<8) { //data bits 0..7
		int out = UART_Tx_data & 0x01;
		txpin = out;
		UART_parity += out;
		UART_Tx_data = (UART_Tx_data >>1);
	}
	if	(UART_Tx_counter==8) { //control bit
		txpin =0;
	}
	if	(UART_Tx_counter==9) { //parity bit
		UART_parity++;
		txpin= UART_parity%2;
	}
	if (UART_Tx_counter<10) {
		UART_Tx_nextBit.attach_us(&ISRdataBitTx , UART_W1);  //next tx bit
		UART_Tx_counter++;
	} else { //end of transmit- prepare the reception
		if (UART_Tx_buttons!= 0xF0)
			rc.jetiptr->status=2;
		UART_Tx_buttons= 0xF0;
		txpin=1; //including the stop bits
#ifndef UART_DEBUG
		UART_startBit.fall(& ISRstartBit);//detect the next received start bit
		UART_pin.input();
#endif
	}
#ifdef UART_DEBUG
	debugpin=txpin;
#else
	UART_pin=txpin;
#endif
}


//----communication with RCCL----------------------
Jeti::Jeti() { //start the interface with the Jeti transmitter module
	rx_count=0; //count the messages received
	//UART_lasttick=0;
	for (int i=0; i<32; i++) {// copy the 32 characters received
		jx[i]=0x20;
		UART_buffer[i]=0x20;
	}
	jx[32]='\0';
	UART_buffer[32]='\0';
	//outside the class: the variables prefixed with UART_
	UART_Tx_buttons= 0xF0; //default ACK
	UART_pin.mode(PullNone);
	UART_pin.input();
	UART_startBit.mode(PullNone);
	UART_startBit.fall(& ISRstartBit); // attach startBit to detect the next start bit
	status = 0; // status Tx-Rx
	connect_status=1; // to explore the sensors
	jeti_box_busy=false;
	//changed=false;
	next=0;
	//statReset();
}
void Jeti::putJeti(char x) {
	/*if (status >0){
		printf("putJeti status=%d\n",status);
		return;
	}*/
	iop_rx_count = rx_count;
	__disable_irq();
	status=1;
	UART_Tx_buttons= x & 0xF0;
	__enable_irq();
}
string Jeti::get1() { //get at he first line
	string x;
	for (int i=0;i<16; i++)
		x.push_back(jx[i]);
		return x;
}
string Jeti::get2() { //get at he first line
	string x;
	for (int i=16;i<32; i++)
		x.push_back(jx[i]);
		return x;
}
/*string Jeti::get1() { //get at he first line
	string x = string(jx);
	return x.substr(0,16);
}
string Jeti::get2() { //get the second line
	string x = string(jx);

	return x.substr(16,16);

}*/
void Jeti::jeti_replace() { //replace strange characters   = <   ~=>
	for (int i=0 ; i<32; i++) {
		if (jx[i] ==0x7F) {
			jx[i]='<';
			continue;
		}
		if (jx[i] == '~') {
			jx[i]='>';
			//printf("replace %s  \n",jx);
		}
	}
}

