#include "mbed.h"
#include "core.h"
//#include "calc.h"
//#include <string>
#include <vector>
#include "mcp3208.h"
extern rccl_data rc; //common data
//
//______ calchw_eric_______    compiler off-line avec link calchw-> ce fichier - compiler online en supprimant les autres calchw
//
/* calc functions depending on the hardware of the transmitter
   functions:
   1)hwinit defines the hardware inputs (when a calc object is created)
       this is not depending on the microprocessor, but on the real hardware of the transmitter (switches, ...)
   2)hwread the values of each input (once / rc loop)
       ------- this is depending on the microprocessor ----here the easy way: the mbed
   headers -> calc.h
   3)hwout send the output results to the transmission procedure (PPM or other, depending on the transmission)

*/

//#define DEBUGHW       // use this to get a detailed display of the "system" inputs and adjust pin nummer and direction
//1. ------------ definition of all the IO pins ---------------------------------------------------------------
//1.1--------do not change
SPI spibus(p5,p6,p7);  //SPI bus
MCP3208 adc(spibus,p8);//p8 = cs pin for the MCP3208 adc
//lcdhw defines the pins p9 =Tx, pin 10=Rx, pin 11=reset
//1.2 ====special digital io for the trims - change  the pin number if needed
DigitalIn dsta(p12); // the special switch STa
DigitalIn dstb(p15); //the switch Stb
DigitalIn dBA(p13); // the BA button  TRUE=on (pushed) FALSE=off
Debounce DebBA;
// p14 output to cut the power see power.cpp
DigitalIn dp16(p16); //sw
AnalogIn ap17(p17); //battery 1 current   1= left 2=right (pcb)
AdcFilter fp17;
DigitalIn dp21(p21); //sw
AnalogIn ap18(p18); //battery 2 current
AdcFilter fp18;
AnalogIn ap19(p19); //battery 2 voltage
AdcFilter fp19;
AnalogIn ap20(p20); //battery 1 voltage
AdcFilter fp20;
DigitalIn dp22(p22); //sw
DigitalIn dp23(p23); //sw

// specifique pour Eric  24  =@BG(bouton gauche) 25/26 @S7a/b
DigitalIn dp24(p24); //sw
DigitalIn dp25(p25); //sw
DigitalIn dp26(p26); //sw
//27,28 -> other radio
//29 -> hwjeti/jeti.cpp = communication as JetiBox
//30  ->hwmbed/timer.cpp =PPM output


float hwmix(float x, float highvalue ,float centervalue, float lowvalue) {//calibration of the sticks
	/*
	1.comment out the instruction x= hwmix(  ...)
	        example:
	            x = adc.read_input(5)*100.0 ;
	            // x =hwmix(x,    ,    ,     );
	            setVarN ("@M1",x*TOINT);
	2. using the variable display, measure for each stick the value up,center and low
	            stick UP , the variable M1 displays 62,76
	            stick CEnter  52,39
	            stick DOWN 37,32
	3. replace x = hwmix( x, 62.76, 52.39, 37.32)
	*/
	if (highvalue > lowvalue) {
		if (x > centervalue)
			return (100.0)*(x -centervalue)/(highvalue-centervalue);
		else
			return (-100.0)*(centervalue -x)/(centervalue -lowvalue);
	} else {
		if (x > centervalue)
			return (-100.0)*(x -centervalue)/(lowvalue-centervalue);
		else
			return (100.0)*(centervalue -x)/(centervalue -highvalue);
	}
}

void Calc::hwinit() { //initialisation when a calc model is loaded
	//2. ==== init ===============
	//2.1================   change this to use your own variable name===================
	// add each hardware input variable to the vector
	calc_vars.push_back(Var("@M1",IN,NUM));
	calc_vars.push_back(Var("@M2",IN,NUM));
	calc_vars.push_back(Var("@M3",IN,NUM));
	calc_vars.push_back(Var("@M4",IN,NUM));
	calc_vars.push_back(Var("@C1",IN,NUM));
	calc_vars.push_back(Var("@C2",IN,NUM));
	calc_vars.push_back(Var("@C3",IN,NUM));
	calc_vars.push_back(Var("@S1",IN,LOGICAL));
	calc_vars.push_back(Var("@S2",IN,LOGICAL));
	calc_vars.push_back(Var("@S3",IN,LOGICAL));
	calc_vars.push_back(Var("@S6",IN,LOGICAL));
	//calc_vars.push_back(Var("@S7",IN,LOGICAL));
	calc_vars.push_back(Var("@S4a",IN,LOGICAL));
	calc_vars.push_back(Var("@S4b",IN,LOGICAL));
	calc_vars.push_back(Var("@S5a",IN,LOGICAL));
	calc_vars.push_back(Var("@S5b",IN,LOGICAL));
	
	//specifique eric
	calc_vars.push_back(Var("@BG",IN,LOGICAL)); // bouton poussoir gauche
	calc_vars.push_back(Var("@S7a",IN,LOGICAL));
	calc_vars.push_back(Var("@S7b",IN,LOGICAL));
	//2.2 ======== spi init - do not change
	spibus.format(7,0);
	spibus.frequency(1000000);
	//2.3 mbed digital input have no external pull-up resistor -> define the internal pullup
	dp16.mode(PullUp);
	dp21.mode(PullUp);
	dp22.mode(PullUp);
	dp23.mode(PullUp);
	dsta.mode(PullUp);
	dstb.mode(PullUp);
	dBA.mode(PullUp);
	//eric
	dp24.mode(PullUp);
	dp25.mode(PullUp);
	dp26.mode(PullUp);
	
	//2.4 ========= do NOT change the definition of theses variables ============
	calc_vars.push_back(Var("@H1",IN,LOGICAL)); // @H1 = 1 only for the first loop - remains 1 if a BREAK occurs
	calc_vars.push_back(Var("@HS",IN,LOGICAL)); // @HS = clock tick
	calc_vars.push_back(Var("@HT",IN,NUM)); // @HT = transmitter clock
	calc_vars.push_back(Var("txV",IN,NUM)); //  transmitter voltage
	calc_vars.push_back(Var("@BA",IN,LOGICAL)); //button autotrim when autotrim active
	calc_vars.push_back(Var("@BB",IN,LOGICAL)); //the same button when the autotrim is not active
	calc_vars.push_back(Var("@BC",IN,LOGICAL)); //second rapid push of @BB
	calc_vars.push_back(Var("@dsta",IN,LOGICAL));
	calc_vars.push_back(Var("@dstb",IN,LOGICAL));

	//===================================
#ifdef DEBUGHW  //display the value of the system variables
	calc_vars.push_back(Var("dBA",IN,LOGICAL));
	calc_vars.push_back(Var("dt1P",IN,LOGICAL)); //trims  + and -
	calc_vars.push_back(Var("dt1M",IN,LOGICAL));
	calc_vars.push_back(Var("dt2P",IN,LOGICAL));
	calc_vars.push_back(Var("dt2M",IN,LOGICAL));
	calc_vars.push_back(Var("dt3P",IN,LOGICAL));
	calc_vars.push_back(Var("dt3M",IN,LOGICAL));
	calc_vars.push_back(Var("dt4P",IN,LOGICAL));
	calc_vars.push_back(Var("dt4M",IN,LOGICAL));
#endif
	kick(); //init auto cutoff#endif
}

void Calc::hwread() { //read the digital and analog inputs  (each 10 msec)
	// 3.1/========do not change this====================
	// set the value of HS to 1 each 1/100 minute (because 6000 tiks = 1 min)
	setVarL("@HS",((rc.rccl_task_tick % 60)==0));
	setVarN("@HT",rc.rccl_task_tick / 60);
	//  3.2===change the calibration coefficients
	/*    mesurer la tension de batterie au voltmetre (mettre la masse du voltmetre au "trou" du regulateur
	        tension V1P au pole + =6.31V , idem tension batterie 2 , mesurer V2P
	remettre les coeficients de calibrage a leur valeur initiale  bx  10  ; cx = 0
	compiler
	noter les tensions affichees vb1 et vb2  (6.00)
	calculer les valeurs
	b1 = (V1M/ vb1)*10   idem b2=(V2M/vb2)*10   = 10.52
	  remettre c a sa valeur normale .56/1000
	 recompiler...dorenavant la tension  de la batterie est affichee
	*/
	/* Resistances de mesure
		- courant 0.56 R
		- tension batterie : 7.2k et 3.3 k -> ratio = 3.3 /(7.2+3.3) = 0.3143
	*/
	//===current measure=====================
	// no change needed with the resistors ( 0.56 Ohm measure  and opamp *10 ): current 100mA -> 0.56V input adc -> read = 0.56/3.3=0.1697
	// coeff = 100/0.1697 = 589  ()
	//rc.batt.ib1= fp17.filter(ap17.read()) *589; for 0.56 Ohm
	rc.batt.ib1= fp17.filter(ap17.read()) *589; //current mA
	rc.batt.ib2= fp18.filter(ap18.read()) *589;
	//==========do not change
	rc.batt.cb1 += (rc.batt.ib1/360000.0);
	rc.batt.cb2 += (rc.batt.ib2/360000.0);
	//==========changer ici=============
	float const b1=10.52 ; //  6.43    /6.11      
	float const b2=10.19; //   6.43     /6.31
	//=====resistance de mesure de courant 0.56 R
	float const c=0.56 /1000. ;   // 0.56/1000
	//float const c=0; //test
	//=============================
	rc.batt.vb1 = c*rc.batt.ib1+ b1*fp20.filter(ap20.read());
	rc.batt.vb2 = c*rc.batt.ib2+  b2*fp19.filter(ap19.read());
	//======do not change this  - see VBLOADED in core.h ======================
	if (rc.rccl_task_tick < 1000 && rc.batt.vb1> VBLOADED  && rc.batt.vb2 > VBLOADED ) { // the battery is loading and loaded  - reset the cumulated mAh when starting
		rc.batt.cb1 =0;
		rc.batt.cb2 =0;
	}
	short txV =  rc.batt.vb1*TOINT; //maximum of the 2 batteries
	if (rc.batt.vb2>rc.batt.vb1)
		txV =  rc.batt.vb2*TOINT;

	setVarN("txV",txV);

	//3.3 ===== do not change the dtrim_level ====================
	/*-----------------digital trims -------------------------------------------------------------------
	  theses variables  are within the Calc object
	    1) dtrim_level depends on a switch with 3 positions
	        0 = normal
	        1 = fast trim with the autotrim button
	        2 = parameter trim (in flight update of some parameters , using the digital trims)
	        Here this variable is depending on @ST
	    2) the array dtrimArray[4] - each entry in this array is a Dtrim object ( indexed in C from 0 to 3)
	    3) each object has a get  method get( pin + , pin -)
	*/
	//---- the special 3 positions switch for the digital trims,
	// Sta TRUE,Stb=FALSE => autotrim actve
	// Sta= FALSE,Stb =false  => normal status (no autotrim)
	// Stb = TRUE,Sta=FALSE => special use of the trims to tune selected parameters
	// ! sign to reverse
	// reset the autotrim variable:
	setVarL("@BA",false);
	dtrim_level=0;
	if (!dsta.read())
		dtrim_level = 1;
	if (!dstb.read())
		dtrim_level = 2;
	setVarL("@dsta",dtrim_level==1) ;
	setVarL("@dstb",dtrim_level==2) ;

	// 3.4  ===== the digital trims are all connected to the lcd digital io pins  =======change the pin if needed / control the SIGN of the trim ===========
	//
	dtrimArray[0].get(!rc.lcdptr->PinGet(12),!rc.lcdptr->PinGet(13));  //trim channel @M1 first pin is  trim +   , second pin is trim - )
	dtrimArray[1].get(!rc.lcdptr->PinGet(10),!rc.lcdptr->PinGet(9));//M2
	dtrimArray[2].get(!rc.lcdptr->PinGet(14),!rc.lcdptr->PinGet(15));//M3
	dtrimArray[3].get(!rc.lcdptr->PinGet(1),!rc.lcdptr->PinGet(0));//M4
	// check  for the correct pins to confirm the power-off
/*	if (!rc.lcdptr->PinGet(10) && !rc.lcdptr->PinGet(0)){
		rc.poweroffconfirm='Y';
		printf ("Power-off confirmation\n");
	}
	else
		rc.poweroffconfirm='N'; */
#ifdef DEBUGHW
	// each button =TRUE  when pushed ww
	setVarL("dBA", !dBA.read());
	setVarL("dt1P", !rc.lcdptr->PinGet(12));
	setVarL("dt1M", !rc.lcdptr->PinGet(13));
	setVarL("dt2P", !rc.lcdptr->PinGet(10));
	setVarL("dt2M", !rc.lcdptr->PinGet(9));
	setVarL("dt3P", !rc.lcdptr->PinGet(14));
	setVarL("dt3M", !rc.lcdptr->PinGet(15));
	setVarL("dt4P", !rc.lcdptr->PinGet(0)); 
	setVarL("dt4M", !rc.lcdptr->PinGet(1));
#endif
	//3.5====change this : ADC input from external adc ======================
	// each variable is assigned to a channel of the external adc (channel 0 ..7, 0 is the utmost right position)
	// chaque variable est assignee a un canal x dans adc.read_input(x)
	// mettre // devant la ligne de calibrage x= hwmix...
	// noter la valeur affichee aux positions +100% (en haut ou a droite) , -100% et centre
	// ecrire ces valeurs dans hwmix ( valeur + 100%  , valeur au centre, valeur -100%) et SUPPRIMER les  // en tete
	// verifier ques les valeurs affichees sont correctement calibrees
	float x,sum;
	sum=0; //add all the sticks to detect inactivity
	x = adc.read_input(0)*100.0 ;
	x = hwmix(x,  37.91,50.14,62.8) ;
	sum+=x;
	setVarN("@M1",x*TOINT);

	x = adc.read_input(1)*100.0 ;
	x =hwmix(x,  37.67  ,50.12    , 62.59);
	sum+=x;
	setVarN("@M2",x*TOINT);

	x = adc.read_input(2)*100.0 ;
	x =hwmix(x, 63.64   , 50.48   , 37.50);
	sum+=x;
	setVarN("@M3",x*TOINT);

	x = adc.read_input(3)*100.0 ;
	x =hwmix(x,62.06   ,  49.43  , 36.88);
	sum+=x;
	setVarN("@M4",x*TOINT);
	const    int delta = 4;
	int diff ;
	diff = sum- autocutoff_sticks;
	string prefix = rc.current_file.substr(0,2); //prefix of the model file
	if ( (prefix.compare("TE" )==0   )|| (diff > delta) || (diff< -delta)) {//		the sticks are moved or auto power-off not activated for a test model
		kick(); //this is Calc::kick() see calcload.cpp
		autocutoff_sticks=sum;
	}

	//for the linear cursor there is no "center" position, replace with 50
	x = adc.read_input(7)*100.0 ;
	x =hwmix(x,0  ,  50  , 99.9);
	setVarN("@C1",x*TOINT);

	x = adc.read_input(6)*100.0 ;
	x=hwmix(x,0  , 50  , 99.9);
	setVarN("@C2",x*TOINT);

	x = adc.read_input(4)*100.0 ;
	x =hwmix(x,0,  50  , 99.9);
	setVarN("@C3",x*TOINT);

	// 3.6 ====change this====  gpio from the mbed
	// example of reverse :use the ! as in setVarL("@S4b",!dp16.read()) ;
	setVarL("@S4a",!dp21.read()) ;
	setVarL("@S4b",!dp16.read()) ;
	setVarL("@S5a",!dp23.read()) ;
	setVarL("@S5b",!dp22.read()) ;
	//eric
	setVarL("@BG",!dp24.read()) ;
	setVarL("@S7a",!dp25.read()) ;
	setVarL("@S7b",!dp26.read()) ;
	// 3.7=====change this ===   other gpio from the Lcd
	// example of reverse :use the ! as in setVarL("@S1",!rc.lcdptr->PinGet(4)) ;
	//setVarL("@P2",!rc.lcdptr->PinGet(2)) ;
	//setVarL("@P3",!rc.lcdptr->PinGet(3)) ;
	//setVarL("@S7",rc.lcdptr->PinGet(5)) ; //????
	setVarL("@S1",rc.lcdptr->PinGet(4)) ;
	setVarL("@S2",rc.lcdptr->PinGet(6)) ;
	setVarL("@S3",rc.lcdptr->PinGet(8)) ;
	setVarL("@S6",rc.lcdptr->PinGet(11)) ;

	//======== autotrim button ====change the (!) in !dBA.read() to reverse the button (has to be TRUE when pushed)
	bool buttonBA = DebBA.get(!dBA.read());
	//=============
	setVarL("@BB",false);
	setVarL("@BC",false);
	if (dtrim_level==1 && buttonBA) { //autotrim is active -> set the variable @BA
		setVarL("@BA",true);
		rc.sndptr->putFile("2T2");//sound  "reset" replace with ...
#ifdef DEBUGP
		printf("set BA = true\n");
#endif
	}
	if (dtrim_level!=1  && buttonBA) { //autotrim desactvated -> set @BB with the button
    kick();
		if (GetTicker() < (DebBA.ticker +50)) {//second push of the button  within 500 msec
			setVarL("@BC",buttonBA);
			//printf("set @BC  ticker=%d / %d  \n",GetTicker(),DebBA.ticker);
		} else {
			setVarL("@BB",buttonBA); // no autotrim =this button -> @BB
			//printf("set @BB  ticker=%d / %d  \n",GetTicker(),DebBA.ticker);
		}	
	}

}
void Calc::hwout() {

	// for a PPM signal variable name = @Vxx  xx=PPM channel+1
	vector<Var>::iterator it;
	int i=0;
	for (it=calc_vars.begin(); it<calc_vars.end(); it++) {
		Var v = *it;
		if (v.getTypeOrig() == OUT && v.typeNum== NUM && v.name.size() >=2 && v.name.substr(0,2)== "@V") {
			string channel = v.name.substr(2); // channel number from 1 !
			int ch = atoi(channel.c_str()) -1; //channel from 0
			ppm_set(ch , v.value);
		}
		//pc.printf("Var index=%3i type=%i  org=%i  val=%i  %s\n",i,v.typeNum,v.typeOrig,v.value,v.name.c_str());
		i++;
	}


}