#include "mbed.h"
#include "core.h"
//#include <string>
/* Serial connection to the 4D - depending on the platform (here mbed) */
extern rccl_data rc; //common data

#ifdef LCD_RESET_IN_OUT
//option opendrain
DigitalInOut ecranReset(p11); //reset pin for the LCD   =P0.18
#else
//DigitalOut ecranReset(p11); //reset pin for the LCD   =P0.18
#endif


Serial uart_ecran(p9,p10);       // tx and rx

//---------lcd 4D and all the low level calls depending on mbed -----------------------
void Lcd4D::flush() {
	if (uart_ecran.readable()== 1)  //flush the buffer
		uart_ecran.getc();
}
void Lcd4D::Init1() { //first step in the initialization of the Lcd
#ifdef DEBUG_LCD_START
	printf("lcd4D Init1 a\n");
#endif
	ecranReset.mode(OpenDrain);
	ecranReset=1;//normal status
	uart_ecran.baud(9600);

	wait_ms(10);
	int startTicker= GetTicker();
	ecranReset=0; //pin for reset hardware + wait 3s
	wait_ms(2);   //reset pulse
	ecranReset=1;//normal status
#ifdef DEBUG_LCD_START
	printf("lcd4D Init1 - b\n");
#endif
	while (GetTicker()< startTicker + 400) {//delay 0.5 to 3 seconds to allow the 4D to setup
		flush();
		 rccl_task(0); // let the rc loop run during a warm start
	}
#ifdef DEBUG_LCD_START
	printf("lcd4D Init1-c   end Init1\n");
#endif
}
void Lcd4D::PinRead() { //physical read of the 16 GPIO - save to register RPC 1
	uint32_t reg1 = 0;
	for (int i =0; i<= 6; i++) {//read individual pins do not read the pin 7 :this is a output pin
		unsigned char port = i;
		putc('\x69'); //ReadPin
		putc(port);
		while (uart_ecran.readable()!= 1)
			;
		reg1 |=  0x01 & uart_ecran.getc();
		reg1 <<= 1;
	}	
	reg1 <<= 1; // skip pin 7 (output)
	//read bus :pins 8..15
	char bus;
	putc('\x61'); //ReadBus
	while (uart_ecran.readable()!= 1)
		;
	bus = uart_ecran.getc();//bits
	for (int i=8; i<=15; i++) {
		reg1 |= bus & 0x01;
		bus >>= 1;
		if (i ==15)
			break;
		reg1 <<= 1;
	}
	LPC_RTC->GPREG1 = reg1; // save to register 1 of RTC
	// format register RPC 1 16 bits 0 + 16values of the Pins 0..15
}
//----------digital inputs---------------------------------------------------
bool Lcd4D::PinGet(int n) {
	uint32_t reg1 = LPC_RTC->GPREG1;
	bool x = false;
	for (int i =15; i>=0; i--) {
		if (i==n)
		 x=  reg1 & 0x00000001;
		reg1>>=1;
	}
	return x;
}
void Lcd4D::PinDump(){
	printf("PinDump at %d  ",GetTicker());
	unsigned char uc1 = (unsigned char)(LPC_RTC->GPREG1 >> 24);
	unsigned char uc2 = (unsigned char)(LPC_RTC->GPREG1 >> 16);
	unsigned char uc3 = (unsigned char)(LPC_RTC->GPREG1 >>  8);
	unsigned char uc4 = (unsigned char)(LPC_RTC->GPREG1 >>  0);
	printf("  hex bytes: %02X %02X %02X %02X\n",uc1,uc2,uc3,uc4);	
}

void Lcd4D::putc(char x) {
	//while (ecran.writeable()!= 1){}
	uart_ecran.putc(x);
}
char Lcd4D::getc() {
	while (uart_ecran.readable()!= 1) {//wait
		rccl_task(0);
	}
	return uart_ecran.getc();
}
char Lcd4D::getcB() { //blocking call, for along transmission lcd->micro
	return uart_ecran.getc();
}
void Lcd4D::Fast() { //switch  to a fast transmission
	// normal reliable high speed 0D 115200 bauds
	// 282353 bauds in use at 20130302
	uart_ecran.putc('\x51'); //set new baud rate
	uart_ecran.putc( '\x0F');//256000 not correct  0Fhex : 256000 Baud (It is actually 282353Baud !!!)
	//uart_ecran.putc( '\x11');//256000 
	uart_ecran.putc('\x0D'); //115200 ok with short wires
	//uart_ecran.putc( '\x0C');//57600
	//uart_ecran.putc( '\x0A');//38400
	wait_ms(5); // some sync delay is needed
	//uart_ecran.baud(115200);
	//uart_ecran.baud(256000);
	uart_ecran.baud(282353); // 
	wait_ms(3);//? how to avoid
	Ack();
}
void  Lcd4D::CalibrateWrite() {
	FILE *fp = fopen("/local/RCCLTS.txt", "w");
	fprintf(fp,"%5.3f   %5.3f  %5.3f  %5.3f  %5.3f  %5.3f  %i %i \n" ,calibrate.A,calibrate.B,calibrate.C,calibrate.D,calibrate.E,calibrate.F, colorChoice,calibrate.volume);
	fclose(fp);
	printf("CalibrateWrite  + save color=%i\n",colorChoice);
}
void  Lcd4D::CalibrateREAD() {
	FILE *fp = fopen("/local/RCCLTS.txt", "r");
	float A,B,C,D,E,F;
	int G, V;
	fscanf(fp,"%f %f %f %f %f %f %i %i\n" ,&A,&B,&C,&D,&E,&F,&G ,&V);
	calibrate.A=A;
	calibrate.B=B;
	calibrate.C=C;
	calibrate.D=D;
	calibrate.E=E;
	calibrate.F=F;

	//extension to other tx parameters
	colorChoice = colorSet(G);
	calibrate.volume = V; // volume 7 ...127 for the 4D
	//printf ("calibrateRead volume=%i\n",calibrate.volume);
	//printf ("calibrateRead colorchoice=%i\n",colorChoice);
	//printf ("***calibrate A=%5.3f B=%5.3f C=%5.3f\n",A,B,C);
	//printf ("***calibrate D=%5.3f E=%5.3f F=%5.3f\n",D,E,F);

	fclose(fp);
}
//-----------sound files update ----------------------------------------------------------------------------
void Lcd4D::soundMove(string suffix) {//move all the .wav  files or other froms the local file system to the 4D sd card
	// search the .w* files in the local file system
	Filey mfile;
	vector<string> dirlist; // directory with the full filename of .WAV files
	dirlist = mfile.fgetlist("",suffix);
	if (dirlist.size()==0) return;
	Cls();
	char font='\x02';
	int font_y=Lcd4D::font_y(font);
	int  n_y= (LCDY)/font_y;
	for (int i =0; i<(int)dirlist.size(); i++) {// copy each file to the sd card
		char x;
		if (!mfile.fopenread(dirlist[i])) //open the input file in the local file system
			continue;
		if (i%n_y==0)
			Cls();
		Textg(dirlist[i],5,font_y*(i%n_y) ,font,RED,1,1);
		int t = GetTicker();
		while (GetTicker()< t+100) {
			rccl_task(1);
		}
		int len = mfile.getlen();
		fopenwrite(dirlist[i]); //open the output file on the sd card - with erase
#ifdef DEBUGP
		printf("soundMove--------------------  move :%s   len=%d\n",dirlist[i].c_str(),len);
#endif
		while (len>0) {
			int trf =100; //transmit 100 bytes maximum
			string t= string();
			while (len >0 && trf >0) {
				mfile.fgetone(x);
				len--;
				trf--;
				t.push_back(x);
			}
			fputline(t);
			rccl_task(1);
		}
		mfile.ftoclose();
		mfile.fremove(dirlist[i].c_str());
	}
}
