#include "core.h"
#include "PowerControl.h"
#include "EthernetPowerControl.h"


extern rccl_data rc; //common data

void InitSave() {
	//write this current model to autostart filename RCCLINIT.TXT
	Filey f;
	if(rc.current_file.size()>4 &&    f.fopenwrite("RCCLINIT.mdp")) {
		char buffer[100];
		sprintf(buffer,"%s    %5.2f    %5.2f \n",rc.current_file.c_str(), rc.batt.cb1,rc.batt.cb2);
		string z ;
		z= buffer;
		f.fputline(z);
		f.ftoclose();
	}
}
void power(int power_on) {
	/* power_on
	 =2 -> immediate check at the startup      (rccl.cpp)
	    =1 -> set the power on during the start   (rccl.cpp)
	    =0 -> power off  asked (cancel with a movement of the sticks)                     (menu.cpp)
	    =3 ->  automatic power off
	watchdog  byte:
	    0 = normal start from power-off status
										1 = restart called by the program (voluntary - to restart the C++ memory state)
												LCD: no restart needed
										2 = restart called after Lcd failure (NAK answer)-restart LCD
	    3 = lock detected by the watchdog

	2011: use only 0 and 3 !!!!!!
	see watchdog.cpp
	see define WDT in core.h
	*/

	if(power_on==1) {  //power on=========================================
		// do not start the program if the only alimentation is the USB cable (to avoid opening files when they can be dmaged)
		AnalogIn ap18(p18); //battery 2 current
		AdcFilter fp18;
		AnalogIn ap17(p17); //battery 1 voltage
		AdcFilter fp17;
		for(int i=0; i< 100; i++) {
			rc.batt.ib1 = 589 * fp17.filter(ap17.read()); //see calchw
			rc.batt.ib2 = 589 *fp18.filter(ap18.read());
		}
#ifdef NOSTART_V
		if(rc.batt.ib1< 50  && rc.batt.ib2< 50) {
			printf("RCCL not started  - no alimentation \n");
			while(1)
				;
		}
#endif
		PHY_PowerDown();
#ifdef WATCHDOG
		Watchdog::setBU(WARMSTART);
#endif

	}
	string prefix = rc.current_file.substr(0,2); //prefix of the model file
	if(prefix.compare("TE")==0    &&  power_on==0) {   //immediate power-off (the normal power-off is not activated for a test model)
		power_on = 3;
	}
	if(power_on==0) {
		rc.calcptr->autocutoff_ticker = GetTicker()  +POWEROFF;
		rc.lcdptr->Cls();
		rc.lcdptr->Textg("Power Off begin",50,80, '\x02',GREEN,3, 3);
		rc.sndptr->putFile("2P");
	}
	if(power_on==3) {  //power off=========================================
		DigitalOut powerpin(p14) ;  //  this pin to  0 cuts the power
		powerpin = 1;
#ifdef DEBUGP
		//printf("Power(0) :  power-off\n");
#endif
		rc.lcdptr->Cls();
		rc.lcdptr->Textg("Power Off ",50,80, '\x02',GREEN,3, 3);
#ifdef WATCHDOG
		Watchdog::setBU(COLDSTART);
		Watchdog::kick(WATCHDOG_STOP);  // a watchdog timeout 60 sec = disable
#endif
		if(rc.calcptr->ready_to_calc) {
			rc.calcptr->param_save(); // save the parameters of the current model
			InitSave();
			rc.lcdptr->Textg("Closed ",50,120, '\x02',GREEN,3, 3);
			int tick = GetTicker();
			while(GetTicker() < tick +100)  //wait to be sure that the buffers are written
				;
		}
		// confirm the power-off
		powerpin =0;//this cut the power
		wait(1);
		// if no USB cable, the micro is without power and everything is stopped
		// if there is a USB cable, the LCD is powered off but  the micro remains powered
		powerpin=1; // to enable a manual power -on of the LCD
		while(1) {  //virtual stop
#ifdef WATCHDOG
			Watchdog::kick(WATCHDOG_STOP);  // a watchdog timeout 60 sec = disable
#endif
		}
	}
	//======================================
	if(power_on==2) { //check immediatly after the start===========================
		rc.startmode =COLDSTART;
#ifdef WATCHDOG
		rc.startmode =  Watchdog::getRSID();
		Watchdog::kick(WATCHDOG_STOP);
#endif
	}
}

