/* General interupt processing
 1)Generate a radio control transmitter ppm signal 
    Functions :
      ppm_init(6 ,20000) initialize for 6 channels, frame 20msec
      ppm_set( x , 1500) set the channel x (from 0 to 5 in this example) to the value 1500&#65533;sec
       (use ppm_set for each channel and frequently enough)
     Test:  a extra pin can provide one pulse/frame, to help synchronize a oscilloscope (not needed with a logic analyzer)
  2) pseudo systick   
*/
//#include "mbed.h"
#include "core.h"
#ifdef DEBUGP
extern Serial pc;
#endif 
#define PPM_PULSE 300 //pulse between channels, in ticks (300&#65533;s normal value)
#define PPM_PIN p30    // mbed pin dedicated to the ppm output signal 
//#define PPM_TEST          // a second pin provides a sync pulse/frame
DigitalOut ppm (PPM_PIN); // output pin for the PPM
#ifdef PPM_TEST
DigitalOut ppm_test (p27); 
#endif
// definitions from the "MODULE" operation
int PPM_ch ; //how many channels  from min 6 to 16
int PPM_frame_pulse; //repeat the PPM frame every x msec
bool PPM_active; // true to output the PPM signal, false to cancel the pulses 
int PPM_servo[PPM_MAX_CH]; // pulse value in microseconds for 6 to 16 channels - if 0 replace with center value
volatile int    PPM_current_ch ; /* current transmitted channel 255 initial*/
volatile int      PPM_output_high ;
volatile int    PPM_length;  // length to the end of the frame
volatile int PPM_pulse; //computed PPM signal - not yet sent to the pin ppm 
Timeout timeout;
void attimeout(); //internal timer interrupt routine
void ppm_init(int ch,int frame_pulse  ){//initialization of the PPM :number of channels and frame length
    PPM_ch = ch;
    if (ch <6 || ch>PPM_MAX_CH)
        PPM_ch=PPM_MAX_CH;
    for(int i=0;i<PPM_ch;i++){
        PPM_servo[i]=1500- PPM_PULSE;
    }
    PPM_frame_pulse = frame_pulse;
    PPM_current_ch=0;
    PPM_output_high =1;
    PPM_length =PPM_frame_pulse ; // length of the frame, not yet transmitted
    PPM_pulse = 1; // pin init value for a negative modulation at the pin
		PPM_active=false; //init without pulse output
    timeout.attach_us(&attimeout, PPM_frame_pulse);
}
void ppm_start(){ //output the pulses, after the start of the calc loop
	PPM_active=true;
}
void ppm_stop(){//cancel the pulses
	PPM_active=false;
	wait (1); //let the fail safe take control
}
void ppm_set(int ch , int value){ //update the servo position channel=ch
    __disable_irq();     // Disable Interrupts  ??? not needed
    if (value < 900 || value > 2200) //incorrect value
        PPM_servo[ch] =1500- PPM_PULSE;
    else
        PPM_servo[ch]=value - PPM_PULSE;
    __enable_irq();     // Enable Interrupts
}
void attimeout() {  //internal "timer interrupt" processing
     if( PPM_output_high ==1){ //start  fixed pulse
         timeout.attach_us(&attimeout, PPM_PULSE);
        PPM_length -=PPM_PULSE;
        PPM_output_high=0;//for the next interrupt
        PPM_pulse=!PPM_pulse;
        #ifdef PPM_TEST
            ppm_test=0;
        #endif
    }
    else {
        if (PPM_current_ch < PPM_ch){  
            timeout.attach_us(&attimeout, PPM_servo[PPM_current_ch]);
            PPM_length -=PPM_servo[PPM_current_ch];
            PPM_output_high=1;
            PPM_current_ch++;//0 to  PPM_ch -1
            PPM_pulse=!PPM_pulse;
        }
        else { //sync pulse
            timeout.attach_us(&attimeout, PPM_length);
            PPM_length  =PPM_frame_pulse;
            PPM_current_ch=0;
            PPM_output_high =1;
            PPM_pulse=!PPM_pulse;    
            #ifdef PPM_TEST
                ppm_test=1;
            #endif    
        }
    }
		if (PPM_active)
			ppm = PPM_pulse;
		else
			ppm = 0;
}
//---------Ticker system : a kind of systick ----------------------------------
Ticker ownSysTicker;
int sysTicker;
void ownSysTickerAdd(){ //interrupt every 10 msec
    sysTicker++;
}
void startOwnSysTicker(){
    //start the sysTicker clock
    sysTicker=0;
    ownSysTicker.attach(&ownSysTickerAdd, TICKER/1000.0);// TICKER =10 -> systicker unit = 10msec
}
int GetTicker(){
    return  sysTicker;
}
//---------busy statistic----------------------------------------------



void printTime(string text ,int t1){//print a time interval
    //t1=GetTicker avant
    #ifdef DEBUGP
    pc.printf (" --- %s  %i msec\n", text.c_str(), ((GetTicker() -t1) * TICKER));
    #endif
}


