/*============================================================================
 Copyright (C) 2008 -2012 Robert Spilleboudt

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
============================================================================*/





#include "core/core.h"
struct rccl_data rc; //common data  
#ifdef DEBUGP
Serial pc(USBTX, USBRX);//needed only for the pc.printf
#endif
//-------- control data for the scheduling of tasks
bool task_rc_busy ;
bool task_readpin_busy ;
int    task_readpin_tick ;
void rccl_init() {//----------------initialization---------------------
	power(2); //set the rc.startmode
#ifdef DEBUGP
	pc.baud(115200); //init the pc  connection to the terminal, for debugging
	pc.printf("--------------rcclmain Start RCCL   Available memory :  %d Startmode=%c\n",FreeMem(),rc.startmode);
#endif
	startOwnSysTicker();
	rc.current_file=""; //current filename
	rc.calcptr = NULL;
#ifdef JETI
	rc.jetiptr  = NULL;
#endif
	rc.lcdstate = LCD_NO_READY; //not started
	rc.rccl_task_tick=0;
	task_readpin_tick=0;
	task_rc_busy=false;
	task_readpin_busy=false;
	power(1);

	//  special transmitter start without loading a model -> recovery with the editor
	DigitalIn dBA(p13); // the BA button  false = pushed => do not load the model
	dBA.mode(PullUp);
	Filey f; //...defined within this block
	if(f.fopenread("RCCLINIT.mdp")) {    //autostart the last model
		string str;
		char buffer[100];
		char model[10];
		f.fgetline(str);
		strcpy(buffer, str.c_str());
		sscanf(buffer,"%s  %f  %f",model, &rc.batt.cb1, &rc.batt.cb2);
		rc.current_file = model;
		f.ftoclose();
		if (!f.fopenread(rc.current_file+".mdx")) {
			rc.current_file =""; //error -> model not defined
		}



#ifdef JETI
		rc.jetiptr=new(nothrow) Jeti();
#endif
		if(dBA) {
			rc.calcptr= new(nothrow) Calc(rc.current_file);
			rc.calcptr->setVarL("@H1",1);

#ifdef WATCHDOG
			if(rc.startmode== WARMSTART) {
				rc.calcptr->setVarL("@H1",0);
#ifdef DEBUGP
				printf("Rccl Init :WARMSTART\n");
#endif

			}
#endif
		}
		rc.sndptr = new(nothrow) Snd();
		//rc.calcptr->varDump(1);//0 = only var 1=full +++++++++++++++ debug
	}
	{
		//init
		//pc.printf ("rcclmain reload:%s Time= %d ms \n",rc.current_file.c_str(),10*GetTicker() );
		rc.lcdptr= new(nothrow) Lcd4D();
		rc.lcdptr->TextOpaque(false);
		rc.lcdptr->ChangeColor();
		rc.varSelection=NULL;
		if(rc.calcptr!=NULL) {
			rc.calcptr->lcd[6]=	rc.current_file;
			rc.calcptr->lcd_protect[6]=GetTicker()+500;
			//pc.printf ("rcclmain Available memory  : %d   startmode %c\n", FreeMem(),rc.startmode);
		}
	}
	// message model file name .WAV
	rc.sndptr->putFile(rc.current_file);

	rc.lcdstate=LCD_READY; //full initialization
	if(!dBA) { //recovery start when BA is active
		printf("Rccl recovery start\n");
		Ed ed(rc.current_file+".mdx",false);
		ed.display();
		printf("Rccl recovery end\n");
	}
	rc.lcdptr->fopenwrite("LOG.TEXT");
#ifdef DEBUGP
	printf("Rccl Init :LCD_READY\n");
	//rc.lcdptr->PinDump();
#endif

#ifdef WATCHDOG
	Watchdog::kick(WATCHDOG_TIMER); //watchdog  reload after ... seconds
#endif
  // rc.lcdptr->Calibrate(); //activate for urgency 

}
void rccl_task(int cause) {
	/*real time task scheduler -called from each "wait"
		 "cause"  avoid a reentrant call to a waiting program
	    cause = 0 : waiting for a uart/Lcd4D
	    cause = 1 : waiting for a human reaction (touchscreen) or in a long program loop
		cause = 2 : waiting for a Jeti answer  */
	int tick = GetTicker(); //tick each 10 msec
#ifdef WATCHDOG
	Watchdog::kick();
#endif
	if(cause !=0 && rc.lcdstate ==LCD_READY) {  //------------- process the sound queue --
		/*		-call of the task-switcher not by lcd wait  
		   			-lcd fully available (sound object constructed)
		   			-immediatly process (highest priority - no wait)
		*/
		rc.sndptr->getNext(tick);
		tick = GetTicker(); //????
	}
	if(cause !=0 && rc.lcdstate ==LCD_READY && rc.calcptr!= NULL){//logger
		rc.calcptr->logger_sd();
	}
	if((rc.calcptr!= NULL) &&rc.calcptr->ready_to_calc && rc.lcdstate ==LCD_READY && cause !=0) {//-----------detect inactivity-------------------------------------------------------------------------
		int inact = rc.calcptr->autocutoff_ticker - tick; //time to go to the automatic power-off
		if(inact < 1500) {  //<15"
			rc.calcptr->lcd[7] = "Power-off";
			//!!! silent ??
		}
		if(inact<0)
			power(3); //automatic power off
	}

	if(tick > task_readpin_tick  && !task_readpin_busy && cause !=0&& (rc.lcdstate ==LCD_READY)) {   //read the digital input of the Lcd 4D and store to registers RTC and array Pin
		/*---------------- process the digital in from the LCD -------------------------------
		-every 10 msec  (task_readpin_tick )
		-not reentrant (task_readpin_busy)
		-not called when there is a wait for the lcd (other reentrancy) : cause != 0
		-the lcd is ready for digital IO
		*/
		task_readpin_busy=true; // ??? not interrupted in PinRead
		task_readpin_tick = tick;
		rc.lcdptr->PinRead();//physical read of the 16 GPIO - save to array PIN and register RPC 1
		task_readpin_busy=false;
		tick = GetTicker();
	}

	/*-------------- process the rc loop -------------------------------------------------*/
	if(tick >rc.rccl_task_tick 
			&& !task_rc_busy
			&& !task_readpin_busy
			&&(rc.calcptr!= NULL)
			&& (rc.sndptr!=NULL)
			&&rc.calcptr->ready_to_calc
			&& (rc.lcdstate == LCD_PIN_RESTORED ||rc.lcdstate ==LCD_READY)) {
		/*-------------- process the rc loop -------------------------------------------------
		-every 10 msec  (rc.rccl_task_tick)
		-not reentrant (task_rc_busy)
		-the calc object is ready (rc.calcptr!= NULL) &&rc.calcptr->ready_to_calc)
		- any call of the task-switcher
		- for a normal start, the lcd is fully initialized
		*/
		task_rc_busy=true;
		rc.jetiptr->explorer_connect();
		rc.rccl_task_tick=tick;
		rc.calcptr->hwread();//read all the input from the microprocessor-> variables
		rc.calcptr->run(); // compute the rc model
		rc.calcptr->hwout();
		//wait(2); pc.printf("rccl end loop\n");
		task_rc_busy= false;
		tick = GetTicker();
	}
}
int main() { 
	rccl_init();
	//rccl_test(); // -> core/test.cpp only for a test
	rccl_dialog();// -> core/menu.cpp the human interface dialogs with lcd+ touchscreen
}